C
C=======================================================================
C=======================================================================
C
      SUBROUTINE CKLMTS(P, D, T)
C
C  PURPOSE:
C     Check to see if the density and temperature are within the limits of
C     the EOS.  The pressure, density and temperature bounds are checked.
C     The solid line is calculated to make sure D and T are not in the solid
C     region.  If the inputs are not within the limits, they will be set
C     to zero.
C
C  DESCRIPTION OF ARGUMENT:
C     P       Pressure, MPa    (Input, will be calculated if zero)
C     D       Density, mol/dm3 (Input)
C     T       Temperature, K   (Input)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      REAL*8 P, D, T, PS, DL, DV, PL, PV
      REAL*8 P2PEQ, PFNDA, D2PEQ
      IF (T.EQ.0) GOTO 100
      IF (D.EQ.0) GOTO 100
      IF (P.EQ.0) P = PFNDA(D, T)
      IF (P.GT.PHIGH(NDX) .OR. P.LT.PLOW(NDX)) GOTO 100
      IF (T.GT.THIGH(NDX) .OR. T.LT.TLOW(NDX)) GOTO 100
      IF (D.GT.DHIGH(NDX) .OR. D.LT.DLOW(NDX)) GOTO 100
      PS = P2PEQ(T, 3)
      IF (P.GT.PS) GOTO 100
      IF (T.LT.TDH1(NDX)) THEN
        CALL SATT(T, 0, DL, DV, PL, PV)
        IF (D.GT.DV .AND. D.LT.DL) GOTO 100
      ELSEIF (T.LT.TC(NDX) .AND. P.LT.PC(NDX)) THEN
        IF (D.LT.D2PEQ(T, 1)*0.95D0 .AND. D.GT.D2PEQ(T, 2)*1.05D0) THEN
          CALL SATT(T, 0, DL, DV, PL, PV)
          IF (D.GT.DV .AND. D.LT.DL) GOTO 100
        ENDIF
      ENDIF
      RETURN
 100  P = 0
      T = 0
      D = 0
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE FACT
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C     Units selection routine for cats thermodynamic property
C     formulations.
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comdrv.for'

C--Local Variables----------------------
      REAL*8 CP0, CV0, IDGAS
      LOGICAL TEMP

C-----------------------------------------------------------------------
      REAL*8 RtoK, FtoR, ATMtoMPA, BARtoMPA, MMHGtoMPA, PSIAtoMPA
      REAL*8 KGFtoN, LBMtoKG, LBFtoN
      REAL*8 INtoM, FTtoM, FT3toM3, HRtoS
      REAL*8 BTUtoKJ, BTUtoW, CALtoJ
C     CtoK = 273.15D0
      FtoR = 459.67D0
      RtoK = 5.D0/9.D0
      ATMtoMPa   = 0.101325D0           !Exact conversion
      BARtoMPA   = 0.1D0                !Exact conversion
      MMHGtoMPA  = 0.0001333224D0
      KGFtoN     = 9.80665D0            !Exact conversion
      LBMtoKG    = 0.45359237D0         !Exact conversion
      INtoM      = 0.0254D0             !Exact conversion
      FTtoM      = 12.D0*INtoM          !Exact conversion
      FT3toM3    = FTtoM**3
      HRtoS      = 3600                 !Exact conversion
      BTUtoKJ    = 1.05506D0
      BTUtoW     = BTUtoKJ*1000.D0
      CALtoJ     = 4.184D0              !Exact conversion
      LBFtoN     = LBMtoKG*KGFtoN
      PSIAtoMPA  = LBMtoKG/INtoM**2*KGFtoN/1000000.D0

C-----------------------------------------------------------------------
      IF (ALPHT.EQ.'K') THEN
        TFACT = 1
        CFACT = 0
      ELSEIF (ALPHT.EQ.'C') THEN
        TFACT = 1
C...All the extra zeros are necessary for the C to K conversion of water
C.....at 0.01C.
        CFACT = 273.1500000000001D0
      ELSEIF (ALPHT.EQ.'R') THEN
        TFACT = RtoK
        CFACT = 0
      ELSEIF (ALPHT.EQ.'F') THEN
        TFACT = RtoK
        CFACT = FtoR
      ELSEIF (ALPHT.EQ.'T/Tc') THEN
        TFACT = TC(NDX)
        CFACT = 0
      ELSE
        WRITE (*,*) 'Temperature units do not exist.'
        CALL PAUSE2
      ENDIF

      IF (ALPHP.EQ.'MPa') THEN
        PFACT = 1
        GFACT = 0
      ELSEIF (ALPHP.EQ.'psia') THEN
        PFACT = PSIAtoMPA
        GFACT = 0
      ELSEIF (ALPHP.EQ.'P/Pc') THEN
        PFACT = PC(NDX)
        GFACT = 0
      ELSEIF (ALPHP.EQ.'bar') THEN
        PFACT = BARtoMPA
        GFACT = 0
      ELSEIF (ALPHP.EQ.'mmHg') THEN
        PFACT = MMHGtoMPA
        GFACT = 0
      ELSEIF (ALPHP.EQ.'atm') THEN
        PFACT = ATMtoMPA
        GFACT = 0
      ELSEIF (ALPHP.EQ.'MPa gage') THEN
        PFACT = 1
        GFACT = ATMtoMPA
      ELSEIF (ALPHP.EQ.'psig') THEN
        PFACT = PSIAtoMPA
        GFACT = ATMtoMPA/PSIAtoMPA
      ELSEIF (ALPHP.EQ.'bar gage') THEN
        PFACT = BARtoMPA
        GFACT = ATMtoMPA/BARtoMPA
      ELSEIF (ALPHP.EQ.'mmHg gage') THEN
        PFACT = MMHGtoMPA
        GFACT = ATMtoMPA/MMHGtoMPA
      ELSEIF (ALPHP.EQ.'atm gage') THEN
        PFACT = ATMtoMPA
        GFACT = 1
      ELSE
        WRITE (*,*) 'Pressure units do not exist.'
        CALL PAUSE2
      ENDIF

      IF (ALPHD.EQ.'mol/dm3') THEN
        DFACT = 1.D0
      ELSEIF (ALPHD.EQ.'kg/m3') THEN
        DFACT = 1.D0/AMOL(NDX)
      ELSEIF (ALPHD.EQ.'lbm/ft3') THEN
        DFACT = LBMtoKG/FT3toM3/AMOL(NDX)
      ELSEIF (ALPHD.EQ.'D/Dc') THEN
        DFACT = DC(NDX)
      ELSE
        WRITE (*,*) 'Density units do not exist.'
        CALL PAUSE2
      ENDIF


      IF (ALPHV.EQ.'dm3/mol') THEN
        VFACT = 1
      ELSEIF (ALPHV.EQ.'m3/kg') THEN
        VFACT = AMOL(NDX)
      ELSEIF (ALPHV.EQ.'ft3/lbm') THEN
        VFACT = AMOL(NDX)/(LBMtoKG/FT3toM3)
      ELSEIF (ALPHV.EQ.'V/Vc') THEN
        VFACT = 1.D0/DC(NDX)
      ELSE
        WRITE (*,*) 'Volume units do not exist.'
        CALL PAUSE2
      ENDIF

      IF (ALPHH.EQ.'J/mol') THEN
        HFACT = 1
      ELSEIF (ALPHH.EQ.'kJ/kg') THEN
        HFACT = AMOL(NDX)
      ELSEIF (ALPHH.EQ.'Btu/lbm') THEN
        HFACT = BTUtoKJ/LBMtoKG*AMOL(NDX)
      ELSEIF (ALPHH.EQ.'/RTc') THEN
        HFACT = RGAS(NDX)*1000.D0*TC(NDX)
      ELSEIF (ALPHH.EQ.'kcal/kg') THEN
        HFACT = CALtoJ*AMOL(NDX)
      ELSE
        WRITE (*,*) 'Enthalpy units do not exist.'
        CALL PAUSE2
      ENDIF


      IF (ALPHS.EQ.'J/mol-K') THEN
        SFACT = 1
      ELSEIF (ALPHS.EQ.'kJ/kg-K') THEN
        SFACT = AMOL(NDX)
      ELSEIF (ALPHS.EQ.'Btu/lbm-R') THEN
        SFACT = BTUtoKJ/LBMtoKG/RtoK*AMOL(NDX)
      ELSEIF (ALPHS.EQ.'/R') THEN
        SFACT = RGAS(NDX)*1000.D0
      ELSEIF (ALPHS.EQ.'kcal/kg-K') THEN
        SFACT = CALtoJ*AMOL(NDX)
      ELSE
        WRITE (*,*) 'Entropy units do not exist.'
        CALL PAUSE2
      ENDIF

      IF (ALPHW.EQ.'m/s') THEN
        WFACT = 1
      ELSEIF (ALPHW.EQ.'ft/s') THEN
        WFACT = FTtoM
      ELSEIF (ALPHW.EQ.'---') THEN
        TEMP = IDGPRP
        IDGPRP = .TRUE.
        CP0 = IDGAS(TC(NDX), 0)
        CV0 = CP0 - 1
        WFACT = SQRT(RGAS(NDX)/AMOL(NDX)*TC(NDX)*1000000.D0*CP0/CV0)
        IDGPRP = TEMP
      ELSE
        WRITE (*,*) 'Speed units do not exist.'
        CALL PAUSE2
      ENDIF

      IF (ALPHVIS.EQ.'Pa-s' .OR. ALPHVIS.EQ.'kg/m-s') THEN
        VISFACT = 1
      ELSEIF (ALPHVIS.EQ.'g/cm-s' .OR. ALPHVIS.EQ.'poise') THEN
        VISFACT = 0.1D0
      ELSEIF (ALPHVIS.EQ.'c-poise') THEN
        VISFACT = 1.D0/1000.D0
      ELSEIF (ALPHVIS.EQ.'u-poise') THEN
        VISFACT = 1.D0/10000000.D0
      ELSEIF (ALPHVIS.EQ.'lbm/ft-s') THEN
        VISFACT = LBMtoKG/FTtoM
      ELSEIF (ALPHVIS.EQ.'lbf-s/ft2') THEN
        VISFACT = LBFtoN/FTtoM**2
      ELSEIF (ALPHVIS.EQ.'lbm/ft-hr') THEN
        VISFACT = LBMtoKG/FTtoM/HRtoS
      ELSE
        WRITE (*,*) 'Viscosity units do not exist.'
        CALL PAUSE2
      ENDIF

      IF (ALPHK.EQ.'mW/m-K') THEN
        THCNDFACT = 1
      ELSEIF (ALPHK.EQ.'W/m-K') THEN
        THCNDFACT = 1000
      ELSEIF (ALPHK.EQ.'g-cm/s3-K') THEN
        THCNDFACT = 1.D0/100.D0
      ELSEIF (ALPHK.EQ.'kg-m/s3-K') THEN
        THCNDFACT = 1000
      ELSEIF (ALPHK.EQ.'cal/s-cm-K') THEN
        THCNDFACT = CALtoJ*100000.D0
      ELSEIF (ALPHK.EQ.'lbm-ft/s3-F') THEN
        THCNDFACT = 1000.D0*LBMtoKG*FTtoM/RtoK
      ELSEIF (ALPHK.EQ.'lbf/s-F') THEN
        THCNDFACT = 1000.D0*LBFtoN/RtoK
      ELSEIF (ALPHK.EQ.'Btu/hr-ftF') THEN
        THCNDFACT = 1000.D0*BTUtoW/HRtoS/FTtoM/RtoK
      ELSE
        WRITE (*,*) 'Thermal conductivity units do not exist.'
        CALL PAUSE2
      END IF

      IF (ALPHST.EQ.'N/m') THEN
        STFACT = 1.D0
      ELSEIF (ALPHST.EQ.'dyne/cm') THEN
        STFACT = 1/1000.D0
      ELSEIF (ALPHST.EQ.'lbf/ft') THEN
        STFACT = LBFtoN/FTtoM
      ELSE
        WRITE (*,*) 'Surface Tension units do not exist.'
        CALL PAUSE2
      END IF
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE INPUT1(X, PROP, ERRORX)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comdrv.for'
      REAL*8 X0OLD, X1OLD, X2OLD, XINCOLD
      COMMON /INPUTX/ X0OLD, X1OLD, X2OLD, XINCOLD

C--Arguments----------------------------
      INTEGER PROP
      REAL*8 X
      LOGICAL ERRORX
      CHARACTER*80 LINEX

C-----------------------------------------------------------------------
      ERRORX = .FALSE.
      IF (PROP.EQ.1 .AND. PRNTFLG) WRITE (*,1000) ALPHP
      IF (PROP.EQ.2 .AND. PRNTFLG) WRITE (*,1010) ALPHT
      IF (PROP.EQ.3 .AND. PRNTFLG) WRITE (*,1020) ALPHD
      READ (*,'(A80)',ERR=9000) LINEX

C.....Use previous inputs
      IF (LINEX(1:1).EQ.'R' .OR. LINEX(1:1).EQ.'r') THEN
        X = X0OLD
      ELSEIF (LINEX(1:1).EQ.'H' .OR. LINEX(1:1).EQ.'h') THEN
        IF (PROP.EQ.1) X = PHIGH(NDX)/PFACT - GFACT
        IF (PROP.EQ.2) X = THIGH(NDX)/TFACT - CFACT
        IF (PROP.EQ.3) X = DHIGH(NDX)/DFACT
      ELSEIF (LINEX(1:1).EQ.'L' .OR. LINEX(1:1).EQ.'l') THEN
        IF (PROP.EQ.1) X = PLOW(NDX)/PFACT - GFACT
        IF (PROP.EQ.2) X = TLOW(NDX)/TFACT - CFACT
        IF (PROP.EQ.3) X = DLOW(NDX)/DFACT
      ELSE
        READ (LINEX,*,ERR=9000) X
        X0OLD = X
      ENDIF

      IF (PROP.EQ.1) THEN
        X = (X+GFACT)*PFACT
        IF (X.LT.PLOW(NDX) .OR. X.GT.PHIGH(NDX)) THEN
          WRITE (*,*) ' *** PRESSURE OUT OF BOUNDS ***'
          CALL PAUSE2
          GOTO 9000
        ENDIF
      ELSEIF (PROP.EQ.2) THEN
        X = (X+CFACT)*TFACT
        IF (X.LT.TLOW(NDX) .OR. X.GT.THIGH(NDX)) THEN
          WRITE (*,*) ' *** TEMPERATURE OUT OF BOUNDS ***'
          CALL PAUSE2
          GOTO 9000
        ENDIF
      ELSEIF (PROP.EQ.3) THEN
        X = X*DFACT
        IF (X.LT.DLOW(NDX) .OR. X.GT.DHIGH(NDX)) THEN
          WRITE (*,*) ' *** DENSITY OUT OF BOUNDS ***'
          CALL PAUSE2
          GOTO 9000
        ENDIF
      ENDIF
      RETURN
 9000 ERRORX = .TRUE.
 1000 FORMAT ('+ Input Pressure, ',A10,'    ====>  ')
 1010 FORMAT ('+ Input Temperature, ',A10,' ====>  ')
 1020 FORMAT ('+ Input Density, ',A10,'     ====>  ')
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE INPUT2(X1, X2, XINC, PROP, ERRORX, MULT)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Routine to enter the starting point, ending point, and increment.
C    Also prints out a header.
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comdrv.for'
      REAL*8 X0OLD, X1OLD, X2OLD, XINCOLD
      COMMON /INPUTX/ X0OLD, X1OLD, X2OLD, XINCOLD

C--Local Variables----------------------
      CHARACTER*80 LINEX
      INTEGER PROP
      REAL*8 X, X1, X2, XINC, HIGH, LOW
      LOGICAL ERRORX, MULT

C-----------------------------------------------------------------------
      ERRORX = .FALSE.
      IF (PROP.EQ.1 .AND. PRNTFLG) WRITE (*,1010) ALPHP
      IF (PROP.EQ.2 .AND. PRNTFLG) WRITE (*,1000) ALPHT
      READ (*,'(A80)',ERR=9000) LINEX

      X1 = -10000
      X2 = 10000
      IF (MULT .AND. PROP.EQ.1) X1 = .001

C.....Use previous inputs
      IF (LINEX(1:1).EQ.'R' .OR. LINEX(1:1).EQ.'r') THEN
        X1 = X1OLD
        X2 = X2OLD
        XINC = XINCOLD

      ELSEIF (LINEX(1:1).EQ.'Q' .OR. LINEX(1:1).EQ.'q') THEN
        XINC = 1
        IF (MULT .AND. PROP.EQ.1) XINC = 1.1
      ELSEIF (LINEX(1:1).EQ.'W' .OR. LINEX(1:1).EQ.'w') THEN
        XINC = 10
        IF (MULT .AND. PROP.EQ.1) XINC = 1.2
      ELSEIF (LINEX(1:1).EQ.'E' .OR. LINEX(1:1).EQ.'e') THEN
        XINC = 100
        IF (MULT .AND. PROP.EQ.1) XINC = 1.5
      ELSE

C  .....Check to see if a letter was entered.
        READ (LINEX,*,ERR=9000) X1

C  .....Read in new values
        READ (LINEX,*,ERR=300,END=300) X1, X2, XINC
        GOTO 310
 300    READ (*,*,ERR=9000) X2, XINC
 310    X1OLD = X1
        X2OLD = X2
        XINCOLD = XINC
      ENDIF
      XINC = DABS(XINC)
      IF (PROP.EQ.1) THEN
        X1 = (X1+GFACT)*PFACT
        X2 = (X2+GFACT)*PFACT
        IF (.NOT.MULT) XINC = DABS(XINC*PFACT)
        IF (XINC.EQ.1.D0 .AND. MULT) XINC = 1.1D0
        HIGH = PHIGH(NDX)
        LOW = PLOW(NDX)
      ELSEIF (PROP.EQ.2) THEN
        X1 = (X1+CFACT)*TFACT
        X2 = (X2+CFACT)*TFACT
        XINC = DABS(XINC*TFACT)
        HIGH = THIGH(NDX)
        LOW = TLOW(NDX)
      ENDIF

      IF (XINC.EQ.0.D0) XINC = 10000.D0
      IF (X1.GT.X2) THEN
        X = X1
        X1 = X2
        X2 = X
      ENDIF

      IF (X1.GT.HIGH) THEN
        X1 = LOW
        XINC = 10000.D0
      ENDIF
      IF (X2.LT.LOW) THEN
        X2 = HIGH
        XINC = 10000.D0
      ENDIF
      IF (X2.GT.HIGH) X2 = HIGH
      IF (X1.LT.0) X1 = 0

      CALL SYSTEM("CLS")
      IF (IDGPRP) THEN
        IF (PRNTFLG) WRITE (*,IDGTITLE) TNAMEQ(NDX)
        IF (TITLEQ .AND. PRNTFLG) WRITE (IOUT,IDGTITLE) TNAMEQ(NDX)
      ELSE
        IF (PRNTFLG) WRITE (*,TITLE) TNAMEQ(NDX)
        IF (TITLEQ .AND. PRNTFLG) WRITE (IOUT,TITLE) TNAMEQ(NDX)
      ENDIF
      IF (PRNTFLG) WRITE (*,TITLE1) STRING1, STRING2
      IF (TITLEQ .AND. PRNTFLG) WRITE (IOUT,TITLE2) STRING1, STRING2
      RETURN

 9000 ERRORX = .TRUE.
 1000 FORMAT ('  Input Temperature Information in ', A10,/,
     &'  (1)  T1   (Starting Temperature)',/,
     &'  (2)  T2   (Ending Temperature)',/,
     &'  (3)  TINC (Temperature Increment)',//)
 1010 FORMAT ('  Input Pressure Information in ', A10,/,
     &'  (1)  P1   (Starting Pressure)',/,
     &'  (2)  P2   (Ending Pressure)',/,
     &'  (3)  PINC (Pressure Increment)',//)
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE OUTX(OPT)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C     Allows user to select a different set of output properties.
C     Also concatenates together the strings forming the titles.
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comdrv.for'

C--Local Variables----------------------
      CHARACTER*10 STR(NV), STRA(NV)
      CHARACTER*1 VAR1(NV)
      CHARACTER OP*3, CHR*3
      INTEGER OPT, I, J, SI, LENGTH

C--If OPT=0 the Selection Routine is Run--------------------------------
      IF (OPT.EQ.0) THEN
        DO 10 I = 1, NS
 10     STATES(I) = ' '
 400    DO 20 I = 1, NV
          VAR1(I) = ' '
          IF (VAR(I).EQ.1) VAR1(I) = '*'
 20     CONTINUE
        CALL SYSTEM("CLS")
        WRITE (*,1000) VAR1( 1), VAR1(12), VAR1( 2), VAR1(13), VAR1( 3),
     &       VAR1(14), VAR1( 4), VAR1(15), VAR1( 5), VAR1(16), VAR1( 6),
     &       VAR1(17), VAR1( 7), VAR1(18), VAR1( 8), VAR1(19), VAR1( 9),
     &       VAR1(20), VAR1(10), VAR1(21), VAR1(11), VAR1(22),
     &       VAR1(23), VAR1(27), VAR1(24), VAR1(28),
     &       VAR1(25), VAR1(29), VAR1(26), VAR1(30)
        READ (*,'(A3)') OP
        IF (OP.EQ.'1' .OR. OP.EQ.'P' .OR. OP.EQ.'p') VAR(1)=1-VAR(1)
        IF (OP.EQ.'2' .OR. OP.EQ.'T' .OR. OP.EQ.'t') VAR(2)=1-VAR(2)
        IF (OP.EQ.'3' .OR. OP.EQ.'D' .OR. OP.EQ.'d') VAR(3)=1-VAR(3)
        IF (OP.EQ.'4' .OR. OP.EQ.'V' .OR. OP.EQ.'v') VAR(4)=1-VAR(4)
        IF (OP.EQ.'5' .OR. OP.EQ.'U' .OR. OP.EQ.'u') VAR(5)=1-VAR(5)
        IF (OP.EQ.'6' .OR. OP.EQ.'H' .OR. OP.EQ.'h') VAR(6)=1-VAR(6)
        IF (OP.EQ.'7' .OR. OP.EQ.'S' .OR. OP.EQ.'s') VAR(7)=1-VAR(7)
        IF (OP.EQ.'8' .OR. OP.EQ.'CV' .OR. OP.EQ.'cv') VAR(8)=1-VAR(8)
        IF (OP.EQ.'9' .OR. OP.EQ.'CP' .OR. OP.EQ.'cp') VAR(9)=1-VAR(9)
        IF (OP.EQ.'10' .OR. OP.EQ.'W' .OR. OP.EQ.'w') VAR(10)=1-VAR(10)
        IF (OP.EQ.'11' .OR. OP.EQ.'Z' .OR. OP.EQ.'z') VAR(11)=1-VAR(11)
        IF (OP.EQ.'12' .OR. OP.EQ.'A' .OR. OP.EQ.'a') VAR(12)=1-VAR(12)
        IF (OP.EQ.'13' .OR. OP.EQ.'G' .OR. OP.EQ.'g') VAR(13)=1-VAR(13)
        IF (OP.EQ.'14' .OR. OP.EQ.'B' .OR. OP.EQ.'b') VAR(14)=1-VAR(14)
        IF (OP.EQ.'15' .OR. OP.EQ.'C' .OR. OP.EQ.'c') VAR(15)=1-VAR(15)
        IF (OP.EQ.'16' .OR. OP.EQ.'F' .OR. OP.EQ.'f') VAR(16)=1-VAR(16)
        IF (OP.EQ.'17' .OR. OP.EQ.'PT'.OR. OP.EQ.'pt')VAR(17)=1-VAR(17)
        IF (OP.EQ.'18' .OR. OP.EQ.'PD1'.OR.OP.EQ.'pd1')VAR(18)=1-VAR(18)
        IF (OP.EQ.'19' .OR. OP.EQ.'PD2'.OR.OP.EQ.'pd2')VAR(19)=1-VAR(19)
        IF (OP.EQ.'20' .OR. OP.EQ.'VIS'.OR.OP.EQ.'vis')VAR(20)=1-VAR(20)
        IF (OP.EQ.'21' .OR. OP.EQ.'THC'.OR.OP.EQ.'thc')VAR(21)=1-VAR(21)
        IF (OP.EQ.'22' .OR. OP.EQ.'ST'.OR. OP.EQ.'st')VAR(22)=1-VAR(22)
        IF (OP.EQ.'23') VAR(23)=1-VAR(23)
        IF (OP.EQ.'24') VAR(24)=1-VAR(24)
        IF (OP.EQ.'25') VAR(25)=1-VAR(25)
        IF (OP.EQ.'26') VAR(26)=1-VAR(26)
        IF (OP.EQ.'27') VAR(27)=1-VAR(27)
        IF (OP.EQ.'28') VAR(28)=1-VAR(28)
        IF (OP.EQ.'29') VAR(29)=1-VAR(29)
        IF (OP.EQ.'30') VAR(30)=1-VAR(30)
        IF (OP.EQ.'-1') THEN
          DO 30 I=1, NV
 30       VAR(I) = 0
        ENDIF
        IF (OP.EQ.'0' .OR. OP.EQ.'R' .OR. OP.EQ.'r') GOTO 410
        GOTO 400
      ENDIF

 410  J = 1
      DO 40 I=1, NV
 40   IF (VAR(I).EQ.1) J = J + 1
      WRITE(CHR,'(I3)') J * 10 - 1
      TITLE2 = '(1X,'//CHR//'(''=''),/,2(A'//CHR//
     &         ',/),1X,'//CHR//'(''=''))'
      ALNGTH = '(A'//CHR//')'

      STR(1) = 'P'
      STRA(1) = ALPHP
      STR(2) = 'T'
      STRA(2) = ALPHT
      STR(3) = 'D'
      STRA(3) = ALPHD
      STR(4) = 'V'
      STRA(4) = ALPHV
      STR(5) = 'U'
      STRA(5) = ALPHH
      STR(6) = 'H'
      STRA(6) = ALPHH
      STR(7) = 'S'
      STRA(7) = ALPHS
      STR(8) = 'Cv'
      STRA(8) = ALPHS
      STR(9) = 'Cp'
      STRA(9) = ALPHS
      STR(10) = 'W'
      STRA(10) = ALPHW
      STR(11) = 'Z'
      STRA(11) = '---'
      STR(12) = 'A'
      STRA(12) = ALPHH
      STR(13) = 'G'
      STRA(13) = ALPHH
      STR(14) = 'B'
      STRA(14) = ALPHB
      STR(15) = 'C'
      STRA(15) = ALPHC
      STR(16) = 'F'
      STRA(16) = '---'
      STR(17) = 'dP/dT'
      STRA(17) = ALPHP(1:LENGTH(ALPHP))//'/'//ALPHT
      STR(18) = 'dP/dD'
      STRA(18) = ' '
      STR(19) = 'd2P/dD2'
      STRA(19) = ' '
      STR(20) = 'VIS'
      STRA(20) = ALPHVIS
      STR(21) = 'THCND'
      STRA(21) = ALPHK
      STR(22) = 'ST'
      STRA(22) = ALPHST
      STR(23) = 'JT'
      STRA(23) = ALPHT(1:LENGTH(ALPHT))//'/'//ALPHP
      STR(24) = 'k'
      STRA(24) = '---'
      STR(25) = 'kT'
      STRA(25) = '---'
      STR(26) = 'Beta'
      STRA(26) = '1/'//ALPHT
      STR(27) = 'BetaS'
      STRA(27) = '1/'//ALPHP
      STR(28) = 'BulkModS'
      STRA(28) = ALPHP
      STR(29) = 'Kappa'
      STRA(29) = '1/'//ALPHP
      STR(30) = 'BulkModT'
      STRA(30) = ALPHP


      STRING1 = '        '
      STRING2 = '        '
      SI = 10
      DO 50 I=1, NV
        IF (VAR(I).EQ.1) THEN
          J = LENGTH(STR(I))
          IF (J.LE.5) STR(I)=' '//STR(I)
          IF (J.LE.3) STR(I)=' '//STR(I)
          IF (J.LE.1) STR(I)=' '//STR(I)
          STRING1 = STRING1(1:SI)//STR(I)
          J = LENGTH(STRA(I))
          IF (J.LE.5) STRA(I)=' '//STRA(I)
          IF (J.LE.3) STRA(I)=' '//STRA(I)
          IF (J.LE.1) STRA(I)=' '//STRA(I)
          STRING2 = STRING2(1:SI)//STRA(I)
          SI = SI + 10
        ENDIF
 50   CONTINUE


 1000 FORMAT (
     &  '  OUTPUT MENU (To see eight or more marked properties, ',
     &  'use an output file)',//,
     &  2X,A1,'1.  Pressure (P)',T40,A1,
     &        '12. Helmholtz Energy (A)'/,
     &  2X,A1,'2.  Temperature (T)',T40,A1,
     &        '13. Gibbs Free Energy (G)'/,
     &  2X,A1,'3.  Density (D)',T40,A1,
     &        '14. Second Virial Coefficient (B)'/,
     &  2X,A1,'4.  Specific Volume (V)',T40,A1,
     &        '15. Third Virial Coefficient (C)'/,
     &  2X,A1,'5.  Internal Energy (U)',T40,A1,
     &        '16. Fugacity Coefficient (F)'/,
     &  2X,A1,'6.  Enthalpy (H)',T40,A1,
     &        '17. dP/dT (PT)'/,
     &  2X,A1,'7.  Entropy (S)',T40,A1,
     &        '18. dP/dD (PD1)'/,
     &  2X,A1,'8.  Isochoric Heat Capacity (CV)',T40,A1,
     &        '19. d^2(P)/(dD)^2 (PD2)'/,
     &  2X,A1,'9.  Isobaric Heat Capacity (CP)',T40,A1,
     &        '20. Viscosity (VIS)'/,
     &  2X,A1,'10. Speed of Sound (W)',T40,A1,
     &        '21. Thermal Conductivity (THC)'/,
     &  2X,A1,'11. Compressibility Factor (Z)',T40,A1,
     &        '22. Surface Tension (ST)'//,
     &  2X,A1,'23. Joule-Thomson Coefficient       ',T40,A1,
     &        '27. Adiabatic Compressibility       '/,
     &  2X,A1,'24. Isentropic Expansion Coefficient',T40,A1,
     &        '28. Adiabatic Bulk Modulus          '/,
     &  2X,A1,'25. Isothermal Expansion Coefficient',T40,A1,
     &        '29. Isothermal Compressibility      '/,
     &  2X,A1,'26. Volume Expansivity              ',T40,A1,
     &        '30. Isothermal Bulk Modulus         '//,
     &  2X,  ' -1. Erase all current settings',/,
     &  2X,  '  0. Return to the main menu',//,
     &  ' Enter the corresponding letter (or number) to mark or unmark',
     &  '  ---> ')
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE PAUSE2
      INCLUDE 'commons.for'
      CHARACTER*1 DUMMY
      IF (PRNTFLG) WRITE (*,*)
      IF (PRNTFLG) WRITE (*,*) 'Press <Enter> to continue ... '
      READ (*,'(A1)') dummy
      DUMMY = DUMMY
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE PROPS(P, D, T, TAG, PFLG, QFLG, SATFLG)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C     Calculate all fluid properties using inputs of density and
C     and temperature.
C
C  DESCRIPTION OF ARGUMENTS:
C
C     P - Pressure, in SI units.
C     D - Density, in SI units. (Input, unless PFLG = 2)
C     T - Temperature, in SI units. (Input)
C     TAG - A character string printed in the first output column
C           to indicate a solid, liquid, or vapor point, or to
C           print out the quality of a two-phase point.
C     PFLG - Set to 1 for screen output, otherwise set to 0.
C     QFLG - For quality properties, set to 1, otherwise set to 0.
C     SATFLG - Should always be set to one when SATT or SATP was called
C              prior to calling to PROPS, otherwise set to 0.
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comdrv.for'
      INTEGER QUALITYFLAG
      REAL*8 DSL, DSV, Q
      COMMON /QUALITY/ DSL, DSV, Q, QUALITYFLAG

C--Arguments----------------------------
      INTEGER PFLG, QFLG, SATFLG
      REAL*8 P, D, T
      CHARACTER*10 TAG

C--Functions Called---------------------
      REAL*8 PFNDA, UFNDA, HFNDA, SFNDA, CVFNDA, CPFNDA
      REAL*8 WFNDA, ZFNDA, AFNDA, GFNDA, VIR2, VIR3, FUGCOE
      REAL*8 DPDT, DPDD1, DPDD2, VISCTY, THCND, SURFTN

C--Local Variables----------------------
      CHARACTER*10 STR(NV), TAG1, FORMT
      INTEGER I, SI, CHECK
      REAL*8 P1, D1, JT, K, KT, BETA, BulkModS, BS, KAPPA, BulkModT
      REAL*8 Q1

C--Calculate Properties in the SI Units System -------------------------
      DO 10 I=1, NV
 10   STR(I) = ' '
      TAG1 = TAG
      P1 = P
      IF (P.EQ.0) P1 = PFNDA(D, T)
      IF (SATFLG.EQ.0 .AND. QFLG.EQ.0) THEN
        IF (T-THIGH(NDX).GT.0.5D-7 .OR. T.LT.TLOW(NDX)) RETURN
        IF (D-DHIGH(NDX).GT.0.5D-7) RETURN
        IF (P1-PHIGH(NDX).GT.0.5D-7) RETURN
      ENDIF
      IF (VAR(1).EQ.1) STR(1) = FORMT(P1/PFACT - GFACT)
      IF (VAR(2).EQ.1) STR(2) = FORMT(T/TFACT - CFACT)
      IF (QFLG.EQ.0) THEN
        IF (VAR(3).EQ.1) STR(3) = FORMT(D/DFACT)
        IF (VAR(4).EQ.1) STR(4) = FORMT(1.D0/D/VFACT)
        IF (SATFLG.EQ.1) THEN
          IF (D.GT.DC(NDX)) THEN
            IF (VAR(5).EQ.1) STR(5) = FORMT(ULQ/HFACT)
            IF (VAR(6).EQ.1) STR(6) = FORMT(HLQ/HFACT)
            IF (VAR(7).EQ.1) STR(7) = FORMT(SLQ/SFACT)
          ELSE
            IF (VAR(5).EQ.1) STR(5) = FORMT(UVQ/HFACT)
            IF (VAR(6).EQ.1) STR(6) = FORMT(HVQ/HFACT)
            IF (VAR(7).EQ.1) STR(7) = FORMT(SVQ/SFACT)
          ENDIF
        ELSE
          IF (VAR(5).EQ.1) STR(5) = FORMT(UFNDA(D, T)/HFACT)
          IF (VAR(6).EQ.1) STR(6) = FORMT(HFNDA(D, T)/HFACT)
          IF (VAR(7).EQ.1) STR(7) = FORMT(SFNDA(D, T)/SFACT)
        ENDIF
        IF (SATFLG.EQ.0 .OR.
     &     (SATFLG.EQ.1 .AND. DHIGH(NDX)-D.GT.-0.5D-9)) THEN
          IF (VAR(8).EQ.1)  STR(8)  = FORMT(CVFNDA(D, T)/SFACT)
          IF (VAR(9).EQ.1)  STR(9)  = FORMT(CPFNDA(D, T)/SFACT)
          IF (VAR(10).EQ.1) STR(10) = FORMT(WFNDA(D, T)/WFACT)
          IF (VAR(11).EQ.1) STR(11) = FORMT(ZFNDA(D, T))
          IF (VAR(12).EQ.1) STR(12) = FORMT(AFNDA(D, T)/HFACT)
          IF (VAR(13).EQ.1) STR(13) = FORMT(GFNDA(D, T)/HFACT)
          IF (VAR(14).EQ.1) STR(14) = FORMT(VIR2(T)*DFACT)
          IF (VAR(15).EQ.1) STR(15) = FORMT(VIR3(T)*DFACT*DFACT)
          IF (VAR(16).EQ.1) STR(16) = FORMT(FUGCOE(D, T))
          IF (VAR(17).EQ.1) STR(17) = FORMT(DPDT(D, T)/(PFACT/TFACT))
          IF (VAR(18).EQ.1) STR(18) = FORMT(DPDD1(D, T)/(PFACT/DFACT))
          IF (VAR(19).EQ.1) STR(19) = FORMT(DPDD2(D,T)/(PFACT/DFACT**2))
          IF (VAR(20).EQ.1) STR(20) = FORMT(VISCTY(D, T)/VISFACT)
          IF (VAR(21).EQ.1) STR(21) = FORMT(THCND(D, T)/THCNDFACT)
          IF (VAR(22).EQ.1 .AND. TAG.EQ.' Liquid')
     &        STR(22) = FORMT(SURFTN(T)/STFACT)
          CHECK = VAR(23)+VAR(24)+VAR(25)+VAR(26)+VAR(27)+VAR(28)+
     &        VAR(29)+VAR(30)
          IF (CHECK.NE.0) THEN
            CALL PROPERTY(D,T,JT,K,KT,BETA,BulkModS,BS,KAPPA,BulkModT)
            IF (VAR(23).EQ.1) STR(23) = FORMT(JT/(TFACT/PFACT))
            IF (VAR(24).EQ.1) STR(24) = FORMT(K)
            IF (VAR(25).EQ.1) STR(25) = FORMT(KT)
            IF (VAR(26).EQ.1) STR(26) = FORMT(BETA*TFACT)
            IF (VAR(27).EQ.1) STR(27) = FORMT(BulkModS*PFACT)
            IF (VAR(28).EQ.1) STR(28) = FORMT(BS/PFACT)
            IF (VAR(29).EQ.1) STR(29) = FORMT(KAPPA*PFACT)
            IF (VAR(30).EQ.1) STR(30) = FORMT(BulkModT/PFACT)
          ENDIF
        ENDIF
      ELSE
        Q1 = 1.D0 - Q
        D1 = 1.D0/(Q1/DSL + Q/DSV)
        IF (VAR(3).EQ.1) STR(3) = FORMT(D1/DFACT)
        IF (VAR(4).EQ.1) STR(4) = FORMT(1.D0/D1/VFACT)
        IF (VAR(5).EQ.1) STR(5) = FORMT((Q1*ULQ + Q*UVQ)/HFACT)
        IF (VAR(6).EQ.1) STR(6) = FORMT((Q1*HLQ + Q*HVQ)/HFACT)
        IF (VAR(7).EQ.1) STR(7) = FORMT((Q1*SLQ + Q*SVQ)/SFACT)
        DO 50 I=8,NV
 50     STR(8)  = '   UND    '
        WRITE(TAG1,'(F6.4)') Q
        TAG1 = ' X='//TAG1(1:6)
        IF (QUALITYFLAG.EQ.1) TAG1 = ' Two Phs  '
      ENDIF

      DO 20 I = 2, NS
 20   STATES(I - 1) = STATES(I)
      STATES(NS)(1:10)=TAG1
      SI = 10
      DO 30 I=1, NV
        IF (VAR(I).EQ.1) THEN
          STATES(NS) = STATES(NS)(1:SI)//STR(I)
          SI=SI+10
        ENDIF
 30   CONTINUE
      WRITE (IOUT, ALNGTH) STATES(NS)
      IF (PFLG.EQ.1 .AND. PRNTFLG) WRITE (*,'(A80)') STATES(NS)
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE STATUS
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Routine to display the current program settings and fixed points
C    for the current fluid.
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comdrv.for'

C--Local Variables----------------------
      INTEGER I, LENGTH
      CHARACTER*3 YES, YES2
      CHARACTER*1 VAR1(NV)
      CHARACTER*80 UNITOPT
      CHARACTER*10 FORMT, A1, A2, A3, A4

C--DISPLAY PROGRAM SETTINGS.--------------------------------------------
      DO 10 I = 1, NV
        VAR1(I) = ' '
 10     IF (VAR(I).EQ.1) VAR1(I) = '*'
      YES = 'NO '
      YES2 = 'NO '
      IF (IDGPRP) YES = 'YES'
      IF (TITLEQ) YES2 = 'YES'
      A1 = FORMT(RGAS(NDX)*1000.D0/SFACT)
      IF (A1(8:10).EQ.' ') A1 = '   '//A1(1:7)
      CALL SYSTEM("CLS")
      UNITOPT = ALPHP(1:LENGTH(ALPHP))//', '//
     &          ALPHT(1:LENGTH(ALPHT))//', '//
     &          ALPHD(1:LENGTH(ALPHD))//', '//
     &          ALPHH(1:LENGTH(ALPHH))//', '//
     &          ALPHS(1:LENGTH(ALPHS))//', '//
     &          ALPHW(1:LENGTH(ALPHW))
      WRITE (*,1000) TNAMEQ(NDX), A1, ALPHS, AMOL(NDX),
     &               OUTOPT, UNITOPT, YES, YES2, VAR1
      A1 = FORMT(TC(NDX)/TFACT-CFACT)
      A2 = FORMT(PC(NDX)/PFACT-GFACT)
      A3 = FORMT(DC(NDX)/DFACT)
      IF (A1(8:10).EQ.' ') A1 = '   '//A1(1:7)
      IF (A2(8:10).EQ.' ') A2 = '   '//A2(1:7)
      IF (A3(8:10).EQ.' ') A3 = '   '//A3(1:7)
      WRITE (*,1010) A1, ALPHT, A2, ALPHP, A3, ALPHD
      A1 = FORMT(TLOW(NDX)/TFACT-CFACT)
      A2 = FORMT(THIGH(NDX)/TFACT-CFACT)
      A3 = FORMT(PHIGH(NDX)/PFACT-GFACT)
      A4 = FORMT(DHIGH(NDX)/DFACT)
      IF (A1(8:10).EQ.' ') A1 = '   '//A1(1:7)
      IF (A2(8:10).EQ.' ') A2 = '   '//A2(1:7)
      IF (A3(8:10).EQ.' ') A3 = '   '//A3(1:7)
      IF (A4(8:10).EQ.' ') A4 = '   '//A4(1:7)
      WRITE (*,1020) A1, A2, ALPHT, A3, ALPHP, A4, ALPHD
      IF (INTERQ) CALL PAUSE2
C-----------------------------------------------------------------------
 1000 FORMAT(
     &'  WORKING FLUID:  ',A25,/,
     &'  GAS CONSTANT:',A10,1X,A10,/,
     &'  MOLAR MASS:    ',F7.3,' g/mol',//,
     &'  CURRENT OPTION SETTINGS:',/,
     &'    OUTPUT DEVICE:  ',A60,/,
     &'    UNITS SYSTEM:   ',A60,/,
     &'    IDEAL GAS PROPERTIES:  ',A3,'  TITLES:  ',A3,/,
     &'    VARIABLES DISPLAYED (Marked with an asterisk):',/,
     &'    ',A1,'P   ',A1,'T   ',A1,'D   ',A1,'V   ',A1,'U   ',A1,'H   '
     &      ,A1,'S   ',A1,'Cv  ',A1,'Cp  ',A1,'W   ',A1,'Z',/
     &'    ',A1,'A   ',A1,'G   ',A1,'B   ',A1,'C   ',A1,'F   ',
     &       A1,'dP/dT ',A1,'dP/dD ',A1,'d2P/dD2   ',
     &       A1,'VI  ',A1,'THCND ',A1,'ST  ',/,
     &'    ',A1,'JT  ',A1,'k   ',A1,'kT  ',A1,'Beta ',
     &       A1,'BetaS ',A1,'BulkModS ',A1,'Kappa ',A1,'BulkModT '/)

 1010 FORMAT('  CRITICAL POINT VALUES:',/,
     &'    CRITICAL TEMPERATURE:',A10,1X,A10,/,
     &'    CRITICAL PRESSURE:   ',A10,1X,A10,/,
     &'    CRITICAL DENSITY:    ',A10,1X,A10,/)
 1020 FORMAT('  RANGE OF VALIDITY:',/,
     &'    TEMPERATURE:        ',A10,' TO ', A10,1X,A10,/,
     &'    HIGH PRESSURE LIMIT:',A10,1X,A10,/,
     &'    MAXIMUM DENSITY:    ',A10,1X,A10)
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE UNITS(ISET)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C     Routine to setup units conversion factors and labels for headings.
C
C  DESCRIPTION OF ARGUMENT:
C     ISET    When ISET = 0, the user is not asked to select units,
C             rather, the units system defined by the OPTSET variable
C             is used.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comdrv.for'

C--Arguments----------------------------
      INTEGER ISET, I

C--Local Variables----------------------
      INTEGER UNIT, OPT1
      CHARACTER OPT*2, ALPHA(6,13)*10, UNITITLE(10)*32, ALPHxx*10

C-----------------------------------------------------------------------
      DATA UNITITLE/
     & 'TEMPERATURE MENU',
     & 'PRESSURE MENU',
     & 'DENSITY MENU',
     & 'SPECIFIC VOLUME MENU',
     & 'ENERGY MENU',
     & 'HEAT CAPACITY AND ENTROPY MENU',
     & 'SPEED MENU',
     & 'VISCOSITY MENU',
     & 'THERMAL CONDUCTIVITIY MENU',
     & 'SURFACE TENSION MENU'/

      DATA ALPHA/
     & 'K','C','R','F','T/Tc',' ',
     & 'MPa','psia','bar','mmHg','atm','P/Pc',
     & 'mol/dm3','kg/m3','lbm/ft3','D/Dc',' ',' ',
     & 'dm3/mol','m3/kg','ft3/lbm','V/Vc',' ',' ',
     & 'J/mol','kJ/kg','Btu/lbm','/RTc','kcal/kg',' ',
     & 'J/mol-K','kJ/kg-K','Btu/lbm-R','/R','kcal/kg-K',' ',
     & 'm/s','ft/s','---',' ',' ',' ',
     & 'Pa-s','g/cm-s','c-poise','u-poise','lbm/ft-s','lbm/ft-hr',
     & 'mW/m-K','g-cm/s3-K','cal/s-cm-K','lbf/s-F','Btu/hr-ftF',' ',
     & 'N/m','dyne/cm','lbf/ft',' ',' ',' ',
     & 'dm3/mol','m3/kg','ft3/lbm','Dc/D',' ',' ',
     & 'dm6/mol2','m6/kg2','ft6/lbm2','(Dc/D)2',' ',' ',
     & 'MPa gage','psig','bar gage','mmHg gage','atm gage',' '/

C--Display the Units Menu-----------------------------------------------
      DO 10 I = 1, NS
 10   STATES(I) = ' '
 700  IF (ISET.EQ.0) THEN
        OPT = OPTSET
        IF (OPT.EQ.' ') THEN
          WRITE (*,*) ' UNITS:  OPTSET NOT SET'
          CALL PAUSE2
        ENDIF
        IF (OPT.EQ.'5') THEN
          CALL FACT
          RETURN
        ENDIF
      ELSE
        CALL SYSTEM("CLS")
        IF (PRNTFLG) WRITE (*,1000)
        READ (*,'(A2)') OPT
      ENDIF

C--S. I. Units with molar density and Megapascals for pressure.---------
      IF (OPT.EQ.'1' .OR. OPT.EQ.'S' .OR. OPT.EQ.'s') THEN
        ALPHT = 'K'
        ALPHP = 'MPa'
        ALPHD = 'mol/dm3'
        ALPHB = 'dm3/mol'
        ALPHC = 'dm6/mol2'
        ALPHH = 'J/mol'
        ALPHS = 'J/mol-K'
        ALPHW = 'm/s'
        ALPHV = 'dm3/mol'
        ALPHVIS = 'Pa-s'
        ALPHK   = 'mW/m-K'
        ALPHST  = 'dyne/cm'
        OPTSET  = '1'
        CALL FACT

C--S. I. Units with mass density and Megapascals for pressure.----------
      ELSEIF (OPT.EQ.'2') THEN
        ALPHT = 'C'
        ALPHP = 'MPa'
        ALPHD = 'kg/m3'
        ALPHB = 'm3/kg'
        ALPHC = 'm6/kg2'
        ALPHH = 'kJ/kg'
        ALPHS = 'kJ/kg-K'
        ALPHW = 'm/s'
        ALPHV = 'm3/kg'
        ALPHVIS = 'Pa-s'
        ALPHK   = 'mW/m-K'
        ALPHST  = 'dyne/cm'
        OPTSET  = '2'
        CALL FACT

C--Engineering Units with mass density and psia for pressure.-----------
      ELSEIF (OPT.EQ.'3'.OR.OPT.EQ.'E'.OR.OPT.EQ.'e') THEN
        ALPHT = 'F'
        ALPHP = 'psia'
        ALPHD = 'lbm/ft3'
        ALPHB = 'ft3/lbm'
        ALPHC = 'ft6/lbm2'
        ALPHH = 'Btu/lbm'
        ALPHS = 'Btu/lbm-R'
        ALPHW = 'ft/s'
        ALPHV = 'ft3/lbm'
        ALPHVIS = 'lbm/ft-s'
        ALPHK   = 'lbf/s-F'
        ALPHST  = 'lbf/ft'
        OPTSET  = '3'
        CALL FACT

C--Reduced units.-------------------------------------------------------
      ELSEIF (OPT.EQ.'4' .OR. OPT.EQ.'R' .OR. OPT.EQ.'r') THEN
        ALPHT = 'T/Tc'
        ALPHP = 'P/Pc'
        ALPHD = 'D/Dc'
        ALPHB = 'Dc/D'
        ALPHC = 'Dc/D2'
        ALPHH = '/RTc'
        ALPHS = '/R'
        ALPHW = '---'
        ALPHV = 'V/Vc'
        ALPHVIS = 'Pa-s'
        ALPHK   = 'mW/m-K'
        ALPHST  = 'dyne/cm'
        OPTSET  = '4'
        CALL FACT

C--Customized units.----------------------------------------------------
      ELSEIF (OPT.EQ.'5' .OR. OPT.EQ.'C' .OR. OPT.EQ.'c') THEN
 710    CALL SYSTEM("CLS")
        WRITE(*,1010) ALPHT, ALPHP, ALPHD, ALPHV, ALPHH, ALPHS, ALPHW,
     &                ALPHVIS, ALPHK, ALPHST
        READ(*,'(A2)') OPT
        OPT1 = 0
        READ (OPT,*,ERR=750) OPT1
 750    IF (OPT.EQ.'T' .OR. OPT.EQ.'t') OPT1 = 1
        IF (OPT.EQ.'P' .OR. OPT.EQ.'p') OPT1 = 2
        IF (OPT.EQ.'D' .OR. OPT.EQ.'d') OPT1 = 3
        IF (OPT.EQ.'V' .OR. OPT.EQ.'v') OPT1 = 4
        IF (OPT.EQ.'E' .OR. OPT.EQ.'e') OPT1 = 5
        IF (OPT.EQ.'H' .OR. OPT.EQ.'h') OPT1 = 6
        IF (OPT.EQ.'S' .OR. OPT.EQ.'s') OPT1 = 7
        IF (OPT1.GT.10) GOTO 710

        IF (OPT1.EQ.0 .OR. OPT.EQ.'R' .OR. OPT.EQ.'r') THEN
          OPTSET = '5'
          CALL FACT
          RETURN
        ENDIF

        IF (OPT1.EQ.1)  ALPHxx = ALPHT
        IF (OPT1.EQ.2)  ALPHxx = ALPHP
        IF (OPT1.EQ.3)  ALPHxx = ALPHD
        IF (OPT1.EQ.4)  ALPHxx = ALPHV
        IF (OPT1.EQ.5)  ALPHxx = ALPHH
        IF (OPT1.EQ.6)  ALPHxx = ALPHS
        IF (OPT1.EQ.7)  ALPHxx = ALPHW
        IF (OPT1.EQ.8)  ALPHxx = ALPHVIS
        IF (OPT1.EQ.9)  ALPHxx = ALPHK
        IF (OPT1.EQ.10) ALPHxx = ALPHST

        WRITE (*,*) ' ', UNITITLE(OPT1)
        WRITE(*,1030) ALPHxx
        DO 730 I=1, 6
 730    IF (ALPHA(I, OPT1).NE.' ') WRITE (*,1040) I, ALPHA(I, OPT1)
        IF (OPT1.EQ.2) THEN
          DO 740 I=1, 5
            IF (I.LE.3) WRITE (*,1040) I+6, ALPHA(I, 13)
            IF (I.GT.3) WRITE (*,1045) I+6, ALPHA(I, 13)
 740      CONTINUE
        ENDIF
        WRITE(*,1050)
        READ(*,*,ERR=720) UNIT
        IF (UNIT.LT.1) GOTO 710
        IF (UNIT.GT.6 .AND. .NOT.(UNIT.LE.12 .AND. OPT1.EQ.2)) GOTO 710
        IF (ALPHA(UNIT, OPT1).NE.' ') ALPHxx = ALPHA(UNIT, OPT1)
        IF (OPT1.EQ.2 .AND. UNIT.GE.7) ALPHxx = ALPHA(UNIT-6, 13)
        IF (ALPHxx.EQ.' ') GOTO 710

        IF (OPT1.EQ.1)  ALPHT   = ALPHxx
        IF (OPT1.EQ.2)  ALPHP   = ALPHxx
        IF (OPT1.EQ.3)  ALPHD   = ALPHxx
        IF (OPT1.EQ.4)  ALPHV   = ALPHxx
        IF (OPT1.EQ.5)  ALPHH   = ALPHxx
        IF (OPT1.EQ.6)  ALPHS   = ALPHxx
        IF (OPT1.EQ.7)  ALPHW   = ALPHxx
        IF (OPT1.EQ.8)  ALPHVIS = ALPHxx
        IF (OPT1.EQ.9)  ALPHK   = ALPHxx
        IF (OPT1.EQ.10) ALPHST  = ALPHxx
        IF (OPT1.EQ.3 .OR. OPT1.EQ.4) THEN
          ALPHB = ALPHA(UNIT, 11)
          ALPHC = ALPHA(UNIT, 12)
        ENDIF
        GOTO 710
 720    WRITE (*,*) 'YOU MUST SELECT A NUMBER IN THIS MENU'
        CALL PAUSE2
        GOTO 710
      ELSEIF (OPT.EQ.'6' .OR. OPT.EQ.'S' .OR. OPT.EQ.'s') THEN
        CALL STATUS
        GOTO 700
      ELSEIF (OPT.EQ.'0' .OR. OPT.EQ.'Q' .OR. OPT.EQ.'q') THEN
      ELSE
        GOTO 700
      ENDIF
C-----------------------------------------------------------------------
 1000   FORMAT ('  UNITS MENU (Select a system of units):',//,
     &  '  (1)  S. I. (molar)   (MPa, mol, K, Joules, m, s)',/,
     &  '  (2)  S. I. (mass)    (MPa, kg, C, kJoules, m, s)',/,
     &  '  (3)  Engineering     (psia, lbm, F, Btu, ft, s)',/,
     &  '  (4)  Dimensionless Parameters',/,
     &  '  (5)  Custom',/,
     &  '  (6)  Status',/,
     &  '  (0)  Return to the main menu',//)

 1010   FORMAT ('  CUSTOM MENU:',//
     &  '  (1)  Temperature                ===>  ',A10,/,
     &  '  (2)  Pressure                   ===>  ',A10,/,
     &  '  (3)  Density                    ===>  ',A10,/,
     &  '  (4)  Volume                     ===>  ',A10,/,
     &  '  (5)  Energy                     ===>  ',A10,/,
     &  '  (6)  Heat Capacity and Entropy  ===>  ',A10,/,
     &  '  (7)  Speed of Sound             ===>  ',A10,/,
     &  '  (8)  Viscosity                  ===>  ',A10,/,
     &  '  (9)  Thermal Conductivity       ===>  ',A10,/,
     &  '  (10) Surface Tension            ===>  ',A10,/,
     &  '  (0)  Return to main menu',//,
     &  '  Select variables to customize   ===>  ')

 1030   FORMAT ( '  The current units are ',A10/)
 1040   FORMAT ( '    (',I1,')    ',A10)
 1045   FORMAT ( '    (',I2, ')   ',A10)
 1050   FORMAT (/'  Select the units desired ====> ')
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION LENGTH(A)
      INTEGER LENGTH, I
      CHARACTER*10 A
      LENGTH = 0
      DO 10 I=1, 10
 10   IF (A(I:I).NE.' ' .AND. A(I:I).NE.CHAR(0)) LENGTH = I
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION FORMT(PRP)
C
C--Local Variables----------------------
      CHARACTER FORMT*10
      REAL*8 PRP

C-----------------------------------------------------------------------
      IF (DABS(PRP).LT.1.D-9) THEN
        WRITE (FORMT, 40) 0.D0
      ELSEIF (PRP.GE.0) THEN
        IF (PRP.GE.0.99995D9) THEN
          WRITE (FORMT, 30) PRP
        ELSEIF (PRP.GE.999999.5D0 .OR. PRP.LT.0.1D0) THEN
          WRITE (FORMT, 20) PRP
        ELSE
          WRITE (FORMT, 10) PRP
        ENDIF
      ELSE
        IF (PRP.LE.-0.999995D9) THEN
          WRITE (FORMT, 70) PRP
        ELSEIF (PRP.LE.-99999.5D0 .OR. PRP.GT.-0.1D0) THEN
          WRITE (FORMT, 60) PRP
        ELSE
          WRITE (FORMT, 50) PRP
        ENDIF
      ENDIF
 10   FORMAT (G10.6E1)
 20   FORMAT (G9.5E1,1X)
 30   FORMAT (G9.4E2,1X)
 40   FORMAT (F7.5,3X)
 50   FORMAT (G10.5E1)
 60   FORMAT (G9.4E1,1X)
 70   FORMAT (G9.3E2,1X)
      END
