C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C=======================================================================
C
C  File Name:  COMDRV.FOR
C
C  Purpose:
C
C     Contains the common blocks in ALLPROPS that are used solely by
C     ALLPROPS.FOR and are not likely to be used by other programmers.
C     This file should be included in all programs and subprograms that
C     reference these common blocks.  The syntax is:
C     INCLUDE 'COMDRV.FOR'
C
C=======================================================================
C  PARAMETERS:   NS = Number of scroll lines.
C                NV = Number of properties available.
C-----------------------------------------------------------------------
      INTEGER NS, NV
      PARAMETER (NS = 10)
      PARAMETER (NV = 30)


C=======================================================================
C  VARIABLES IN /HEADING/ AND /HEADINGL/
C
C    STRING1  - First line of header information.
C    STRING2  - Second line of header information.
C    ALNGTH   - Format string used in printing properties. (ex: '(A80)')
C    IDGTITLE - Ideal gas title.
C    TITLE    - Real gas title.
C    TITLE1   - String of dashes used in the header.
C    TITLE2   - String of dashes for printing to a file,
C               length depends of number of properties selected.
C    TITLEQ   - Logigal variable that prints Heading/Labels only if TRUE.
C-----------------------------------------------------------------------
      LOGICAL TITLEQ
      CHARACTER*340 STRING1, STRING2
      CHARACTER*80 IDGTITLE, TITLE, TITLE1, TITLE2
      CHARACTER*10 ALNGTH
      COMMON /HEADINGL/TITLEQ
      COMMON /HEADING/ STRING1, STRING2, ALNGTH,
     &                 IDGTITLE, TITLE, TITLE1, TITLE2

C=======================================================================
C  VARIABLES IN /SETTINGS/
C    VAR     - An array in which a one specifies that the property
C              will be printed, and a zero specifies that it will not
C              be printed.
C    STATES  - An array holding the last 10 displayed lines.  The lines
C              scroll up once states has been filled in single point
C              calculations.
C    IOUT    - The device number for printing text (for a file or printer.)
C    OUTOPT  - The device name opened under IOUT.
C    OPTSET  - Tells which units have been selected.  1 and 2 are SI,
C              3 and 4 are english, 5 is dimensionless, and 6 is custom.
C-----------------------------------------------------------------------
      INTEGER VAR(NV)
      CHARACTER*340 STATES(NS)
      CHARACTER OUTOPT*80, OPTSET*1
      COMMON /SETTINGI/ VAR
      COMMON /SETTINGS/ OUTOPT, OPTSET, STATES

C=======================================================================
C  VARIABLES IN /UNIT/
C    Conversion factors for thermodynamic properties.
C    Example:  conversion from SI to another system:
C                  TNEW = T/TFACT - CFACT
C                  PNEW = P/PFACT - GFACT
C                  DNEW = D/DFACT
C  PREFIXES:
C    P - Pressure          T - Temperature         C - Temperature offset
C    D - Density           H - Enthalpy            W - Speed of sound
C    V - Volume            S - Entropy             G - Gage offset
C-----------------------------------------------------------------------
      REAL*8      PFACT,GFACT,TFACT,CFACT,DFACT,VFACT,HFACT,SFACT,WFACT,
     &            VISFACT, THCNDFACT, STFACT
      COMMON/UNIT/PFACT,GFACT,TFACT,CFACT,DFACT,VFACT,HFACT,SFACT,WFACT,
     &            VISFACT, THCNDFACT, STFACT

C=======================================================================
C  VARIABLES IN /LABELS/
C    Unit identifiers, such as 'MPa' and 'K', for labeling properties
C    used in ALLPROPS.
C-----------------------------------------------------------------------
      CHARACTER*10 ALPHP, ALPHT, ALPHD, ALPHV, ALPHH, ALPHS, ALPHW
      CHARACTER*10 ALPHB, ALPHC, ALPHVIS, ALPHK, ALPHST
      COMMON /LABLES/ ALPHP, ALPHT, ALPHD, ALPHV, ALPHH, ALPHS, ALPHW,
     &                ALPHB, ALPHC, ALPHVIS, ALPHK, ALPHST
