C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C=======================================================================
C
C  File Name:  COMMONS.FOR
C
C  Purpose:
C
C     Contains the most general common blocks used in ALLPROPS.
C     This file should be included in all programs and subprograms that
C     reference these common blocks.  The syntax is:
C     INCLUDE 'COMMONS.FOR'
C
C=======================================================================
C  PARAMETERS:  NF = Number of fluids.
C-----------------------------------------------------------------------
      INTEGER NF
      PARAMETER (NF = 2)

C=======================================================================
C  VARIABLES IN /FLUID/
C
C    NDX    - Current fluid index.
C    IDGPRP - Ideal gas property flag.
C    NME    - Fluid specific character identification string.
C    HEADRQ - Holds RCS Header for reference file.
C    TNAMEQ - Long name of fluid, used in titles
C-----------------------------------------------------------------------
      LOGICAL IDGPRP
      INTEGER NDX
      CHARACTER HEADRQ*80, NME(NF)*80, TNAMEQ(NF)*25
      COMMON /FLUID/ NDX, IDGPRP
      COMMON /FLUIDC/ HEADRQ, TNAMEQ, NME

C=======================================================================
C  VARIABLES IN /CNST/
C
C    TC   - Critical point temperature. (K)
C    PC   - Critical point pressure. (MPa)
C    DC   - Critical point density. (mol/dm3)
C    TTRP - Triple point temperature. (K)
C    PTRP - Triple point pressure. (MPa)
C    AMOL - Molecular weight. (g/mol)
C    RGAS - Universal molar gas constant. (kJ/mol-K)
C    AF   - Pitzer's acentric factor.
C    TSCL - Equation of state temperature scale. (K)
C    MXWFLG ----- Identifies how saturation properties are calculated:
C                 MXWFLG =  1.0:     Maxwell solution.
C                 0.0 < MXWFLG < 1.0:Maxwell solution up to MXFLG*T/Tc
C                 MXWFLG =  0.0:     Intersection of T and Pv with EOS.
C                 MXWFLG = -1.0:     Intersection of T and Pv with EOS
C                                    to get Dv, Ancillary equation for Dl.
C                 MXWFLG = -2.0:     Ancillary equations.
C    EOS -------- Identifies the equation of state type:
C                 EOS = 1            Helmholtz Equation.
C                 EOS = 2            Martin Hou.
C                 EOS = 3            Wagner methane equation of state.
C                 EOS = 4            Ammonia equation of state.
C-----------------------------------------------------------------------
      INTEGER EOS(NF), TSCL(NF)
      REAL*8 AMOL(NF), RGAS(NF), DC(NF), PC(NF)
      REAL*8 TC(NF), AF(NF), PTRP(NF), TTRP(NF), MXWFLG(NF)
      COMMON /CNST/ AMOL, RGAS, DC, PC, TC, AF, PTRP, TTRP, MXWFLG, EOS,
     &              TSCL

C=======================================================================
C  VARIABLES IN /LIMITS/
C
C    DLOW, DHIGH - Lower and upper density bounds for the EOS. (mol/dm3)
C    PLOW, PHIGH - Lower and upper pressure bounds for the EOS. (MPa)
C    TLOW, THIGH - Lower and upper temperature bounds for the EOS. (K)
C-----------------------------------------------------------------------
      REAL*8 DLOW(NF), DHIGH(NF), PLOW(NF)
      REAL*8 PHIGH(NF), TLOW(NF), THIGH(NF)
      COMMON /LIMITS/ DLOW, DHIGH, PLOW, PHIGH, TLOW, THIGH

C=======================================================================
C  VARIABLES IN /REFPRP/
C
C    P0   - Reference state pressure. (MPa)
C    T0   - Reference state temperature. (K)
C    H0   - Reference state enthalpy divided by RGAS.
C    S0   - Reference state entropy divided by RGAS.
C    SMIX - Dimensionless ideal gas entropy of mixing.
C-----------------------------------------------------------------------
      REAL*8 H0(NF), P0(NF), S0(NF), T0(NF), SMIX(NF)
      COMMON /REFPRP/ H0, P0, S0, T0, SMIX

C=======================================================================
C  VARIABLES IN /SATLNE/
C
C    UVQ  - Saturated vapor energy.
C    HVQ  - Saturated vapor enthalpy.
C    SVQ  - Saturated vapor entropy.
C    ULQ  - Saturated liquid energy.
C    HLQ  - Saturated liquid enthalpy.
C    SLQ  - Saturated liquid entropy.
C-----------------------------------------------------------------------
      REAL*8 UVQ, ULQ, HVQ, HLQ, SVQ, SLQ
      COMMON /SATLNE/ UVQ, ULQ, HVQ, HLQ, SVQ, SLQ

C=======================================================================
C  VARIABLES IN /ERRORX/
C
C    ERRDEV  - Device to which error messages can be printed.
C    ERRNUM  - Indicates an error occurred in a subroutine when set to one.
C    ERRMSG  - Indicates whether the error message should be printed.
C    INTERQ  - Logical flag that controls simple pauses. If INTERQ is
C              true, the program will pause; if false, it won't.  INTERQ
C              is true if the first user response is the ENTER key; any
C              other response and INTERQ is false.
C    PRNTFLG - Logical flag that controls printing to the screen.
C              Printing to the screen will occur when PRNTFLG is true,
C              no printing will occur when it is false.
C-----------------------------------------------------------------------
      INTEGER ERRDEV, ERRNUM, IOUT
      LOGICAL INTERQ, PRNTFLG, ERRMSG
      COMMON /ERRORX/ ERRDEV, ERRNUM, IOUT, INTERQ, PRNTFLG, ERRMSG

C=======================================================================
C  VARIABLES IN /SAVEIDG/
C
C    IDGDONEQ(0:2)  Logical array used by IDGAS, recalculation flags.
C    IDGVALQ(0:2)   Array of IDGAS values calculated for last T and NDX.
C    TLASTQ         Last temperature in IDGAS.
C    NDXLASTQ       Last NDX value in IDGAS.
C-----------------------------------------------------------------------
      REAL*8 IDGVALQ(0:2), TLASTQ
      INTEGER NDXLASTQ
      LOGICAL IDGDONEQ(0:2)
      COMMON /SAVEIDG/ IDGVALQ, TLASTQ, NDXLASTQ, IDGDONEQ

C=======================================================================
C  VARIABLES IN /BOUNDS/
C
C    TSLD, DSLDH      T and D at intersection of PHIGH and Solid line
C    PSLDH, SSLDH     P and S at      "            "         "    "
C    PSLDL, DSLDL     P and D at intersection of TLOW and Solid line
C
C    TDH1, PDH1       T and P at Int. of DHIGH with either sat. or solid line
C    HDH1, SDH1       H and S at  "         "         "     "        "     "
C    TDH2, HDH2, SDH2 T, H, and S at intersection of DHIGH and PHIGH
C
C    DHH, HHH, SHH    D, H, and S at intersection of THIGH and PHIGH
C    DLH              Density at intersection of Thigh and Plow
C    DLL              Density at intersection of Tlow and Plow
C    HMIN, SMIN       Minimum enthalpy and entropy possible.
C    HMAX, SMAX       Maximum enthalpy and entropy possible.
C
C    DSATL, DSATV, PSATL, PSATV     Density and pressure at TLOW
C    SSATL, SSATV, HSATL, HSATV     Entropy and enthalpy at TLOW
C-----------------------------------------------------------------------
      LOGICAL SOLIDLINE(NF)
      REAL*8 TSLD(NF), PSLDH(NF), DSLDH(NF), HSLDH(NF), SSLDH(NF)
      REAL*8 PSLDL(NF), DSLDL(NF)
      REAL*8 TDH1(NF), PDH1(NF), HDH1(NF), SDH1(NF)
      REAL*8 TDH2(NF), HDH2(NF), SDH2(NF)
      REAL*8 DHH(NF), DLH(NF), DLL(NF)
      REAL*8 HHH(NF), SHH(NF), HMIN(NF), SMIN(NF), HMAX(NF), SMAX(NF)
      REAL*8 DSATL(NF), DSATV(NF), PSATL(NF), PSATV(NF)
      REAL*8 SSATL(NF), SSATV(NF), HSATL(NF), HSATV(NF)
      COMMON /BOUNDS/
     &       TSLD, PSLDH, DSLDH, HSLDH, SSLDH,
     &       PSLDL, DSLDL,
     &       TDH1, PDH1, HDH1, SDH1,
     &       TDH2, HDH2, SDH2,
     &       DHH, DLH, DLL,
     &       HHH, SHH, HMIN, SMIN, HMAX, SMAX,
     &       DSATL, DSATV, PSATL, PSATV,
     &       SSATL, SSATV, HSATL, HSATV, SOLIDLINE
