C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C=======================================================================
C
C  File Name:  COMSTATE.FOR
C
C  Purpose:
C
C     Contains the common blocks in ALLPROPS that are used to define
C     the equation of state, and that are used in routines in
C     FUNDEQ.FOR and FUNDIT.FOR.  The include file COMMONS.FOR should
C     be used with this file since it contains the NF parameter.
C     This file should be included in all programs and subprograms that
C     reference these common blocks.  The syntax is:
C     INCLUDE 'COMSTATE.FOR'
C
C=======================================================================
C  PARAMETERS: MAXT = Maximum number of terms in the EOS.
C              MAXSPT = Maximum number of "special" terms in the EOS.
C-----------------------------------------------------------------------
      INTEGER MAXT, MAXSPT
      PARAMETER (MAXT = 50, MAXSPT = 10)

C=======================================================================
C  VARIABLES IN /EOS/
C
C    NT            - Number of terms in the EOS.
C    COEF          - Array of coefficients for the EOS.
C    IEX, EXJ, LEX - Arrays of exponents for the terms of the EOS.
C    GAMMA         - Nonlinear factor in the argument of the exponential.
C-----------------------------------------------------------------------
      INTEGER IEX(MAXT, NF), LEX(MAXT, NF), NT(NF)
      REAL*8 COEF(MAXT, NF), EXJ(MAXT, NF), GAMMA(MAXT, NF)
      REAL*8 KMH, BMH, UMH, CPRIME, DCONV, A0(8), B0(8), C0(8)
      COMMON /EOS/ COEF, EXJ, GAMMA,
     &             KMH, BMH, UMH, CPRIME, DCONV, A0, B0, C0,
     &             IEX, LEX, NT

C=======================================================================
C  VARIABLES IN /STATE/
C    Holds the terms calculated in FUNDEQ.
C    DEL0  - The del term from the previous call to FUNDEQ.
C    TAU0  - The tau term from the previous call to FUNDEQ.
C    NDX0  - The value of NDX from the previous call to FUNDEQ.
C            IF DEL0, TAU0, and NDX0 have not changed, then FXP and HXP
C            do not need to be recalculated.
C    FXP   - The non-exponential part of the EOS.
C    HXP   - The exponential part of the EOS.
C    TERMS - Coefficients of the derivatives of the terms in the EOS.
C    DALARRAY-
C            Array holding calculated derivative functions.
C    DALFLAGS-
C            String of single character flags. If DALFLAGS(i:i) = 1
C            then DALARRAY(i) can be used instead of recalculating
C            the function.
C-----------------------------------------------------------------------
      INTEGER NDX0
      REAL*8 DEL0, FXP(MAXT), HXP(MAXT), TAU0, TERMS(MAXT)
      REAL*8 DALARRAY(16)
      CHARACTER*16 DALFLAGS
      COMMON /STATE/ DALARRAY, DEL0, FXP, HXP, TAU0, TERMS, NDX0
      COMMON /STATEC/ DALFLAGS

C=======================================================================
C  VARIABLES IN /PASSD/
C    Used in FUNDIT.FOR to pass the density in routines where density
C    is not one of the arguments.
C-----------------------------------------------------------------------
      INTEGER IPHASE1
      REAL*8 DENS, PRES
      COMMON /PASSD/ DENS, PRES, IPHASE1

C=======================================================================
C  VARIABLES IN /ANCILLARY/
C     CP0..... - Ideal gas isobaric heat capacity variables.
C     PV...... - Vapor pressure variables.
C     PM...... - Melting pressure variables.
C     DL...... - Saturated liquid density variables.
C     DV...... - Saturated vapor density variables.
C
C     NT...... - Number of terms in the ancillary equations.
C     PWR..... - Exponents for the coefficients contained in the ...COEFQ
C                variables.
C     ...COEFQ - Coefficients for the ancillary equations.
C     ...FLAGQ - Identifies which type of ancillary equation is used.
C
C-----------------------------------------------------------------------
      INTEGER CP0FLAGQ(NF), PVFLAGQ(NF), PMFLAGQ(NF)
      INTEGER DLFLAGQ(NF), DVFLAGQ(NF)
      INTEGER NTCP0Q(NF), NTPVQ(NF), NTPMQ(NF), NTDLQ(NF), NTDVQ(NF)
      REAL*8  PWRCP0Q(20, NF), PWRPVQ(20, NF), PWRPMQ(20, NF)
      REAL*8  PWRDLQ(20, NF), PWRDVQ(20, NF)
      REAL*8  CP0COEFQ(20, NF), CP0COEF2Q(20, NF)
      REAL*8  PVCOEFQ(20, NF), PMCOEFQ(20, NF)
      REAL*8  DLCOEFQ(20, NF), DVCOEFQ(20, NF)
      COMMON /ANCILLARY/
     &        PWRCP0Q, PWRPVQ, PWRPMQ, PWRDLQ, PWRDVQ,
     &        CP0COEFQ, CP0COEF2Q, PVCOEFQ, PMCOEFQ, DLCOEFQ, DVCOEFQ,
     &        NTCP0Q, NTPVQ, NTPMQ, NTDLQ, NTDVQ,
     &        CP0FLAGQ, PVFLAGQ, PMFLAGQ, DLFLAGQ, DVFLAGQ

C=======================================================================
C  VARIABLES IN /METHANE/
C    Parameter MAXSPQ is maximum number of "special" terms in the EOS
C    ALPHAQ(MAXSPQ) -Array of non-zero tau offset values for terms 37-40
C    BETAQ(MAXSPQ)  -Array of non-zero del offset values for terms 37-40
C    SIGMAQ(MAXSPQ) -Array of non-zero tau multipliers for terms 37-40
C
C    TAUA(MAXT)  - TAU  for regular terms; (TAU - ALPHAQ), for special
C    DELB(MAXT)  - DEL for regular terms; (DEL -BETAQ), for special
C    KXP(MAXT)   - 0.D0 for regular terms;
C                  SIGMAQ(K) * TAUA(K) ** LEX(K,NDX) for special terms
C-----------------------------------------------------------------------
      REAL*8 ALPHAQ(MAXSPT), BETAQ(MAXSPT), SIGMAQ(MAXSPT)
      REAL*8 DELB(MAXT), TAUA(MAXT), KXP(MAXT)
      COMMON /METHANE/ ALPHAQ, BETAQ, SIGMAQ, DELB, TAUA, KXP
