C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C--MODULE NAME--
C                FUNDEQ.FOR
C
C--DESCRIPTION--
C                Routines that calculate fundamental properties based
C                on density and temperature.
C
C--CONTENTS------
C  Subroutines:  SUBROUTINE FUNDEQ (DEL, TAU, IDEL, ITAU, ICALC, FUNC)
C                SUBROUTINE FUNDCH4(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
C                SUBROUTINE FUNDNH3(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
C
C  Functions:    AFNDA (D,T)
C                ALPHA0 (DEL, TAU)
C                CVFNDA (D, T)
C                CPFNDA (D, T)
C                CSAT (T, IPHASE)
C                DALP01 (TAU)
C                DALP02 (TAU)
C                DPDD1 (D, T)
C                DPDD2 (D, T)
C                DPDT (D, T)
C                FUGCOE (D, T)
C                GFNDA (D, T)
C                HFNDA (D, T)
C                PFNDA (D, T)
C                SFNDA (D, T)
C                UFNDA (D, T)
C                VIR2 (T)
C                VIR3 (T)
C                WFNDA (D, T)
C                ZFNDA (D, T)
C
C=======================================================================
C
      SUBROUTINE FUNDEQ(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the terms of APLHA-BAR and its partial derivatives with
C    respect to the dimensionless density, DEL, and dimensionless
C    Temperature, TAU.  The value of the function of interest is
C    determined by the choice of the Switches, IDEL and ITAU and
C    is passed back to the calling program in the argument FUNC.
C    The switch, ICALC, determines if the function or only its terms
C    are calculated.  Terms are stored in the array, TERMS.  The TERMS
C    array is accessed by the calling program through COMMON /STATE/.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    DEL    Dimensionless Ratio of the Density divided by the Critical
C           Density. (Input)
C    TAU    Dimensionless Ratio of the Critical Temperature divided by
C           the Temperature. (Input)
C    IDEL   Switch to choose density factors of the terms. (Input)
C    ITAU   Switch to choose temperature factors of the terms. (Input)
C    ICALC  Switch to control term summation.
C             If ICALC = 0, do not sum the terms of the function,
C             otherwise sum the terms of the function.
C    FUNC   Value of the function calculated.  (Output)
C
C  DESCRIPTION OF VARIABLES:
C    DALARRAY(16)...Array of calculated functions. Subscript is
C                   related to IDEL and ITAU by variable NCODE.
C    DALFLAGS.......A character*16 string used to determine if
C                   the function stored in DALARRAY can be used.
C                   If DALFLAGS(i:i) = 1, then DALARRAY(i) can
C                   be used instead of recalculating the function.
C    NCODE..........Integer that maps the IDEL, ITAU switches to a
C                   subscript of DALARRAY and DALFLAGS.
C                   NCODE = 4 * IDEL + ITAU + 1
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IDEL, ITAU, ICALC
      REAL*8 DEL, TAU, FUNC

C--Local Variables----------------------
      INTEGER K, NCODE
      REAL*8 LNDEL, LNTAU, C1, I, I2, I3, L, L2, L3
      REAL*8 A, B, C, D, E, K1, K2, K3 ,K4 ,K5

C-----------------------------------------------------------------------
C  EXPONENTIATE THE TERMS IN THE RESIDUAL HELMHOLTZ FUNCTION.
C-----------------------------------------------------------------------
      FUNC = 0
      ERRNUM = 0
      IF (IDGPRP) THEN
        NDX0 = -1
        RETURN
      ELSEIF (DEL.LE.0) THEN
        WRITE (*,*) ' *** FUNDEQ: DEL IS ZERO OR NEGATIVE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDEQ: DEL IS ZERO OR NEGATIVE ***'
        ERRNUM = 1
        RETURN
      ELSEIF (TAU.LE.0) THEN
        WRITE (*,*) ' *** FUNDEQ: TAU IS ZERO OR NEGATIVE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDEQ: TAU IS ZERO OR NEGATIVE ***'
        ERRNUM = 1
        RETURN
      ELSEIF (EOS(NDX).NE.1) THEN

C    ...Branch to FUNDCH4 if fluid is methane
        IF (EOS(NDX).EQ.3) THEN
          CALL FUNDCH4(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
          RETURN

C    ...Branch to FUNDNH3 if fluid is ammonia
        ELSEIF (EOS(NDX).EQ.4) THEN
          CALL FUNDNH3(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
          RETURN

C    ...Branch to FUNDMH if fluid is a martin hou type equation
        ELSEIF (EOS(NDX).EQ.2) THEN
          CALL FUNDMH(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
          RETURN

C    ...Branch to FUNDPR if fluid is a Peng-Robinson equation or
C      ...other cubic equation
        ELSEIF (EOS(NDX).GE.10) THEN
          CALL FUNDPR(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
          RETURN
        ENDIF

      ELSEIF (DEL.NE.DEL0 .OR. TAU.NE.TAU0 .OR. NDX.NE.NDX0) THEN
        DALFLAGS = '0000000000000000'
        LNDEL = DLOG(DEL)
        LNTAU = DLOG(TAU)
        DO 50 K = 1, NT(NDX)
          HXP(K) = 0
          IF (LEX(K, NDX).NE.0) HXP(K) = GAMMA(K, NDX)*DEL**LEX(K, NDX)
          FXP(K) = DEXP(IEX(K, NDX)*LNDEL + EXJ(K, NDX)*LNTAU + HXP(K))
   50   CONTINUE
        DEL0 = DEL
        TAU0 = TAU
        NDX0 = NDX
      ENDIF
C-----------------------------------------------------------------------
C  CALCULATE THE DENSITY FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
C  Test if function has already been calculated: if so, use it
      IF (ICALC.NE.0 .AND. IDEL.LE.3) THEN
        NCODE = IDEL*4 + ITAU + 1
        IF (DALFLAGS(NCODE:NCODE).EQ.'1') THEN
          FUNC = DALARRAY(NCODE)
          RETURN
        ENDIF
      ENDIF
      IF (IDEL.EQ.0) THEN
        DO 100 K = 1, NT(NDX)
  100   TERMS(K) = FXP(K)
      ELSEIF (IDEL.EQ.1) THEN
        DO 110 K = 1, NT(NDX)
  110   TERMS(K) = (IEX(K, NDX) + LEX(K, NDX)*HXP(K))*FXP(K)
      ELSEIF (IDEL.EQ.2) THEN
        DO 120 K = 1, NT(NDX)
        C1 = LEX(K,NDX)*HXP(K)
  120   TERMS(K) = FXP(K)*(IEX(K,NDX)*(IEX(K,NDX) - 1.D0)
     &           + C1*(LEX(K,NDX) - 1.D0 + 2.D0*IEX(K,NDX) + C1))
      ELSEIF (IDEL.EQ.3) THEN
        DO 130 K = 1, NT(NDX)
        C1 = LEX(K,NDX)*HXP(K)
  130   TERMS(K) = (IEX(K,NDX)*(IEX(K,NDX) - 1.D0)*(IEX(K,NDX) - 2.D0)
     &     + C1*(3.D0*IEX(K,NDX)*((IEX(K,NDX) - 2.D0) + LEX(K,NDX))
     &     + (LEX(K,NDX) - 1.D0)*(LEX(K,NDX) - 2.D0)
     &     + 3.D0*C1*(IEX(K,NDX)+(LEX(K,NDX) - 1.D0))
     &     + C1*C1))*FXP(K)

      ELSEIF (IDEL.EQ.4) THEN
        DO 140 K = 1, NT(NDX)
          C1 = -LEX(K,NDX)*HXP(K)
          I = IEX(K,NDX)
          I2 = I*I
          I3 = I2*I
          L = LEX(K,NDX)
          L2 = L*L
          L3 = L*L2
  140   TERMS(K) = FXP(K)
     &    * (I*(I - 1.D0)*(I - 2.D0)*(I - 3.D0)
     &    + C1*((-4.D0*I3 + 18.D0*I2 - 22.D0*I - 6.D0*I2*L
     &    +   18.D0*L*I - 4.D0*L2*I + 6.D0*L2 - 11.D0*L + 6.D0 - L3)
     &    + C1*((6.D0*I2 - 18.D0*I + 12.D0*I*L + 11.D0
     &    -   18.D0*L + 7.D0*L2)
     &    + C1*((-4.D0*I + 6.D0 - 6.D0*L)
     &    + C1))))

      ELSEIF (IDEL.EQ.5) THEN
        DO 150 K = 1, NT(NDX)
          C1 = -LEX(K,NDX)*HXP(K)
          I = IEX(K,NDX)
          I2 = I*I
          I3 = I2*I
          L = LEX(K,NDX)
          L2 = L*L
          L3 = L2*L

          A =  I*(I - 1.D0)*(I - 2.D0)*(I - 3.D0)
          B = -4.D0*I3 + 18.D0*I2 - 22.D0*I - 6.D0*I2*L
     &      + 18.D0*L*I - 4.D0*L2*I + 6.D0*L2 - 11.D0*L + 6.D0 - L3
          C = 6.D0*I2 - 18.D0*I + 12.D0*I*L + 11.D0
     &      - 18.D0*L + 7.D0*L2
          D = -4.D0*I + 6.D0 - 6.D0*L
          E = 1
  150   TERMS(K) = FXP(K)
     &    * (A*      (I - 4.D0          - C1)
     &    +  B*C1*   (I - 4.D0 +      L - C1)
     &    +  C*C1**2*(I - 4.D0 + 2.D0*L - C1)
     &    +  D*C1**3*(I - 4.D0 + 3.D0*L - C1)
     &    +  E*C1**4*(I - 4.D0 + 4.D0*L - C1))

      ELSEIF (IDEL.EQ.6) THEN
        DO 160 K = 1, NT(NDX)
          C1 = -LEX(K,NDX)*HXP(K)
          I = IEX(K,NDX)
          I2 = I*I
          I3 = I2*I
          L = LEX(K,NDX)
          L2 = L*L
          L3 = L2*L

          A =  I*(I - 1.D0)*(I - 2.D0)*(I - 3.D0)
          B = -4.D0*I3 + 18.D0*I2 - 22.D0*I - 6.D0*I2*L
     &      + 18.D0*L*I - 4.D0*L2*I + 6.D0*L2 - 11.D0*L + 6.D0 - L3
          C = 6.D0*I2 - 18.D0*I + 12.D0*I*L + 11.D0
     &      - 18.D0*L + 7.D0*L2
          D = -4.D0*I + 6.D0 - 6.D0*L
          E = 1
          K1 =  (I - 4.D0 - C1)
          K2 =  (I - 4.D0 +      L - C1)
          K3 =  (I - 4.D0 + 2.D0*L - C1)
          K4 =  (I - 4.D0 + 3.D0*L - C1)
          K5 =  (I - 4.D0 + 4.D0*L - C1)
  160   TERMS(K) = FXP(K)
     &    * (A*      (-C1 * K1 + (I - 4.D0         )
     &               * (I - 5.D0         ) - C1 * (I +      L - 5.D0))
     &    +  B*C1*   (-C1 * K2 + (I - 4.D0 +      L)
     &               * (I - 5.D0 +      L) - C1 * (I + 2.D0*L - 5.D0))
     &    +  C*C1**2*(-C1 * K3 + (I - 4.D0 + 2.D0*L)
     &               * (I - 5.D0 + 2.D0*L) - C1 * (I + 3.D0*L - 5.D0))
     &    +  D*C1**3*(-C1 * K4 + (I - 4.D0 + 3.D0*L)
     &               * (I - 5.D0 + 3.D0*L) - C1 * (I + 4.D0*L - 5.D0))
     &    +  E*C1**4*(-C1 * K5 + (I - 4.D0 + 4.D0*L)
     &               * (I - 5.D0 + 4.D0*L) - C1 * (I + 5.D0*L - 5.D0)))

      ELSE
        WRITE (*,*) ' *** FUNDEQ: IDEL IS OUT OF RANGE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDEQ: IDEL IS OUT OF RANGE ***'
        STOP
      ENDIF
C-----------------------------------------------------------------------
C  CALCULATE THE TEMPERATURE FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
      IF (ITAU.EQ.0) THEN
      ELSEIF (ITAU.EQ.1) THEN
        DO 210 K = 1, NT(NDX)
  210   TERMS(K) = EXJ(K, NDX)*TERMS(K)
      ELSEIF (ITAU.EQ.2) THEN
        DO 220 K = 1, NT(NDX)
  220   TERMS(K) = EXJ(K, NDX)*(EXJ(K, NDX) - 1.D0)*TERMS(K)
      ELSEIF (ITAU.EQ.3) THEN
        DO 230 K = 1, NT(NDX)
  230   TERMS(K) = EXJ(K,NDX)*(EXJ(K,NDX)-1.D0)*(EXJ(K,NDX) - 2.D0)
     &           * TERMS(K)
      ELSE
        WRITE (*,*) ' *** FUNDEQ: ITAU IS OUT OF RANGE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDEQ: ITAU IS OUT OF RANGE ***'
        STOP
      ENDIF
C-----------------------------------------------------------------------
C  SUM THE TERMS FOR THE CALCULATED FUNCTION.
C-----------------------------------------------------------------------
      FUNC = 0
      IF (ICALC.EQ.0) RETURN
      DO 300 K = 1, NT(NDX)
  300 FUNC = FUNC + COEF(K, NDX)*TERMS(K)

C  Update functions and flags
c     IF (IDEL.GE.4) RETURN
      DALARRAY(NCODE) = FUNC
      DALFLAGS(NCODE:NCODE) = '1'
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION PFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Pressure as a function of temperature and density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    PFNDA   Pressure in MPa (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, PFNDA

C--Local Variables----------------------
      REAL*8 DA10, DEL, TAU

C-----------------------------------------------------------------------
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      PFNDA = D*RGAS(NDX)*T*(DA10 + 1.D0)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DALP02(TAU)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the 2nd partial derivative of the dimensionless
C    ideal gas Helmholtz Energy with respect to the dimensionless
C    Temperature ratio, TAU.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    TAU     Dimensionless Ratio of the Critical Temperature divided
C            by the Temperature. (Input)
C    DALP02  2nd partial derivative of the Dimensionless Ideal Gas
C            Helmholtz Energy w.r.t. to TAU. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 TAU, DALP02

C--Functions Called---------------------
      REAL*8 IDGAS

C--Local Variables----------------------
      REAL*8 T

C-----------------------------------------------------------------------
      T = TC(NDX)/TAU
      DALP02 = (1.D0 - IDGAS(T, 0))/(TAU*TAU)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DALP01(TAU)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the first partial derivative of the dimensionless
C    ideal gas Helmholtz Energy with respect to the dimensionless
C    Temperature ratio, TAU.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    TAU     Dimensionless Ratio of the Critical Temperature divided
C            by the Temperature. (Input)
C    DALP01  1st partial derivative of the Dimensionless Ideal Gas
C            Helmholtz Energy w.r.t. to TAU. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 TAU, DALP01

C--Functions Called---------------------
      REAL*8 IDGAS

C--Local Variables----------------------
      REAL*8 T

C-----------------------------------------------------------------------
      IF (TAU.LT.0) THEN
        WRITE (*,*) ' *** DALP01: TAU IS LESS THAN ZERO ***'
        ERRNUM = 1
      ELSE
        T = TC(NDX)/TAU
        DALP01 = (H0(NDX) + IDGAS(T, 1) - T)/TC(NDX)
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION ALPHA0(DEL, TAU)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the dimensionless ideal gas Helmholtz Energy
C
C  DESCRIPTION OF ARGUMENTS:
C
C    DEL     Dimensionless Ratio of the Density divided by the
C            Critical Density. (Input)
C    TAU     Dimensionless Ratio of the Critical Temperature divided
C            by the Temperature. (Input)
C    ALPHA0  Dimensionless Ideal Gas Helmholtz Energy, A0/RT. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 DEL, TAU, ALPHA0

C--Functions Called---------------------
      REAL*8 IDGAS

C--Local Variables----------------------
      REAL*8 D, T

C-----------------------------------------------------------------------
      T = TC(NDX)/TAU
      D = DEL*DC(NDX)
      IF (D.LE.0 .OR. T.LE.0) THEN
        WRITE (*,*) ' *** ALPHA0: DEL OR TAU IS ZERO ***'
        ERRNUM = 1
      ELSE
        ALPHA0 = (H0(NDX) + IDGAS(T, 1))/T - (S0(NDX) + IDGAS(T, 2))
     &         + DLOG(D*RGAS(NDX)*T/P0(NDX)) + SMIX(NDX) - 1.D0
      ENDIF
      END
C
C
C=======================================================================
C=======================================================================
C
      FUNCTION AFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Helmholtz Energy as a function of temperature and density
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    AFNDA   Helmholtz Energy in J/mol (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, AFNDA

C--Functions Called---------------------
      REAL*8 ALPHA0

C--Local Variables----------------------
      REAL*8 ALPHA, DEL, R, TAU

C-----------------------------------------------------------------------
      R = RGAS(NDX)*1000.D0
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 0, 0, 1, ALPHA)
      AFNDA = R*T*(ALPHA0(DEL, TAU) + ALPHA)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION CVFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Isochoric Heat Capacity as a function of Temperature
C    Temperature and Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    CVFNDA  Isochoric Heat Capacity in J/mol-K (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, CVFNDA

C  DECLARE FUNCTIONS:
      REAL*8 DALP02

C--Local Variables----------------------
      REAL*8 DEL, R, TAU, DA02

C-----------------------------------------------------------------------
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      R = 1000.D0*RGAS(NDX)
      CALL FUNDEQ(DEL, TAU, 0, 2, 1, DA02)
      CVFNDA = -R*(TAU*TAU*DALP02(TAU) + DA02)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION CPFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Isobaric Heat Capacity as a function of Temperature
C    and Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    CPFNDA  Isobaric Heat Capacity in J/mol-K (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, CPFNDA

C--Functions Called---------------------
      REAL*8 CVFNDA

C--Local Variables----------------------
      REAL*8 DA10, DA20, DA11, DEL, FACT1, FACT2, R, TAU

C-----------------------------------------------------------------------
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      R = 1000.D0*RGAS(NDX)
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      CALL FUNDEQ(DEL, TAU, 2, 0, 1, DA20)
      CALL FUNDEQ(DEL, TAU, 1, 1, 1, DA11)
      FACT1 = (DA10 - DA11 + 1.D0)
      FACT2 = (DA20 + 2.D0*DA10 + 1.D0)
      CPFNDA = CVFNDA(D, T) + R*(FACT1*FACT1/FACT2)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DPDD1(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the first partial of Pressure with respect to Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3. (Input)
C    T       Temperature in Kelvins. (Input)
C    DPDD1   1st partial of Pressure w.r.t. Density in MPa-dm3/mol
C            (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, DPDD1

C--Local Variables----------------------
      REAL*8 DA10, DA20, DEL, TAU

C-----------------------------------------------------------------------
      TAU = TC(NDX)/T
      DEL = D/DC(NDX)
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      CALL FUNDEQ(DEL, TAU, 2, 0, 1, DA20)
      DPDD1 = RGAS(NDX)*T*(1.D0 + 2.D0*DA10 + DA20)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DPDD2(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the 2nd partial of Pressure with respect to Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3. (Input)
C    T       Temperature in Kelvins. (Input)
C    DPDD2   2nd partial of Pressure w.r.t. Density   (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, DPDD2

C--Local Variables----------------------
      REAL*8 DA10, DA20, DA30, DEL, TAU

C-----------------------------------------------------------------------
      TAU = TC(NDX)/T
      DEL = D/DC(NDX)
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      CALL FUNDEQ(DEL, TAU, 2, 0, 1, DA20)
      CALL FUNDEQ(DEL, TAU, 3, 0, 1, DA30)
      IF (D.EQ.0) THEN
        WRITE (*,*) ' *** DPDD2: DENSITY IS ZERO ***'
        ERRNUM = 1
      ELSEIF (DA30.EQ.0 .AND. TNAMEQ(NDX).EQ.'R-114') THEN
        DPDD2 = 0
      ELSE
        DPDD2 = T*RGAS(NDX)*(2.D0*DA10 + 4.D0*DA20 + DA30)/D
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION DPDT(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the first partial of Pressure with respect to Temperature.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3. (Input)
C    T       Temperature in Kelvins. (Input)
C    DPDT    1st partial of Pressure w.r.t. Temperature in MPa/K.
C            (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, DPDT

C--Local Variables----------------------
      REAL*8 DA10, DA11, DEL, TAU

C-----------------------------------------------------------------------
      TAU = TC(NDX)/T
      DEL = D/DC(NDX)
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      CALL FUNDEQ(DEL, TAU, 1, 1, 1, DA11)
      DPDT = RGAS(NDX)*D*(DA10 - DA11 + 1.D0)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION FUGCOE(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the fugacity pressure ratio at specified temperature
C    and molar density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    FUGCOE  Fugacity pressure ratio f/P (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, FUGCOE

C--Local Variables----------------------
      REAL*8 ALPHA, DA10, DEL, TAU

C-----------------------------------------------------------------------
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 0, 0, 1, ALPHA)
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      FUGCOE = DEXP(ALPHA + DA10)/(1.D0 + DA10)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION GFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Gibbs Energy as a function of temperature and density
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    GFNDA   Gibbs Energy in J/mol (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, GFNDA

C--Functions Called---------------------
      REAL*8 ALPHA0

C--Local Variables----------------------
      REAL*8 ALPHA, DA10, DEL, R, TAU

C-----------------------------------------------------------------------
      R = RGAS(NDX)*1000.D0
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 0, 0, 1, ALPHA)
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      GFNDA = R*T*(ALPHA0(DEL,TAU) + ALPHA + DA10 + 1.D0)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION HFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Enthalpy as a function of Temperature and Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    HFNDA   Enthalpy in J/mol (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, HFNDA

C--Functions Called---------------------
      REAL*8 DALP01

C--Local Variables----------------------
      REAL*8 DA01, DA10, DEL, R, TAU

C-----------------------------------------------------------------------
      R = RGAS(NDX)*1000.D0
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 0, 1, 1, DA01)
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      HFNDA = R*T*(TAU*DALP01(TAU) + DA01 + DA10 + 1.D0)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION SFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Entropy as a function of temperature and density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    SFNDA   Entropy in J/mol-K (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, SFNDA

C  DECLARE FUNCTIONS:
      REAL*8 ALPHA0, DALP01

C--Local Variables----------------------
      REAL*8 DA01, ALPHA, DEL, R, TAU

C-----------------------------------------------------------------------
      R = RGAS(NDX)*1000.D0
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 0, 1, 1, DA01)
      CALL FUNDEQ(DEL, TAU, 0, 0, 1, ALPHA)
      SFNDA = R*(TAU*DALP01(TAU) + DA01 - ALPHA0(DEL, TAU) - ALPHA)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION UFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Internal Energy as a function of Temperature and Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    UFNDA   Internal Energy in J/mol (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, UFNDA

C--Functions Called---------------------
      REAL*8 DALP01

C--Local Variables----------------------
      REAL*8 DA01, DEL, R, TAU

C-----------------------------------------------------------------------
      R = RGAS(NDX)*1000.D0
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 0, 1, 1, DA01)
      UFNDA = R*T*(TAU*DALP01(TAU) + DA01)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION VIR2(T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    This routine evaluates the 2nd virial coefficient from the
C    fundamental equation of state.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    T    - Temperature of the fluid in Kelvins (Input).
C    VIR2 - Value of the 2nd virial coefficient in dm3/mol. (Output).
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 T, VIR2

C--Local Variables----------------------
      REAL*8 D, DEL, TAU, FUNC

C-----------------------------------------------------------------------
      D=.000000001D0
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, FUNC)
      VIR2 = FUNC/D
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION VIR3(T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    This routine evaluates the 3rd virial coefficient from the
C    fundamental equation of state.
C
C  DESCRIPTION OF THE ARGUMENTS:
C
C    T    - Temperature of the fluid in Kelvins (Input).
C    VIR3 - Value of the 3rd virial coefficient in (dm3/mol)**2 (Output).
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 T, VIR3

C--Local Variables----------------------
      REAL*8 D, DEL, TAU, FUNC

C-----------------------------------------------------------------------
      D=.000000001D0
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 2, 0, 1, FUNC)
      VIR3 = FUNC/D**2
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION WFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Velocity of Sound as a function of Temperature
C    and Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3. (Input)
C    T       Temperature in Kelvins. (Input)
C    WFNDA   Velocity of Sound in m/s. (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, WFNDA, WFNDAX

C--Functions Called---------------------
      REAL*8 CPFNDA, CVFNDA, DPDD1

C--Local Variables----------------------
      REAL*8 GAM

C-----------------------------------------------------------------------
      GAM = CPFNDA(D, T)/CVFNDA(D, T)
      WFNDAX = (1.0D6*DPDD1(D, T)*GAM/AMOL(NDX))
      IF (WFNDAX.LT.0) THEN
        WFNDA = 0
        WRITE (*,*) ' WFNDA: NEGATIVE ARGUMENT.'
        IF (IOUT.NE.0) WRITE (IOUT,*) ' WFNDA: NEGATIVE ARGUMENT.'
      ELSE
        WFNDA = SQRT(WFNDAX)
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION ZFNDA(D, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate Compressibility as a function of Temperature and Density.
C
C  DESCRIPTION OF ARGUMENTS:
C
C    D       Density in mol/dm3 (Input)
C    T       Temperature in Kelvins (Input)
C    ZFNDA   Compressibility Factor (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, ZFNDA

C--Local Variables----------------------
      REAL*8 DA10, DEL, TAU

C-----------------------------------------------------------------------
      DEL = D/DC(NDX)
      TAU = TC(NDX)/T
      CALL FUNDEQ(DEL, TAU, 1, 0, 1, DA10)
      ZFNDA = DA10 + 1.D0
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE PROPERTY(D,T,JT,K,KT,BETA,BulkModS,BS,KAPPA,BulkModT)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the following properties given density and temperature:
C
C    JT        Joule-Thomson Coefficient (K/MPa)
C    K         Isentropic Expansion Coefficient (Dimensionless)
C    KT        Isothermal Expansion Coefficient (Dimensionless)
C    BETA      Volume Expansivity (1/K)
C    BulkModS  Adiabatic Compressibility (1/MPa)
C    BS        Adiabatic Bulk Modulus (MPa)
C    KAPPA     Isothermal Compressibility (1/MPa)
C    BulkModT  Isothermal Bulk Modulus (MPa)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      REAL*8 D, T, JT, K, KT, BETA, BulkModS, BS, KAPPA, BulkModT

C--Local Variables----------------------
      REAL*8 P, W, DPDDX, DPDTX, CP

C--Functions Called---------------------
      REAL*8 PFNDA, WFNDA, DPDD1, DPDT, CPFNDA

      P = PFNDA(D, T)
      W = WFNDA(D, T)
      DPDDX = DPDD1(D, T)
      DPDTX = DPDT(D, T)
      CP = CPFNDA(D, T)

C  ...Calculate the Isentropic Expansion Coefficient
      K = W**2/(P/D*1.D6/AMOL(NDX))

C  ...Calculate the Isothermal Expansion Coefficient
      KT = D/P*DPDDX

C  ...Calculate the Volume Expansivity
      BETA = DPDTX/DPDDX/D

C  ...Calculate the Adiabatic Compressibility
      BulkModS = 0
      IF (K*P.NE.0) BulkModS = 1/(K*P)

C  ...Calculate the Adiabatic Bulk Modulus
      BS = K*P

C  ...Calculate the Isothermal Compressibility
      KAPPA = 1/(KT*P)

C  ...Calculate the Isothermal Bulk Modulus
      BulkModT = KT*P

C  ...Calculate the Joule-Thomson Coefficient
      JT = (T*BETA - 1)/D/CP*1000.D0

      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION CSAT(T, IPHASE)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C    Calculate the Heat Capacity at Saturation.
C
C  DESCRIPTION OF ARGUMENTS:
C    T       Temperature in Kelvins (Input)
C    IPHASE  Switch to select liquid or vapor phase. (Input)
C    CSAT    Heat Capacity at saturation in J/mol-K (Output)
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'

C--Arguments----------------------------
      INTEGER IPHASE
      REAL*8 T, CSAT

C--Functions Called---------------------
      REAL*8 CPFNDA, DPDT, DPDD1

C--Local Variables----------------------
      REAL*8 DL, DV, PL, PV, BETA

C-----------------------------------------------------------------------
C    BETA is the Volume Expansivity

      CSAT = 0
      IF (T.LT.TTRP(NDX)) RETURN
      IF (IPHASE.EQ.1) THEN
        CALL SATT(T, 0, DL, DV, PL, PV)
        BETA = DPDT(DL, T)/DPDD1(DL, T)/DL
        CSAT = CPFNDA(DL, T) - BETA/DL*(HLQ - HVQ)/(1/DL - 1/DV)
C       WRITE (*,*) BETA,1/DL,(HLQ - HVQ)/(1/DL - 1/DV)
C       WRITE (*,*) CPFNDA(DL, T),BETA/DL*(HLQ - HVQ)/(1/DL - 1/DV)
      ELSEIF (IPHASE.EQ.2) THEN
        CALL SATT(T, 0, DL, DV, PL, PV)
        BETA = DPDT(DV, T)/DPDD1(DV, T)/DV
        CSAT = CPFNDA(DV, T) - BETA/DV*(HLQ - HVQ)/(1/DL - 1/DV)
C       WRITE (*,*) BETA,1/DV,(HLQ - HVQ)/(1/DL - 1/DV)
C       WRITE (*,*) CPFNDA(DV, T),BETA/DV*(HLQ - HVQ)/(1/DL - 1/DV)
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE FUNDNH3(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C     Uses the EOS for ammonia cited below. FUNDEQ calls FUNDNH3 because
C     the form of the equation is non-standard.  FUNDNH3 provides the same
C     functions as FUNDEQ, and its calling arguments are identical.
C
C     Haar, L. and Gallagher, J.S., "Thermodynamic Properties of
C     Ammonia," J. Phys. Chem. Ref. Data, Vol. 7, No. 3, 635-792 (1978).
C
C     The new variables are:
C       DELNH3 - Reduced density with the proper units for the
C                ammonia equation.
C       TAUNH3 - Reduced temperature variable:  500/T - 500/Tc
C
C     Special care had to be taken when TAUNH3 was negative.
C     Since a negative number cannot be raised to an exponent in
C     FORTRAN, the SGN variable was set up to keep track of the sign.
C     The NT+1 position (where NT is the nubmer of terms) in the arrays
C     was used to include the log variable shown in the equation for ammonia.
C     This term dropped out in the temperature derivatives, and is
C     accounted for separately in the density derivatives.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IDEL, ITAU, ICALC
      REAL*8 DEL, TAU, FUNC

C--Local Variables----------------------
      INTEGER K, NCODE
      REAL*8 DELNH3, TAUNH3, SGNX

C-----------------------------------------------------------------------
C  EXPONENTIATE THE TERMS IN THE RESIDUAL HELMHOLTZ FUNCTION.
C-----------------------------------------------------------------------
      FUNC = 0
      ERRNUM = 0
      IF (DEL.NE.DEL0 .OR. TAU.NE.TAU0 .OR. NDX.NE.NDX0) THEN
        DALFLAGS = '0000000000000000'
        DELNH3 = DEL*DC(NDX)*AMOL(NDX)/1000.D0
        TAUNH3 = TC(NDX)/TAU
        TAUNH3 = 500.D0/TAUNH3 - 1.2333498D0
        SGNX = 1.D0
        SGNX = SIGN(SGNX, TAUNH3)
        DO 50 K = 1, NT(NDX)
          HXP(K) = 0
          FXP(K) = DELNH3**IEX(K, NDX)*DABS(TAUNH3)**EXJ(K, NDX)
          IF (DMOD(EXJ(K, NDX),2.D0).EQ.1) FXP(K) = SGNX*FXP(K)
   50   CONTINUE
        HXP(NT(NDX)+1) = 0
        FXP(NT(NDX)+1) = DLOG(DELNH3)
        TERMS(NT(NDX)+1) = 0
        DEL0 = DEL
        TAU0 = TAU
        NDX0 = NDX
      ENDIF
C-----------------------------------------------------------------------
C  CALCULATE THE DENSITY FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
C  Test if function has already been calculated: if so, use it
      IF (ICALC.NE.0) THEN
        NCODE = IDEL*4 + ITAU + 1
        IF (DALFLAGS(NCODE:NCODE).EQ.'1') THEN
          FUNC = DALARRAY(NCODE)
          RETURN
        ENDIF
      ENDIF
      IF (IDEL.EQ.0) THEN
        DO 100 K = 1, NT(NDX) + 1
  100   TERMS(K) = FXP(K)
      ELSEIF (IDEL.EQ.1) THEN
        DO 110 K = 1, NT(NDX)
  110   TERMS(K) = IEX(K, NDX) * FXP(K)
        TERMS(NT(NDX) + 1) = 1
      ELSEIF (IDEL.EQ.2) THEN
        DO 120 K = 1, NT(NDX)
  120   TERMS(K) = FXP(K) * IEX(K,NDX) * (IEX(K,NDX)-1.D0)
        TERMS(NT(NDX) + 1) = -1
      ELSEIF (IDEL.EQ.3) THEN
        DO 130 K = 1, NT(NDX)
  130   TERMS(K)=FXP(K)*IEX(K,NDX)*(IEX(K,NDX)-1.D0)*(IEX(K,NDX)-2.D0)
        TERMS(NT(NDX) + 1) = 2
      ELSE
        WRITE (*,*) ' *** FUNDEQ: IDEL IS OUT OF RANGE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDEQ: IDEL IS OUT OF RANGE ***'
        STOP
      ENDIF
C-----------------------------------------------------------------------
C  CALCULATE THE TEMPERATURE FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
      IF (ITAU.EQ.0) THEN
      ELSEIF (ITAU.EQ.1) THEN
        DO 210 K = 1, NT(NDX)
  210   TERMS(K) = EXJ(K, NDX) * TERMS(K)/
     &             (1.D0 - 1.2333498D0/500.D0/TAU*TC(NDX))
      ELSEIF (ITAU.EQ.2) THEN
        DO 220 K = 1, NT(NDX)
  220   TERMS(K) = EXJ(K, NDX) * (EXJ(K, NDX) - 1.D0) * TERMS(K) /
     &             (1.D0 - 1.2333498D0/500.D0/TAU*TC(NDX))**2
      ELSEIF (ITAU.EQ.3) THEN
        DO 230 K = 1, NT(NDX)
  230   TERMS(K) = EXJ(K, NDX) * (EXJ(K,NDX) - 1.D0) *
     &             (EXJ(K, NDX) - 2.D0) * TERMS(K) /
     &             (1.D0 - 1.2333498D0/500.D0/TAU*TC(NDX))**3
      ELSE
        WRITE (*,*) ' *** FUNDEQ: ITAU IS OUT OF RANGE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDEQ: ITAU IS OUT OF RANGE ***'
        STOP
      ENDIF
C-----------------------------------------------------------------------
C  SUM THE TERMS FOR THE CALCULATED FUNCTION.
C-----------------------------------------------------------------------
      FUNC = 0
      IF (ICALC.EQ.0) RETURN
      DO 300 K = 1, NT(NDX) + 1
  300 FUNC = FUNC + COEF(K, NDX)*TERMS(K)

C  Update functions and flags
      DALARRAY(NCODE) = FUNC
      DALFLAGS(NCODE:NCODE) = '1'
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE FUNDMH(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the terms of APLHA-BAR and its partial derivatives with
C    respect to the dimensionless density, DEL, and dimensionless
C    Temperature, TAU for the Martin Hou equation.
C    Description of arguments is identical to FUNDEQ.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IDEL, ITAU, ICALC
      REAL*8 DEL, TAU, FUNC

C--Local Variables----------------------
      REAL*8 X2, X3, X4, X5, X6, X7, X8, X0
      REAL*8 GAM, UV, DUV, DUV2, RT, DRT
      REAL*8 T, D, V, R, VB, DADD, DADD2, DADV2, DADV3

C-----------------------------------------------------------------------
C  SETUP THE MARTIN-HOU TERMS IN THE EQUATION OF STATE.
C-----------------------------------------------------------------------
      ICALC = ICALC
      FUNC = 0
      ERRNUM = 0
      T = TC(NDX)/TAU
      D = DEL*DC(NDX)
      D = D*DCONV
      V = 1/D
      VB = V - BMH
      R = RGAS(NDX)/DCONV
      GAM = DEXP(-KMH/TAU)
      X0 = A0(1) + B0(1)*T + C0(1)*GAM
      X2 = A0(2) + B0(2)*T + C0(2)*GAM
      X3 = A0(3) + B0(3)*T + C0(3)*GAM
      X4 = A0(4) + B0(4)*T + C0(4)*GAM
      X5 = A0(5) + B0(5)*T + C0(5)*GAM
      X6 = A0(6) + B0(6)*T + C0(6)*GAM
      X7 = A0(7) + B0(7)*T + C0(7)*GAM
      X8 = A0(8) + B0(8)*T + C0(8)*GAM

      RT = R*T
      DRT = D*RT
      UV = UMH*V
      DUV = 0
      DUV2 = 0
      IF (UV.LT.250) THEN
        DUV = DEXP(UV)
        DUV2 = DEXP(2.D0*UV)
      ENDIF
      DEL0 = DEL
      IF (UV.EQ.0) UV = 1.D8
      TAU0 = TAU
      NDX0 = NDX
C-----------------------------------------------------------------------
C  CALCULATE THE DENSITY FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
      IF (IDEL.EQ.0 .AND. ITAU.EQ.0) THEN
        FUNC = -DLOG(VB) + DLOG(V)
     &       + (X2/VB + X3/(2*VB**2) + X4/(3*VB**3) + X5/(4*VB**4)
     &       + X6/(5*VB**5) + X7/(6*VB**6) + X8/(7*VB**7))/RT
        IF (UV.LT.250)
     &    FUNC = FUNC - X0/(UMH*DUV)*(1.D0 + 2.D0*CPRIME)/RT
      ELSEIF (IDEL.EQ.1 .AND. ITAU.EQ.1) THEN
        GAM = DEXP(-KMH/TAU)*(1 + KMH/TAU)
        X0 = A0(1) + C0(1)*GAM
        X2 = A0(2) + C0(2)*GAM
        X3 = A0(3) + C0(3)*GAM
        X4 = A0(4) + C0(4)*GAM
        X5 = A0(5) + C0(5)*GAM
        X6 = A0(6) + C0(6)*GAM
        X7 = A0(7) + C0(7)*GAM
        X8 = A0(8) + C0(8)*GAM
        FUNC = (X2/VB**2 + X3/VB**3 + X4/VB**4 + X5/VB**5
     &                   + X6/VB**6 + X7/VB**7 + X8/VB**8)/DRT
        IF (UV.LT.250)
     &    FUNC = FUNC + X0/(DUV*(1.D0 + 2.D0*CPRIME))/DRT
      ELSEIF (IDEL.EQ.1) THEN
        FUNC = RT/VB + X2/VB**2 + X3/VB**3 + X4/VB**4 + X5/VB**5
     &                          + X6/VB**6 + X7/VB**7 + X8/VB**8
        IF (UV.LT.250) FUNC = FUNC + X0/DUV
C    &    FUNC = FUNC + X0/DUV + X0/(CPRIME*DUV2)
        FUNC = FUNC/DRT - 1.D0
      ELSEIF (IDEL.EQ.2) THEN
        DADV2 = -1/V**2 + 1/VB**2 + (2*X2/VB**3 + 3*X3/VB**4
     &        + 4*X4/VB**5 + 5*X5/VB**6 + 6*X6/VB**7
     &        + 7*X7/VB**8 + 8*X8/VB**9)/RT
        DADD  = RT/VB + X2/VB**2 + X3/VB**3 + X4/VB**4 + X5/VB**5
     &                           + X6/VB**6 + X7/VB**7 + X8/VB**8
        IF (UV.LT.250) THEN
C         DADV2 = DADV2 + (UMH*X0/DUV + 2.D0*UMH*X0/(CPRIME*DUV2))/RT
          DADV2 = DADV2 + (UMH*X0/DUV)/RT
C         DADD  = DADD + X0/DUV + X0/(CPRIME*DUV2)
          DADD  = DADD + X0/DUV
        ENDIF
        DADD = DADD/DRT - 1.D0
        FUNC = DADV2/D**2 - 2.D0*DADD
      ELSEIF (IDEL.EQ.3) THEN
        IF (TNAMEQ(NDX).EQ.'R-114') THEN
          FUNC = 0
          RETURN
        ENDIF
        DADV2 = -1/V**2 + 1/VB**2 + (2*X2/VB**3 + 3*X3/VB**4
     &        + 4*X4/VB**5 + 5*X5/VB**6
     &        + 6*X6/VB**7 + 7*X7/VB**8 + 8*X8/VB**9)/RT
        DADD  = RT/VB + X2/VB**2 + X3/VB**3 + X4/VB**4 + X5/VB**5
     &                           + X6/VB**6 + X7/VB**7 + X8/VB**8
        DADV3 = 2.D0/V**3 - 2.D0/VB**3 - (6.D0*X2/VB**4 + 12.D0*X3/VB**5
     &        + 20.D0*X4/VB**6 + 30.D0*X5/VB**7
     &        + 20.D0*X6/VB**8 + 30.D0*X7/VB**9 + 30.D0*X8/VB**10)/RT
        IF (UV.LT.250) THEN
C         DADV2 = DADV2 + (UMH*X0/DUV + 2.D0*UMH*X0/(CPRIME*DUV2))/RT
C         DADD  = DADD + X0/DUV + X0/(CPRIME*DUV2)
C         DADV3 = DADV3 +(UMH**2*X0/DUV+4.D0*UMH**2*X0/(CPRIME*DUV2))/RT
          DADV2 = DADV2 + (UMH*X0/DUV)/RT
          DADD  = DADD + X0/DUV
          DADV3 = DADV3 +(UMH**2*X0/DUV)/RT
        ENDIF
        DADD  = DADD/DRT - 1.D0
        DADD2 = DADV2/D**2 - 2.D0*DADD
        FUNC  = -1.D0/D**3*DADV3 - 6.D0*DADD2 - 6.D0*DADD
C-----------------------------------------------------------------------
C  CALCULATE THE TEMPERATURE FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
      ELSEIF (ITAU.EQ.1) THEN
        GAM = DEXP(-KMH/TAU)*(1 + KMH/TAU)
        X0 = A0(1) + C0(1)*GAM
        X2 = A0(2) + C0(2)*GAM
        X3 = A0(3) + C0(3)*GAM
        X4 = A0(4) + C0(4)*GAM
        X5 = A0(5) + C0(5)*GAM
        X6 = A0(6) + C0(6)*GAM
        X7 = A0(7) + C0(7)*GAM
        X8 = A0(8) + C0(8)*GAM
        FUNC = (X2/VB+X3/(2*VB**2) + X4/(3*VB**3) + X5/(4*VB**4)
     &               +X6/(5*VB**5) + X7/(6*VB**6) + X8/(7*VB**7))/RT
        IF (UV.LT.250)
     &    FUNC = FUNC - X0/(UMH*DUV*(1.D0 + 2.D0*CPRIME))/RT
      ELSEIF (ITAU.EQ.2) THEN
        GAM = -KMH/TAU
        GAM = DEXP(GAM)*(KMH**2/TAU**3)
        X0 = C0(1)*GAM
        X2 = C0(2)*GAM
        X3 = C0(3)*GAM
        X4 = C0(4)*GAM
        X5 = C0(5)*GAM
        X6 = C0(6)*GAM
        X7 = C0(7)*GAM
        X8 = C0(8)*GAM
        FUNC = (X2/VB + X3/(2*VB**2) + X4/(3*VB**3) + X5/(4*VB**4)
     &                + X6/(5*VB**5) + X7/(6*VB**6) + X8/(7*VB**7))/
     &         (R*TC(NDX))*TAU**2
        IF (UV.LT.250)
     &  FUNC = FUNC - X0/(UMH*DUV*(1.D0 + 2.D0*CPRIME))/
     &         (R*TC(NDX))*TAU**2
      ELSE
        WRITE (*,*) ' *** FUNDMH: IDEL OR ITAU IS OUT OF RANGE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDMH: IDEL OR ITAU IS OUT OF RANGE ***'
        STOP
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE FUNDPR(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Calculate the terms of APLHA-BAR and its partial derivatives with
C    respect to the dimensionless density, DEL, and dimensionless
C    Temperature, TAU for the Martin Hou equation.
C    Description of arguments is identical to FUNDEQ.
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IDEL, ITAU, ICALC
      REAL*8 DEL, TAU, FUNC

C--Local Variables----------------------
      REAL*8 RT, T, D, V, R, VB, K, ATC, AWTR
      REAL*8 W, U, B, A, X, Q, V1, V2
      REAL*8 DADV2, DADD

C-----------------------------------------------------------------------
C  SETUP THE PENG-ROBINSON TERMS IN THE EQUATION OF STATE.
C-----------------------------------------------------------------------
      ICALC = ICALC
      FUNC = 0
      ERRNUM = 0
      T = TC(NDX)/TAU
      D = DEL*DC(NDX)
      V = 1/D
      R = RGAS(NDX)
      RT = R*T

C.....Equation of state type 10 is the Peng-Robinson equation
      IF (EOS(NDX).EQ.10) THEN
        U = 2
        W = -1
        B = 0.07780D0*R*TC(NDX)/PC(NDX)
        K = 0.37464D0 + 1.54226D0*AF(NDX) - 0.26992D0*AF(NDX)**2
        ATC = 0.45724D0*R**2*TC(NDX)**2/PC(NDX)
C.....Equation of state type 11 is the SRK equation
      ELSEIF (EOS(NDX).EQ.11) THEN
        U = 1
        W = 0
        B = 0.08664D0*R*TC(NDX)/PC(NDX)
        K = 0.480D0 + 1.574*AF(NDX) - 0.176D0*AF(NDX)**2
        ATC = 0.42748D0*R**2*TC(NDX)**2/PC(NDX)
      ENDIF

      AWTR = (1 + K*(1 - DSQRT(1/TAU)))**2
      A = ATC*AWTR
      VB = V - B

      Q = DSQRT((U*B)**2 - 4*W*B**2)
      V1 = 2*V + U*B + Q
      V2 = 2*V + U*B - Q
      X = V2/V1
C-----------------------------------------------------------------------
C  CALCULATE THE DENSITY FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------

      IF (IDEL.EQ.0 .AND. ITAU.EQ.0) THEN
        FUNC = DLOG(V) - DLOG(VB) + A/RT*DLOG(X)/Q
      ELSEIF (IDEL.EQ.1 .AND. ITAU.EQ.1) THEN
        FUNC = -A/(D*RT)/(V**2 + U*B*V + W*B**2)
        FUNC = -ATC/(D*R)/(V**2 + U*B*V + W*B**2)/TC(NDX)
        FUNC = TAU*FUNC*(AWTR + K*((1 + K)/DSQRT(TAU) - K/TAU))
      ELSEIF (IDEL.EQ.1) THEN
C    ...Peng Robinson Equation
        FUNC = V/VB - A/(D*RT)/(V**2 + U*B*V + W*B**2) - 1
C    ...First derivative of alpha (numerically equal to PR equation)
C       FUNC = V/VB - A/(D*RT)*2*(1/(2*V+U*B-Q)-1/(2*V+U*B+Q))/Q - 1
      ELSEIF (IDEL.EQ.2) THEN
        DADV2 = -1/V**2 + 1/VB**2 + A/(RT*Q)*(4/V1**2 - 4/V2**2)
        DADD = V/VB - A/(D*RT)/(V**2 + U*B*V + W*B**2) - 1
        FUNC = V**2*DADV2 - 2*DADD
      ELSEIF (IDEL.EQ.3) THEN
C-----------------------------------------------------------------------
C  CALCULATE THE TEMPERATURE FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
      ELSEIF (ITAU.EQ.1) THEN
        FUNC = ATC/R*DLOG(X)/Q/TC(NDX)
        FUNC = TAU*FUNC*(AWTR + K*((1 + K)/DSQRT(TAU) - K/TAU))
      ELSEIF (ITAU.EQ.2) THEN
        FUNC = ATC/R*DLOG(X)/Q/TC(NDX)
        FUNC = TAU**2*FUNC*
     &         (K*DSQRT(AWTR/TAU**3) + K*(K/TAU**2-(1+K)/2/TAU**1.5D0))
      ELSE
        WRITE (*,*) ' *** FUNDPR: IDEL OR ITAU IS OUT OF RANGE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDPR: IDEL OR ITAU IS OUT OF RANGE ***'
        STOP
      ENDIF
      END
C
C=======================================================================
C=======================================================================
C
      SUBROUTINE FUNDCH4(DEL, TAU, IDEL, ITAU, ICALC, FUNC)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    Uses the EOS for methane cited below. FUNDEQ calls FUNDCH4 because
C    the form of the equation is non-standard.  The equation has 36 regular
C    terms and 4 "special" terms (37:40).  FUNDCH4 provides the same
C    functions as FUNDEQ, and its calling arguments are identical.
C
C    The variables defined below are new variables or variables
C    that have different meanings than in FUNDEQ.  Note that the
C    the derivatives are written in terms of these new variables.
C    Variables not defined below have the same meaning as in FUNDEQ.
C
C      Setzmann, U. and Wagner, W., "A New Equation of State and Tables
C      of Thermodynamic Properties for Methane Covering the Range from
C      the Melting Line to 625 K at Pressures up to 1000 MPa," Institut
C      fur Thermo- und Fluiddynamik, Ruhr-Univeristat Bochum
C      D-4630 Bochum, Federal Republic of Germany
C      Accepted for publication in J. Phys. Chem. Ref. Data, 20, (1991).

C  DESCRIPTION OF VARIABLES
C    ALPHAQ()....Array of tau offset values for special terms.
C    BETAQ().....Array of del offset values for special terms.
C    SIGMAQ()....Array of tau multipliers for special terms.
C    HXP().......GAMMA(K,NDX) * DELB(K) ** LEX(K,NDX)
C    FXP().......DEXP(IEX(K, NDX)*LNDEL+EXJ(K, NDX)*LNTAU +
C                     HXP(K) + KXP(K) )
C
C                FOR REGULAR TERMS    FOR SPECIAL TERMS
C    TAUA()......      TAU                (TAU - ALPHAQ)
C    DELB()......      DEL                (DEL -BETAQ)
C    KXP().......      0.D0          SIGMAQ(K) * TAUA(K) ** LEX(K,NDX)
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comstate.for'

C--Arguments----------------------------
      INTEGER IDEL, ITAU, ICALC
      REAL*8 DEL, TAU, FUNC

C--Local Variables----------------------
      INTEGER K, NCODE, I
      REAL*8 LNDEL, LNTAU, C1, C2

C-----------------------------------------------------------------------
C  EXPONENTIATE THE TERMS IN THE RESIDUAL HELMHOLTZ FUNCTION.
C-----------------------------------------------------------------------
      FUNC = 0
      ERRNUM = 0
      IF (DEL.NE.DEL0 .OR. TAU.NE.TAU0 .OR. NDX.NE.NDX0) THEN
        DALFLAGS = '0000000000000000'
        LNDEL = DLOG(DEL)
        LNTAU = DLOG(TAU)
C-----------------------------------------------------------------------
C  Calculate common terms for regular formulation.
C-----------------------------------------------------------------------
        I = 0
        DO 50 K = 1, NT(NDX)
          DELB(K) = DEL
          TAUA(K) = TAU
          KXP(K)  = 0
          HXP(K)  = GAMMA(K,NDX)*DELB(K)**LEX(K,NDX)
C-----------------------------------------------------------------------
C  Calculate common terms for special formulation.  When tau = alpha or
C  del = beta, ensure there won't be a divide by zero in the derivatives.
C-----------------------------------------------------------------------
          IF (GAMMA(K,NDX).EQ.-20 .OR. GAMMA(K,NDX).EQ.-40) THEN
            I = I + 1
            DELB(K) =  DEL - BETAQ(I)
            IF (DELB(K).EQ.0) THEN
              HXP(K) = 0
              DELB(K) = 1
            ELSE
              HXP(K) = GAMMA(K, NDX)*DELB(K)**LEX(K, NDX)
            ENDIF
            TAUA(K) = TAU - ALPHAQ(I)
            IF (TAUA(K).EQ.0) THEN
              KXP(K) = 0
              TAUA(K) = 1
            ELSE
              KXP(K) = SIGMAQ(I)*TAUA(K)**LEX(K, NDX)
            ENDIF
          ENDIF
          FXP(K) = DEXP(IEX(K, NDX)*LNDEL+EXJ(K, NDX)*LNTAU + HXP(K)
     &           + KXP(K))
   50   CONTINUE
        DEL0 = DEL
        TAU0 = TAU
        NDX0 = NDX
      ENDIF
C-----------------------------------------------------------------------
C  CALCULATE THE DENSITY FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
C  Test if function has already been calculated: if so, use it
      IF (ICALC.NE.0) THEN
        NCODE = IDEL*4 + ITAU + 1
        IF (DALFLAGS(NCODE:NCODE).EQ.'1') THEN
          FUNC = DALARRAY(NCODE)
          RETURN
        ENDIF
      ENDIF
      IF (IDEL.EQ.0) THEN
        DO 100 K = 1, NT(NDX)
  100   TERMS(K) = FXP(K)
      ELSEIF (IDEL.EQ.1) THEN
        DO 110 K = 1, NT(NDX)
  110   TERMS(K) = (IEX(K, NDX) + LEX(K, NDX)*DEL * HXP(K)/DELB(K)) *
     &              FXP(K)
      ELSEIF (IDEL.EQ.2) THEN
        DO 120 K = 1, NT(NDX)
  120   TERMS(K) = FXP(K) * (-IEX(K,NDX) + DEL**2 * (IEX(K,NDX)/
     &             DEL + LEX(K,NDX)*HXP(K)/DELB(K))**2 + DEL**2
     &             *LEX(K,NDX)*(LEX(K,NDX)-1.D0)*HXP(K)/DELB(K)**2)
      ELSEIF (IDEL.EQ.3) THEN
        DO 130 K = 1, NT(NDX)
        C1 = LEX(K,NDX)*HXP(K)/DELB(K) + IEX(K,NDX)/DEL
        C2 =-IEX(K,NDX)/DEL**2 + LEX(K,NDX)*(LEX(K,NDX) - 1.D0)
     &     * HXP(K)/DELB(K)**2
  130   TERMS(K) = DEL**3*FXP(K)*(C1**3 + C1*C2 + 2.D0*C1*C2
     &           + 2.D0*IEX(K,NDX)/DEL**3
     &           + LEX(K,NDX)*(LEX(K,NDX) - 1.D0)*(LEX(K,NDX)-2.D0)
     &           * HXP(K)/DELB(K)**3)
      ELSE
        WRITE (*,*) ' *** FUNDCH4: IDEL IS OUT OF RANGE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDCH4: IDEL IS OUT OF RANGE ***'
        STOP
      ENDIF
C-----------------------------------------------------------------------
C  CALCULATE THE TEMPERATURE FACTORS FOR THE TERMS IN THE SUMMATION.
C-----------------------------------------------------------------------
      IF (ITAU.EQ.0) THEN
      ELSEIF (ITAU.EQ.1) THEN
        DO 210 K = 1, NT(NDX)
  210   TERMS(K) = (EXJ(K, NDX) + TAU*LEX(K,NDX)*KXP(K)/TAUA(K))*
     &              TERMS(K)
      ELSEIF (ITAU.EQ.2) THEN
        DO 220 K = 1, NT(NDX)
  220   TERMS(K) = (-EXJ(K, NDX) + TAU**2*
     &             (EXJ(K,NDX)/TAU + LEX(K, NDX)*KXP(K)/TAUA(K))**2
     &           + TAU**2*LEX(K,NDX)*(LEX(K,NDX) - 1.D0)
     &           * KXP(K)/TAUA(K)**2)*TERMS(K)
      ELSE
        WRITE (*,*) ' *** FUNDCH4: ITAU IS OUT OF RANGE ***'
        IF (IOUT.NE.0)
     &  WRITE (IOUT,*) ' *** FUNDCH4: ITAU IS OUT OF RANGE ***'
        STOP
      ENDIF
C-----------------------------------------------------------------------
C  SUM THE TERMS FOR THE CALCULATED FUNCTION.
C-----------------------------------------------------------------------
      FUNC = 0
      IF (ICALC.EQ.0) RETURN
      DO 300 K = 1, NT(NDX)
  300 FUNC = FUNC + COEF(K, NDX)*TERMS(K)

C  Update functions and flags
      DALARRAY(NCODE) = FUNC
      DALFLAGS(NCODE:NCODE) = '1'
      END
