C
C=======================================================================
C=======================================================================
C
      SUBROUTINE TABLES
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
C
C  PURPOSE:
C
C    To construct a table of properties.
C
C-----------------------------------------------------------------------
      INCLUDE 'commons.for'
      INCLUDE 'comdrv.for'
      INTEGER QUALITYFLAG
      REAL*8 DSL, DSV, Q
      COMMON /QUALITY/ DSL, DSV, Q, QUALITYFLAG

C--Functions Called---------------------
      REAL*8 P2PEQ
      REAL*8 TOFDP, TOFP2P, TOFD2P, DOFST, DOFPT, TOFDS, TOFDH, DOFHT
      REAL*8 PFNDA, HFNDA, SFNDA, DHIGH2

C--Local Variables----------------------
      LOGICAL ERRORX, MULT, FLAG
      CHARACTER TYP*4, PH*10
      INTEGER LIQ, SOL, IPHASE, GRAPHERFLAG
      REAL*8 T, TL, TV, TD3, TD4, T1, T2, TINC, TT1, TSTART, TSAT
      REAL*8 D, DL, DV, D1, D2, D3, D4
      REAL*8 P, PS, PL, PV, P1, P2, P3, P4, PINC, PSTART
      REAL*8 H, HHIGH, S, DUMMY
      CHARACTER*10 TAGB, TAGL, TAGV, TAGS

C--Initialize Variables and Write a Heading to the Output Device--------
      TAGB = '          '
      TAGL = ' Liquid   '
      TAGV = ' Vapor    '
      TAGS = ' Solid    '
      MULT = .FALSE.
      GRAPHERFLAG=0
 10   CALL SYSTEM("CLS")
      IF (PRNTFLG) WRITE (*,1000)
      IF (.NOT.IDGPRP .AND. PRNTFLG) WRITE (*,1010)
      IF (PRNTFLG) WRITE (*,1020)
      READ (*,'(A4)') TYP
      PH = ' '
      DL = 0.D0
      DV = 0.D0
      PL = 0.D0
      PV = 1.D8
      TL = 0.D0
      TV = 0.D0

C-----------------------------------------------------------------------
C  Calculate Isobaric Properties on Temperature Increments
C-----------------------------------------------------------------------
      IF (TYP.EQ.'1' .OR. TYP.EQ.'PT' .OR. TYP.EQ.'pt') THEN
        IPHASE = 0
        CALL INPUT1(P, 1, ERRORX)
        IF (ERRORX) GOTO 10
        CALL INPUT2(T, T2, TINC, 2, ERRORX, MULT)
        IF (ERRORX) GOTO 10
        TSTART = TLOW(NDX)
        IF (T.GT.TSTART) TSTART = T
        IF (IDGPRP) GOTO 105

        IF (P.GE.PSATL(NDX)) THEN
          IF (P.LT.PC(NDX)) CALL SATP(P, 0, DL, DV, TL, TV)
          IF (DHIGH(NDX).LT.DSATL(NDX)) THEN

C      .....Determine starting location on dome.
            IF (P.LT.PDH1(NDX)) THEN
              IF (TSTART.LT.TV) THEN
                TSTART = TV
                GOTO 100
              ENDIF

C      .....Determine starting location on DHIGH line
            ELSE
              TT1 = TOFDP(DHIGH(NDX), P)
              IF (TSTART.LT.TT1) TSTART = TT1
            ENDIF
          ELSE

C      .....Find melting line pressure.
            T1 = TOFP2P(P, 3)

C      .....Print intersection of melting line with selected isobar.
            IF (TSTART.LT.T1) THEN
              TSTART = T1
              D = DOFPT(P, TSTART, 1)
              IF (D.GT.DHIGH(NDX)) THEN
                D = DHIGH(NDX)
                T1 = TOFDP(D, P)
                TSTART = T1
                CALL PROPS(P, D, TSTART, TAGB, 1, 0, 0)
              ELSE
                CALL PROPS(P, D, TSTART, TAGS, 1, 0, 0)
              ENDIF
              IF (T2.LT.T1) GOTO 130
              GOTO 100
            ENDIF

C      .....Find intersection of DHIGH and selected isobar.
            ERRMSG = .FALSE.
            T1 = TOFDP(DHIGH2(P, 0), P)
            ERRMSG = .TRUE.
            IF (TSTART.LT.T1) THEN
              TSTART = T1
              CALL PROPS(P, DHIGH(NDX), TSTART, TAGB, 1, 0, 0)
              IF (T2.LT.T1) GOTO 130
              GOTO 100
            ENDIF
          ENDIF
        ELSEIF (P.GE.PSATV(NDX)) THEN
C    .....For mixtures, trap region Pv(@Ttrp) and Pl(@Ttrp)
C      ...Do not let the isobar go into the wet region
          TSTART = TOFP2P(P, 2)
        ENDIF
 105    D = DOFPT(P, TSTART, 0)
        CALL PROPS(P, D, TSTART, TAGB, 1, 0, 0)
 100    T = T + TINC
        IF (T.LE.TSTART) GOTO 100

 110    IF (T - TINC.LT.TL .AND. T.GE.TL) THEN
C    .....Call SATP to recalculate U, H, and S
          CALL SATP(P, 0, DL, DV, TL, TV)
          CALL PROPS(P, DL, TL, TAGL, 1, 0, 1)
 120      IF (T.LT.TV) THEN
            T = T + TINC
            GOTO 120
          ENDIF
          CALL PROPS(P, DV, TV, TAGV, 1, 0, 1)
          IF (TV.GT.T2) GOTO 130
          IPHASE = 2
        ENDIF
        IF (T.GT.T2) T = T2
        D = DOFPT(P, T, IPHASE)
        IF (IPHASE.EQ.0) THEN
          IPHASE = 2
          IF (D.GT.DC(NDX)) IPHASE = 1
        ENDIF
        CALL PROPS(P, D, T, TAGB, 1, 0, 0)
        IF (T2-T.GT.0.5D-9) THEN
          T = T + TINC
          IF (T.GT.T2) T = T2
          GOTO 110
        ENDIF
 130    IF (INTERQ) CALL PAUSE2

C-----------------------------------------------------------------------
C  Calculate Isothermal Properties on Pressure Increments
C-----------------------------------------------------------------------
      ELSEIF (TYP.EQ.'2'.OR.TYP.EQ.'TP'.OR.TYP.EQ.'tp') THEN
        IPHASE = 0
        CALL INPUT1(T, 2, ERRORX)
        IF (ERRORX) GOTO 10
        CALL INPUT2(P, P2, PINC, 1, ERRORX, MULT)
        IF (ERRORX) GOTO 10
        IF (T.LT.TC(NDX).AND..NOT.IDGPRP) CALL SATT(T,0, DL, DV, PL, PV)
        FLAG = .FALSE.
        IF (P.LT.PL) FLAG = .TRUE.

C  .....Determine stopping location on Solid line
        SOL = 0
        PS = P2PEQ(T, 3)
        IF (P2.GT.PS) THEN
          P2 = PS
          SOL = 1
        ENDIF

C  .....Determine stopping location on Dhigh
        IF(T.GT.TDH1(NDX) .AND. T.LT.TDH2(NDX) .AND. T.GT.TSLD(NDX))THEN
          PS = PFNDA(DHIGH(NDX), T)
          IF (P2.GT.PS) P2 = PS
        ENDIF

C  .....Determine starting location on Plow
        IF (P.LT.PLOW(NDX)) THEN
          D = DOFPT(PLOW(NDX), T, 2)
          CALL PROPS(PLOW(NDX), D, T, TAGB, 1, 0, 0)
          P = PINC
          IF (MULT) P = P*PFACT
          IF (P.GT.P2) P = P2
        ENDIF

 210    IF (P.GE.PV .AND. FLAG) THEN
          FLAG = .FALSE.
C    .....Call SATT to recalculate U, H, and S
          CALL SATT(T, 0, DL, DV, PL, PV)
          CALL PROPS(PV, DV, T, TAGV, 1, 0, 1)
 220      IF (P.LT.PL) THEN
            IF (MULT) THEN
              P = P * PINC
            ELSE
              P = P + PINC
            ENDIF
            GOTO 220
          ENDIF
          CALL PROPS(PL, DL, T, TAGL, 1, 0, 1)
          IPHASE = 1
          IF (P.GT.P2) P = P2

C    .....Quit when density is greater than DHIGH.
C      ...Also check that T>TDH1 for cases where the density ancillary
C      ...and the EOS do not match at DHIGH.
          IF (DL.GT.DHIGH(NDX) .OR. T.LT.TDH1(NDX)) GOTO 230
        ENDIF
        IF (P.GT.P2) P = P2
        D = DOFPT(P, T, IPHASE)
        IF (IPHASE.EQ.0) THEN
          IPHASE = 2
          IF (D.GT.DC(NDX)) IPHASE = 1
        ENDIF
        IF (D.GT.DHIGH(NDX)) THEN
          D = DHIGH(NDX)
          P = 0
        ENDIF
        PH = TAGB
        IF (P.EQ.PS .AND. SOL.EQ.1) PH = TAGS
        CALL PROPS(P, D, T, PH, 1, 0, 0)
        IF (P2-P.GT.0.5D-9 .AND. D.LT.DHIGH(NDX)) THEN
          IF (MULT) THEN
            P = P * PINC
          ELSE
            P = P + PINC
          ENDIF
          IF (P.GT.P2) P = P2
          IF (P.LT.PLOW(NDX)) GOTO 230
          GOTO 210
        ENDIF
 230    IF (INTERQ) CALL PAUSE2

C-----------------------------------------------------------------------
C  Calculate Isochoric Properties on Temperature Increments
C-----------------------------------------------------------------------
      ELSEIF (TYP.EQ.'3'.OR.TYP.EQ.'DT'.OR.TYP.EQ.'dt') THEN
        CALL INPUT1(D, 3, ERRORX)
        IF (ERRORX) THEN
          IF (THIGH(NDX).LT.TC(NDX)) THEN
            CALL SATT(THIGH(NDX), 0, DL, DV, PL, PV)
            IF (D.GT.DL) GOTO 10
          ELSE
            GOTO 10
          ENDIF
        ENDIF
        CALL INPUT2(T, T2, TINC, 2, ERRORX, MULT)
        IF (ERRORX) GOTO 10
C  .....Find where isochore crosses PHIGH line.
        IF (D.GT.DHH(NDX)) THEN
          TT1 = TOFDP(D, PHIGH(NDX))
          IF (T2.GT.TT1) T2 = TT1
        ENDIF

        TSTART = T
        IF (TSTART.LT.TLOW(NDX)) TSTART = TLOW(NDX)
        PH = TAGB
        IF (.NOT. IDGPRP) THEN
C    .....Calculate two phase properties
          IF (D.LE.DSATL(NDX) .AND. D.GE.DSATV(NDX)) THEN
            CALL SATD(D, 0, D2, P, TSAT)
            IF (TSAT.GT.THIGH(NDX)) TSAT = THIGH(NDX)
            IF (TSTART.LE.TSAT) THEN
C        .....Calculate quality
              CALL SATT(TSTART, 0, DSL, DSV, PL, PV)
              IF (PL.EQ.PV .AND. TSTART.NE.TSAT) THEN
                Q = (1.D0/D - 1.D0/DSL)/(1.D0/DSV - 1.D0/DSL)
                CALL PROPS(PV, D, TSTART, PH, 1, 1, 1)
 330            T = T + TINC
                IF (T.LE.TSTART) GOTO 330
 320            IF (T.LE.TSAT) THEN
                  CALL SATT(T, 0, DSL, DSV, PL, PV)
                  Q = (1.D0/D - 1.D0/DSL)/(1.D0/DSV - 1.D0/DSL)
                  CALL PROPS(PV, D, T, PH, 1, 1, 1)
                  T = T + TINC
                  GOTO 320
                ENDIF
              ELSE
 360            T = T + TINC
                IF (T.LE.TSAT) GOTO 360
              ENDIF
              IF (T.GE.THIGH(NDX)) GOTO 350
C        .....Print saturation property
              PH = TAGV
              IF (D.GT.DC(NDX)) PH = TAGL
C        .....Call SATD to recalculate U, H, and S
              CALL SATD(D, 0, D2, P, TSAT)
              CALL PROPS(P, D, TSAT, PH, 1, 0, 1)
C        .....The density can be greater than DHIGH for saturation properties
              IF (D.GT.DHIGH(NDX)) GOTO 350
              IF (T.GT.T2) GOTO 350
              GOTO 300
            ENDIF
          ELSEIF (D.GT.DSLDL(NDX)) THEN
C      .....Determine where isochore meets solid line
            TT1 = TOFD2P(D, 3)
            IF (TSTART.LT.TT1) THEN
              TSTART = TT1
              PH = TAGS
            ENDIF
          ENDIF
        ENDIF

        IF (TSTART.GT.T2) TSTART = T2
        CALL PROPS(0.D0, D, TSTART, PH, 1, 0, 0)
 310    T = T + TINC
        IF (T.LE.TSTART) GOTO 310
        IF (T - T2.GT.-0.5D-9) T = T2

 300    CALL PROPS(0.D0, D, T, TAGB, 1, 0, 0)
        IF (T2-T.GT.0.5D-9) THEN
          T = T + TINC
          IF (T - T2.GT.-0.5D-9) T = T2
          GOTO 300
        ENDIF
 350    IF (INTERQ) CALL PAUSE2

C-----------------------------------------------------------------------
C  Calculate Isenthalpic Properties on Pressure Increments
C-----------------------------------------------------------------------
      ELSEIF (TYP.EQ.'4'.OR.TYP.EQ.'HP'.OR.TYP.EQ.'hp') THEN
        IF (PRNTFLG) WRITE (*,'(''  Input Enthalpy, '',A10,//)') ALPHH
        READ (*,*,ERR=10) H
        H = H*HFACT
        HHIGH = HFNDA(DLH(NDX), THIGH(NDX))
        IF (H.LT.HMIN(NDX) .OR. H.GT.HHIGH) THEN
          WRITE (*,*) ' *** ENTHALPY OUT OF BOUNDS ***'
          CALL PAUSE2
          GOTO 10
        ENDIF

        CALL INPUT2(P, P2, PINC, 1, ERRORX, MULT)
        IF (ERRORX) GOTO 10
        LIQ = 0
        SOL = 0
        PH = TAGB

        IF (P.LT.PLOW(NDX)) THEN
          P = PINC
          IF (MULT) P = P*PFACT
          IF (P.GT.P2) P = P2
        ENDIF
        IF (.NOT.IDGPRP) THEN
          IPHASE = 0
          CALL SATH(H, D4, D3, P4, P3, TD4, TD3, IPHASE)
          IF (TD4.EQ.THIGH(NDX)) GOTO 460

C  .....Enthalpy ends on solid line.
          IF(H.LT.HSLDH(NDX) .AND. SOLIDLINE(NDX)) THEN
            CALL SATH(H, D, DUMMY, PS, DUMMY, T, DUMMY, 3)
            IF (P2.GT.PS) THEN
              P2 = PS
              SOL = 1
            ENDIF

C    .....Enthalpy ends on DHIGH line.
          ELSEIF (H.GT.HDH1(NDX) .AND. H.LE.HDH2(NDX)) THEN
            T = TOFDH(DHIGH(NDX), H)
            PS = PFNDA(DHIGH(NDX), T)
            IF (P2.GT.PS) P2 = PS

C    .....Enthalpy ends of PHIGH line.
          ELSEIF (H.GT.HDH2(NDX) .AND. H.LE.HHH(NDX)) THEN

C    .....Enthalpy ends on THIGH line.
          ELSEIF (H.GT.HHH(NDX)) THEN
            D = DOFHT(H, THIGH(NDX))
            PS = PFNDA(D, THIGH(NDX))
            IF (P2.GT.PS) P2 = PS

C    .....Enthalpy ends on Saturation line when D>DHIGH.
          ELSEIF (.NOT.SOLIDLINE(NDX)
     &            .AND. H.GT.HSATL(NDX) .AND. H.LE.HDH1(NDX)) THEN
            IF (P2.GT.P4) P2 = P4
          ELSE
            WRITE (*,*) ' *** TABLES: ERROR IN CONSTANT H *** '
          ENDIF

C    .....Check saturation boundaries
          IF (IPHASE.NE.0 .AND. D3.EQ.0) THEN
            LIQ = 1
            GOTO 450
          ELSEIF (IPHASE.NE.0 .AND. D3.NE.0) THEN
            LIQ = 1
            IF (P2.LT.P4 .AND. P2.GT.P3) P2 = P4
            IF (P .LT.P4 .AND. P .GT.P3) GOTO 450
          ENDIF
        ELSE
C    .....Find ideal gas limits.
          D = 0.1D0
          IF (H.LT.HFNDA(D,TLOW(NDX)) .OR. H.GT.HFNDA(D,THIGH(NDX)))THEN
            WRITE (*,*) ' *** ENTHALPY OUT OF BOUNDS ***'
            CALL PAUSE2
            GOTO 10
          ENDIF
        ENDIF

C  .....Print results.
        IF (P.GT.P2) GOTO 450
 410    CALL DTOFHP(H, P, D, T)
        IF (D.GT.DHIGH(NDX)) THEN
          P = 0
          D = DHIGH(NDX)
          T = TOFDH(D, H)
        ENDIF
        CALL PROPS(P, D, T, PH, 1, 0, 0)
 420    IF (P2-P.LT.0.5D-9 .OR. P.EQ.0) GOTO 460
        IF (MULT) THEN
          P = P * PINC
        ELSE
          P = P + PINC
        ENDIF
 450    IF (P.GT.P3 .AND. LIQ.EQ.1) THEN

C    .....Calculate saturation properties.
C      ...Recall SATT to find U, H, and S.
          LIQ = 0
          IF (D3.NE.0) THEN
            CALL SATT(TD3, 0, DL, DV, PL, PV)
            CALL PROPS(P3, D3, TD3, TAGV, 1, 0, 1)
          ENDIF
          CALL SATT(TD4, 0, DL, DV, PL, PV)
          IF (IPHASE.EQ.1) CALL PROPS(P4, D4, TD4, TAGL, 1, 0, 1)
          IF (IPHASE.EQ.2) CALL PROPS(P4, D4, TD4, TAGV, 1, 0, 1)
 430      IF (P.LT.P4) THEN
            IF (MULT) THEN
              P = P * PINC
            ELSE
              P = P + PINC
            ENDIF
            GOTO 430
          ENDIF
          IF (P.GT.P2) GOTO 460
          IF (P.LT.PLOW(NDX)) GOTO 460
        ENDIF
        IF (P.GT.P2) THEN
          P = P2
          IF (SOL.EQ.1) PH = TAGS
        ENDIF
        GOTO 410
 460    IF (INTERQ) CALL PAUSE2

C-----------------------------------------------------------------------
C  Calculate Isentropic Properties on Temperature Increments
C-----------------------------------------------------------------------
      ELSEIF (TYP.EQ.'5'.OR.TYP.EQ.'ST'.OR.TYP.EQ.'st') THEN
        IF (PRNTFLG) WRITE (*,'(''  Input Entropy, '',A10,//)') ALPHS
        READ (*,*,ERR=10) S
        S = S*SFACT

        IF (S.GT.SFNDA(DLH(NDX), THIGH(NDX)) .OR. S.LT.SMIN(NDX)) THEN
          WRITE (*,*) ' *** ENTROPY OUT OF BOUNDS ***'
          CALL PAUSE2
          GOTO 10
        ENDIF
        CALL INPUT2(T1, T2, TINC, 2, ERRORX, MULT)
        IF (ERRORX) GOTO 10

        SOL = 0
        TSTART = T1
        T = T1
        D1 = 0
        TT1 = 0
        IF (TSTART.LT.TLOW(NDX)) TSTART = TLOW(NDX)
        IF (S.GT.SSATL(NDX) .AND. S.LT.SSATV(NDX)) THEN
          IPHASE = 0
          CALL SATS(S, D1, TT1, IPHASE)
        ENDIF
        IF (.NOT.IDGPRP) THEN
          IF (S.LT.SHH(NDX)) THEN
C      .....Find entropy limit on Phigh line.
            IF (S.GT.SDH2(NDX)) THEN
              CALL DTOFPS(PHIGH(NDX), S, D, T)
              IF (T2.GT.T) T2 = T

C      .....Find entropy limit on Dhigh line.
            ELSEIF (S.GT.SDH1(NDX) .AND. S.GT.SSLDH(NDX)) THEN
              T = TOFDS(DHIGH(NDX), S)
              IF (T2.GT.T) T2 = T

C      .....Find solid entropy limit.
            ELSEIF (SOLIDLINE(NDX)) THEN
              IPHASE = 3
              CALL SATS(S, D, T, IPHASE)
              IF (T2.GT.T) THEN
                T2 = T
                SOL = 1
              ENDIF

C      .....Entropy ends on Dhigh line
            ELSEIF (D1.LT.DHIGH(NDX) .AND. D1.NE.0) THEN
              T = TOFDS(DHIGH(NDX), S)
              IF (T2.GT.T) T2 = T

C      .....Entropy ends on saturation line
            ELSE
              IF (T2.GT.TT1) T2 = TT1
            ENDIF
          ENDIF

          T = T1
          PH = TAGB
          IF (S.GT.SFNDA(DLL(NDX),TLOW(NDX))) THEN

C      .....Entropy is beyond intersection of Tlow and Plow.
C        ...Table starts at Plow.
            CALL DTOFPS(PLOW(NDX), S, D, TSTART)
            IF (TSTART.LT.T1) TSTART = T1

          ELSEIF (S.GT.SSATV(NDX)) THEN
C      .....Entropy is between dome and Plow.  Leave TSTART at TLOW

          ELSEIF (S.LT.SSATL(NDX)) THEN
C      .....Entropy is between dome and solid line.  Leave TSTART at TLOW

          ELSE
C      .....Find intersection of S with dome.
            IF (TT1.GT.THIGH(NDX)) TT1 = THIGH(NDX)
            IF (TT1.GT.TDH1(NDX).AND.DHIGH(NDX).LT.DC(NDX))TT1=TDH1(NDX)
            IF (TSTART.LT.TT1) THEN

C        .....Calculate two-phase properties.
              CALL SATT(TSTART, 0, DSL, DSV, PL, PV)
              IF (PL.EQ.PV) THEN
                Q = (S - SLQ)/(SVQ - SLQ)
                CALL PROPS(PV, 0.D0, TSTART, PH, 1, 1, 1)
 535            T = T + TINC
                IF (T.LE.TSTART) GOTO 535
 520            IF (T.LE.TT1) THEN
                  CALL SATT(T, 0, DSL, DSV, PL, PV)
                  Q = (S - SLQ)/(SVQ - SLQ)
                  CALL PROPS(PV, 0.D0, T, PH, 1, 1, 1)
                  T = T + TINC
                  GOTO 520
                ENDIF
              ELSE
 536            T = T + TINC
                IF (T.LE.TT1) GOTO 536
              ENDIF

C        .....Print saturation property.
C        .....Call SATT to get saturation pressure for Subroutine PROPS
C          ...and to calculate U, H, and S.
              CALL SATT(TT1, 0, DL, DV, PL, PV)
              IF (D1.GT.DC(NDX)) THEN
                CALL PROPS(PL, D1, TT1, TAGL, 1, 0, 1)
              ELSE
                CALL PROPS(PV, D1, TT1, TAGV, 1, 0, 1)
              ENDIF
              IF (TT1.EQ.T2) GOTO 550
              IF (T.GT.T2) T = T2
              PH = TAGB
              GOTO 510
            ENDIF
          ENDIF
        ENDIF

        IF (TSTART.GT.T2) GOTO 550
        D = DOFST(S, TSTART)
        CALL PROPS(0.D0, D, TSTART, TAGB, 1, 0, 0)
 500    IF(T.LE.TSTART) THEN
          T = T + TINC
          IF (T.GT.T2) GOTO 550
          GOTO 500
        ENDIF

 510    D = DOFST(S, T)
        CALL PROPS(0.D0, D, T, PH, 1, 0, 0)
        IF (T2-T.GT.0.5D-9) THEN
          T = T + TINC
          IF (T.GT.T2) THEN
            T = T2
            IF (SOL.EQ.1) PH = TAGS
          ENDIF
          GOTO 510
        ENDIF
 550    IF (INTERQ) CALL PAUSE2

C-----------------------------------------------------------------------
C  Calculate Saturation Properties on Temperature Increments
C-----------------------------------------------------------------------
      ELSEIF (TYP.EQ.'6'.OR.TYP.EQ.'TSAT'.OR.TYP.EQ.'tsat') THEN
        CALL INPUT2(T1, T2, TINC, 2, ERRORX, MULT)
        IF (ERRORX) GOTO 10
        IF (T2.GT.TC(NDX)) T2 = TC(NDX)
        IF (T1.GT.TC(NDX)) THEN
          T1 = TLOW(NDX)
          TINC = 10000.D0
        ENDIF
        IF (DHIGH(NDX).LT.DC(NDX) .AND. T2.GT.TDH1(NDX)) T2 = TDH1(NDX)
        T = T1
        IF (T1.LE.TLOW(NDX)) T = TLOW(NDX)
 600    CALL SATT(T, 0, DL, DV, PL, PV)
        CALL PROPS(PL, DL, T, TAGL, 1, 0, 1)
        CALL PROPS(PV, DV, T, TAGV, 1, 0, 1)
        IF (T2-T.GT.0.5D-9) THEN
          T = T + TINC
 610      IF (T1.LE.TLOW(NDX)) THEN
            T1 = T1 + TINC
            T = T1
            GOTO 610
          ENDIF
          IF (T.GT.T2) T = T2
          GOTO 600
        ENDIF
        IF (INTERQ) CALL PAUSE2

C-----------------------------------------------------------------------
C  Calculate Saturation Properties on Pressure Increments
C-----------------------------------------------------------------------
      ELSEIF (TYP.EQ.'7'.OR.TYP.EQ.'PSAT'.OR.TYP.EQ.'psat') THEN
C  .....Change the .FALSE. to MULT when multiplication factors are added.
        CALL INPUT2(P1, P2, PINC, 1, ERRORX, MULT)
        IF (ERRORX) GOTO 10
        IF (P2.GT.PC(NDX)) P2 = PC(NDX)
        IF (P1.GT.PC(NDX)) THEN
          P1 = PSATL(NDX)
          PINC = 10000.D0
        ENDIF
        IF (DHIGH(NDX).LT.DC(NDX) .AND. P2.GT.PDH1(NDX)) P2 = PDH1(NDX)
        P = P1
        IF (P1.LE.PSATL(NDX)) THEN
          P = PSATL(NDX)
          IF (MULT) P1 = PINC*PFACT
        ENDIF
 700    CALL SATP(P, 0, DL, DV, TL, TV)
        CALL PROPS(P, DL, TL, TAGL, 1, 0, 1)
        CALL PROPS(P, DV, TV, TAGV, 1, 0, 1)
        IF (P2-P.GT.0.5D-9) THEN
          IF (MULT) THEN
            P = P * PINC
          ELSE
            P = P + PINC
          ENDIF
 710      IF (P1.LE.PSATL(NDX)) THEN
            IF (MULT) THEN
              P1 = P1 * PINC
            ELSE
              P1 = P1 + PINC
            ENDIF
            P = P1
            GOTO 710
          ENDIF
          IF (P.GT.P2) P = P2
          GOTO 700
        ENDIF
        IF (INTERQ) CALL PAUSE2

C-----------------------------------------------------------------------
C  Calculate Melting Properties on Temperature Increments
C-----------------------------------------------------------------------
      ELSEIF (TYP.EQ.'8'.OR.TYP.EQ.'MELT'.OR.TYP.EQ.'melt') THEN
        IF (.NOT.SOLIDLINE(NDX)) THEN
          WRITE (*,*) ' *** THIS FLUID HAS NO MELTING LINE EQUATION ***'
          CALL PAUSE2
          GOTO 10
        ENDIF
        CALL INPUT2(T, T2, TINC, 2, ERRORX, MULT)
        IF (ERRORX) GOTO 10
        IF (T2.GT.TSLD(NDX)) T2 = TSLD(NDX)
        IF (T2.LT.TLOW(NDX) .OR. T.GT.T2) THEN
          WRITE (*,*) ' *** TEMPERATURE OUT OF BOUNDS *** '
          CALL PAUSE2
          GOTO 10
        ENDIF
        TSTART = T
        IF (TSTART.LT.TLOW(NDX)) TSTART = TLOW(NDX)
 800    P = P2PEQ(TSTART, 3)
        D = DOFPT(P, TSTART, 1)
        CALL PROPS(P, D, TSTART, TAGS, 1, 0, 0)
 820    T = T + TINC
        IF (T.LE.TSTART) GOTO 820
        IF (T.GT.T2) GOTO 830

 810    P = P2PEQ(T, 3)
        D = DOFPT(P, T, 1)
        IF (D.GT.DHIGH(NDX)) THEN
          P = 0
          D = DHIGH(NDX)
          T = TOFD2P(D, 3)
        ENDIF
        CALL PROPS(P, D, T, TAGS, 1, 0, 0)
        IF (T2-T.GT.0.5D-9 .AND. P.NE.0) THEN
          T = T + TINC
          IF (T.GT.T2) T = T2
          GOTO 810
        ENDIF
 830    IF (INTERQ) CALL PAUSE2

C-----------------------------------------------------------------------
C  Calculate Lines of Constant Quality
C-----------------------------------------------------------------------
      ELSEIF (TYP.EQ.'9'.OR.TYP.EQ.'Q'.OR.TYP.EQ.'q') THEN
        IF (PRNTFLG) WRITE (*,1040)
        READ (*,*,ERR=10) Q
        IF (Q.GT.1.D0 .OR. Q.LT.0.D0) THEN
          WRITE (*,*) ' *** QUALITY OUT OF BOUNDS ***'
          CALL PAUSE2
          GOTO 10
        ENDIF
        CALL INPUT2(P, P2, PINC, 1, ERRORX, MULT)
        IF (ERRORX) GOTO 10
        IF (P2.GT.PC(NDX)) P2 = PC(NDX)
        PSTART = P
        IF (PSTART.LT.PSATL(NDX)) PSTART = PSATL(NDX)
        IF (MULT .AND. P.LT.PSATL(NDX)) P = PSATL(NDX)
        IF (DHIGH(NDX).LT.DC(NDX) .AND. P2.GT.PDH1(NDX)) P2 = PDH1(NDX)
        CALL SATP(PSTART, 0, DSL, DSV, TL, TV)
        IF (TL.EQ.TV) CALL PROPS(PSTART, D, TV, TAGB, 1, 1, 1)
 920    IF (MULT) THEN
          P = P * PINC
        ELSE
          P = P + PINC
        ENDIF
        IF (P.LE.PSTART) GOTO 920

 900    CALL SATP(P, 0, DSL, DSV, TL, TV)
        IF (TL.EQ.TV) CALL PROPS(P, D, TV, TAGB, 1, 1, 1)
        IF (P2-P.GT.0.5D-9) THEN
          IF (MULT) THEN
            P = P * PINC
          ELSE
            P = P + PINC
          ENDIF
          IF (P.GT.P2) P = P2
          GOTO 900
        ENDIF
        IF (INTERQ) CALL PAUSE2

C--View the Current Program and Fluid Status----------------------------
      ELSEIF (TYP(1:1).EQ.'S'.OR.TYP(1:1).EQ.'s') THEN
        CALL STATUS

C--Toggle the quality flag----------------------------------------------
      ELSEIF (TYP(1:1).EQ.'N'.OR.TYP(1:1).EQ.'n') THEN
        QUALITYFLAG = 1 - QUALITYFLAG

C--Toggle the flag the determines if PINC is added or multiplied--------
      ELSEIF (TYP(1:1).EQ.'M'.OR.TYP(1:1).EQ.'m') THEN
        IF (MULT) THEN
          MULT = .FALSE.
        ELSE
          MULT = .TRUE.
        ENDIF

C--Change the ouput file------------------------------------------------
      ELSEIF (TYP(1:1).EQ.'C'.OR.TYP(1:1).EQ.'c') THEN
        WRITE (*,'('' Enter the filespec for your output file.'',//)')
        READ (*,'(A80)') OUTOPT
        IF (OUTOPT .EQ. ' ') OUTOPT = 'nul'
        OPEN (IOUT, FILE = OUTOPT, STATUS = 'UNKNOWN')

C--Toggle the Grapher flag----------------------------------------------
C  This flag puts a large number between constant properties lines so
C  that programs like Grapher can data clip between lines.
      ELSEIF (TYP(1:2).EQ.'GR'.OR.TYP(1:2).EQ.'gr') THEN
        GRAPHERFLAG = 1 - GRAPHERFLAG

C--Return to the Main Menu----------------------------------------------
      ELSEIF (TYP.EQ.'0' .OR. TYP.EQ.'QUIT' .OR. TYP.EQ.'quit') THEN
        RETURN
      ENDIF
      IF (GRAPHERFLAG.EQ.1) THEN
        WRITE (IOUT,*) '0.000001 -100000 -100000 -100000 -100000 ',
     &                          '-100000 -100000 -100000'
      ENDIF
      GOTO 10
C-----------------------------------------------------------------------
 1000 FORMAT ('  TABLES MENU (Select an option):',//,
     &'  (1) PT     (Isobaric properties on temperature increments)',/,
     &'  (2) TP     (Isothermal properties on pressure increments)',/,
     &'  (3) DT     (Isochoric properties on temperature increments)',/,
     &'  (4) HP     (Isenthalpic properties on pressure increments)',/,
     &'  (5) ST     (Isentropic properties on temperature increments)')
 1010 FORMAT (
     &'  (6) TSAT   (Saturation properties on temperature increments)',/
     &'  (7) PSAT   (Saturation properties on pressure increments)',/,
     &'  (8) MELT   (Melting properties on temperature increments)',/,
     &'  (9) Q      (Constant quality properties)')
 1020 FORMAT(
     &'  (S) Status (View program status)',/,
     &'  (N) Turn off printing the value of the quality',/,
     &'  (M) Multiply Pinc rather than add',/,
     &'  (C) Change output file name',/,
     &'  (0) Quit   (Return to the main menu)',//)
 1040 FORMAT ('  Enter Quality (limits: 0 to 1)',//)
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION VISCTY(D, T)
C
      REAL*8 D, T, VISCTY
      VISCTY = D*T
      VISCTY = 0
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION THCND(D, T)
C
      REAL*8 D, T, THCND
      THCND = D*T
      THCND = 0
      END
C
C=======================================================================
C=======================================================================
C
      FUNCTION SURFTN(P, T)
C
C-----------------------------------------------------------------------
C (C) Copyright 1993  |  Center for Applied Thermodynamic Studies
C                     |  University of Idaho, Moscow, Idaho
C-----------------------------------------------------------------------
      REAL*8 P,T,SURFTN
      SURFTN=P*T
      SURFTN=0
      END
