close all;clear all;clc
% Dummy case, only one measurement set available

%% Define the matrix

%load('matrices')
load('sensitivity_matrices.mat')
y_opt = full(casadi_struct2vec(y));

% Choose temperature, pressure and flow measurements, and u
fields = fieldnames(y);

fields2 = {'T101','T102','T103','T104',...
           'P101','P102','P103','P104',...
           'T201','T202','T203','T204','T205',...
           'P201','P202','P203','P204','P205',...
           'T301','T302','T303','T304','T305','T306',...
           'P301','P302','P303','P304','P305','P306',...
           'T402','T403','T404',...
           'm101','m201','m301',...
           'Wcomp_P','Wcomp_E','Wcomp_M',...
           'T401','m401',...
           'alpha1','alpha2','alpha3','Tambient',...
           'mcond_P','mcond_E','mcond_M',...
           'Tcond_P','Tcond_E','Tcond_M',...
          };      

subset = zeros(size(fields2));
for i = 1:length(fields2)
    subset(i) = strmatch(fields2{i},fields);
end

y_opt = y_opt(subset);
F  = F(subset,:);
Gy = Gy(subset,:);
Gd = Gd(subset,:);
Wd = diag([0.250,0.02,0.02,0.10,0.10,0.10]);
       
ys = y;
ys.alpha1 = 1;
ys.alpha2 = 1;
ys.alpha3 = 1;
ys.T101 = 1; ys.T102 = 1; ys.T103 = 1; ys.T104 = 1;
ys.T201 = 1; ys.T202 = 1; ys.T203 = 1; ys.T204 = 1; ys.T205 = 1;
ys.T301 = 1; ys.T302 = 1; ys.T303 = 1; ys.T304 = 1; ys.T305 = 1; ys.T306 = 1;
ys.T401 = 1; ys.T402 = 1; ys.T403 = 1; ys.T404 = 1; ys.Tambient = 1;
ys.P101 = 0.2; ys.P102 = 1; ys.P103 = 1; ys.P104 = 0.2;
ys.P201 = 0.1; ys.P202 = 1; ys.P203 = 1; ys.P204 = 1; ys.P204 = 0.1;
ys.P301 = 0.5; ys.P302 = 2; ys.P303 = 2; ys.P304 = 2; ys.P305 = 2; ys.P306 = 0.5;
ys = full(casadi_struct2vec(ys));
ys = ys(subset);
Wn = diag(ys*0.01);


F = Gd-Gy*inv(Juu)*Jud;

Juu12=Juu^(1/2);

nu=length(Juu);
nd=length(Jud);
ny=length(Gy);

Y = [F*Wd Wn];  


%RandomCase
%% Building the restructured matrices
Ydel=Y;

for i=1:nu-1
    Ydel=blkdiag(Ydel,Y);
end
Fdel=Ydel*Ydel'

Gydel=[];
for i=1:nu
    Gydel=blkdiag(Gydel,Gy);
end
Gydel';

jdel=[];
for i=1:nu
    jdel=[jdel ; Juu12(:,i)];
end


%% Find the optimal number of measurements
M=1E3; %Big-M
% price for measurements
mprices=0.00*ones(1,ny);  
%Define empty results matrices
cost = zeros(length(nu:ny),1);
loss = zeros(length(nu:ny),1);
Hres=zeros(nu,ny,length(nu:ny));
Meas=zeros(1,ny,length(nu:ny));
% measurements required
for nm=nu:ny % Run through all possible number of measurements
%for nm = 3

    Q=blkdiag(Fdel,zeros(ny,ny),0);

    A= [Gydel'            zeros(nu*nu,ny)             zeros(nu*nu,1)  ;
        zeros(1,nu*ny)  ones(1,ny)                  -1              ;
        -eye(nu*ny)     -M*repmat(eye(ny),nu,1)     zeros(nu*ny,1)  ;
        eye(nu*ny)      -M*repmat(eye(ny),nu,1)      zeros(nu*ny,1)  ;
        zeros(1,nu*ny)  zeros(1,ny)                  1    ;
        ];

    c = [zeros(1,nu*ny) mprices                0               ];

    lb=[-inf(nu*ny,1);  zeros(ny,1)   ;0  ];
    ub=[inf(nu*ny,1);   ones(ny,1)    ;ny ];

    b= [jdel; 
        0
        zeros(nu*ny,1);
        zeros(nu*ny,1)
        nm
        ];

    
        clear model;
        %names = {'y1u1', 'y2u1', 'y3u1','y4u1','y1u2','y2u2','y3u2'...
        %         'y4u2','sigma1','sigma2','sigma3','sigma4','nm'};
        %model.varnames = names;
        model.Q = sparse(Q);
        model.A = sparse(A);
        model.obj = c;
        model.rhs = b;
        model.lb = lb;
        model.ub = ub;
        model.sense = [repmat('=',nu*nu+1,1);repmat('<',2*nu*ny,1);'='];
        model.modelsense='min'
        model.vtype = [repmat('C',nu*ny,1);repmat('B',ny,1);'I']
        gurobi_write(model, 'MeasSelectPrice0.lp');

        %params.ResultFile = 'test.mps'
        params.Presolve = -1;
        params.FeasibilityTol=1e-6;
        params.OptimalityTol=1e-6;
        %params.FeasibilityTol=1e-4
        results = gurobi(model, params) 
        %results = gurobi(model)
        % for v=1:length(names)
        %     fprintf('%s %e\n', names{v}, results.x(v));
        % end
        fprintf('Obj: %e\n', results.objval);
        % Fill results matrices
        % Selected measurements
        Meas(:,:,nm-nu+1) = results.x(2*ny+1:3*ny)';
        % Selection matrix
        Hres(:,:,nm-nu+1) = [results.x(1:ny)';  results.x(ny+1:2*ny)']; 
        H = (Hres(:,:,nm-nu+1));
        % Total cost
        cost(nm-nu+1)= (results.x'*Q*results.x)/2+model.obj*results.x;
        % Calculate loss with expression from Exact Local Method
        loss(nm-nu+1)= 0.5*norm(Juu12/(H*Gy)*H*Y,'fro')^2;
    
end