function [x_struct,z_struct,u_struct,x0_struct,z0_struct,u0_struct,p_struct,ode,alg] = LNG_model()
    
    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])

    %% Extra functions
    
    xPropane = [0,0,1,0,0,0,0,0,0,0,0]';
    xEthane  = [0,1,0,0,0,0,0,0,0,0,0]';
    xMethane = [1,0,0,0,0,0,0,0,0,0,0]';
    
    g = 9.80665;                            % gravitational constant [m/s2]
    R = 8.31446;                       % universal gas constant [J/(K.mol)]
                      
    dHvap = @(P) -2914.5*P + 17552; % dHvap = @(P) -337.2*P^3 + 1992*P^2 - 6375.3*P + 19233;
    Tsat = @(P) 0.24817*log10(P)^2 + 0.93582*log10(P) + 3.0053;
    
    Hliq = @(T) 4416.4*T^2 - 13786*T + 128.98;
    Hvap = @(T) -10339*T^4 + 119629*T^3 - 517288*T^2 + 994920*T - 708456;
    
    %% Define the streams
    
    M1 = molarMass(xPropane);
    
    z_struct = struct();
    u_struct = struct();
    
    T1 = MX.sym('T1');                                    % Temperature [K]
    P1 = MX.sym('P1');                                     % Pressure [MPa]
    m1 = MX.sym('m1');                            % Molar flow rate [mol/s]
    H1 = MX.sym('H1');                                           % Enthalpy
    Cp1 = MX.sym('Cp1');                        % Heat capacity [J/(mol.K)]  
    Z1 = MX.sym('Z1');
    
    T2 = MX.sym('T2');                                    % Temperature [K]
    P2 = MX.sym('P2');                                      % Pressure [Pa]
    m2 = MX.sym('m2');                            % Molar flow rate [mol/s]
    H2 = MX.sym('H2');                                           % Enthalpy
    Cp2 = MX.sym('Cp2');                        % Heat capacity [J/(mol.K)] 
    
    T3 = MX.sym('T3');                                    % Temperature [K]
    P3 = MX.sym('P3');                                      % Pressure [Pa]
    m3 = MX.sym('m3');                            % Molar flow rate [mol/s] 
    H3 = MX.sym('H3');                                           % Enthalpy
    
    X34 = MX.sym('X34');              % opening of Joule Thompson valve [-]
    
    T4 = MX.sym('T4');                                    % Temperature [K]
    P4 = MX.sym('P4');                                      % Pressure [Pa]
    m4 = MX.sym('m4');                            % Molar flow rate [mol/s] 
    H4 = MX.sym('H4');                                           % Enthalpy     
    
    N12     = MX.sym('N12');                    % average adiabatic ratio [-]
    gamma12 = MX.sym('gamma12');            % average adiabatic ratio [-]
    k12     = MX.sym('k12');                            % k = n/(n+1) [-] 
    nu12    = MX.sym('nu12');                 % compressor efficiency [%]
    head12  = MX.sym('head12');                        % compressor head [m]
    Pow12   = MX.sym('Pow12');         % compressor power consumption [W]
    %srg12   = MX.sym('srg12');                      % indicates surge [-]
    %stw12   = MX.sym('stw12');          % indicates Stonewall / choke [-]
    
    Q_23 = MX.sym('Q_23'); % heat transfered in condenser
    
    Q_PE = MX.sym('Q_PE'); % heat from ethane to propane
    Q_PM = MX.sym('Q_PM'); % heat from methane to propane
    Q_PLNG = MX.sym('Q_PLNG'); % heat from lng to propane
    
    z_struct.stream1.T = T1;
    z_struct.stream1.P = P1;
    z_struct.stream1.m = m1;
    z_struct.stream1.H = H1;
    z_struct.stream1.Cp = Cp1;
    z_struct.stream1.Z = Z1;
    
    z_struct.stream2.T = T2;
    z_struct.stream2.P = P2;
    z_struct.stream2.m = m2;
    z_struct.stream2.H = H2;
    z_struct.stream2.Cp = Cp2;
    
    z_struct.stream3.T = T3;
    z_struct.stream3.P = P3;
    z_struct.stream3.m = m3;
    z_struct.stream3.H = H3;
    
    z_struct.stream4.T = T4;
    z_struct.stream4.P = P4;
    z_struct.stream4.m = m4;
    z_struct.stream4.H = H4;
    
    z_struct.comp12.gamma = gamma12;
    z_struct.comp12.k = k12;
    z_struct.comp12.nu = nu12;
    z_struct.comp12.head = head12;
    z_struct.comp12.Pow = Pow12;
 
    z_struct.cond23.Q = Q_23;
    
    z_struct.HEX_PE.Q = Q_PE;
    z_struct.HEX_PM.Q = Q_PM;
    z_struct.HEX_PLNG.Q = Q_PLNG;
    
     % Inputs
    u_struct.comp12.N = N12;
    u_struct.jtvalve34.X = X34;
    
    % Define equations 
    algs = struct();

    algs.H1 = H1*1E5 - Hvap(T1)*m1;
    algs.Cp1 = Cp1 - heatCapacity(xPropane,T1);
    algs.Z1 = Z1 - compressibilityFactor(xPropane,T1,P1,Z1);
    
    algs.T2 = head12*1E-2  - k12*Z1*R/(g*M1)*(T2-T1);
    algs.P2 = T2/T1 - (P2/P1)^(1/k12);
    algs.m2 = m1 - m2;
    algs.H2 = H2*1E5 - (Hvap(Tsat(P2)) + 0.5*(Cp2+heatCapacity(xPropane,Tsat(P2)))*(T2-Tsat(P2))*100)*m2;
    algs.Cp2 = Cp2 - heatCapacity(xPropane,T2);
    
    algs.T3 = T3-Tsat(P3);
    algs.P3 = P3-P2;
    algs.m3 = m3-m2;
    algs.H3 = H3*1E5 - Hliq(T3)*m3;
    
    algs.T4 = T4 - T1;
    algs.P4 = P4 - P1;
    X = 0.5;
    Cv = 11.7;
    algs.m4 = m4 - X34*Cv*sqrt(abs(P4-P3));
    %algs.m4 = m4 - m3;
    algs.H4 = H4 - H3;
    
    algs.k12 = k12  - nu12*gamma12/(gamma12-1);
    algs.gamma12 = gamma12 - 1/2*( Cp1/(Cp1-R) + Cp2/(Cp2-R));

    % parameters for the fit nu = f(q)
    x = (7.253*N12/m1);    
    algs.nu12 = nu12 - (0.639645 - 1.09059*x + 0.479651*x^2)/(1.14411 - 2.10497*x + x^2);
    
    c6 = -6.82;
    c7 = 15.4;
    c8 = 6.48;
    %algs.head12 = head12 - ((c6*N12^2 + c7*N12 + c8) * (1 - 0.1/7.253*m1)); 
    algs.head12 = (head12 - (10.5+3/(1+exp(-N12*12.5+7.5))) * (1 - 0.1/7.253*m1));
    
    algs.Pow12 = Pow12 - (H2-H1);        
    algs.Q_23  = Q_23  - (H3-H2);
    
    algs.Q_PE = Q_PE - 0.6753;%;
    algs.Q_PM = Q_PM - 0.08271;%;
    algs.Q_PLNG = (Q_PE + Q_PM + Q_PLNG) - (H1-H4);
    
    close all
    %for j=[7.253*0.9 7.253*1.0 7.253*1.1];
    for j=[7.253*1.0];
    Ps = [];
    nus = [];
    Pows = [];
    heads = [];
    testheads = [];
    
    compspeeds = linspace(0.5,1.05,20);
    for i=compspeeds
        
        i = 0.8;
        
        algs.m1 = m1 - 7.253; % *1E4 kgmole/h
        algs.mtot = m1 - m4; % *1E4 kgmole/h
        %algs.T1 = T1 - (298.15-41.99)/100;
        algs.P1 = P1 - 0.1022;
        algs.T1s = T1 - Tsat(P1);
        algs.N12 = N12 - i;
        %algs.X34 = X34 - 0.5;
        
        rf_function = casadi.Function('rf_function',{vertcat(N12,X34,casadi_struct2vec(z_struct))},{casadi_struct2vec(algs)});
        
        %algs.srg = srg - (qN-qmin);
        %algs.stw = stw - (qmax-qN);

        rf = casadi.rootfinder('rf','kinsol',rf_function);


        z0 = [0.80,... N12
              0.50,... X34
              2.56,... T1
              0.1022,... P1
              18.75,... m1
              1,... H1
              67,... Cp1
              0.96,... Z1 
              2.97,... T2
              1.2,... P2
              7.75,... m2
              1,... H2
              80,... Cp2
              3.83,... T3
              1.2,... P3
              7.75,... m3
              1,... H3
              2.56,... T4
              0.1022,... P4
              7.75,... m4
              1,... H4
              1.2,... gamma12 
              7,... k12
              0.8,... nu12
              12, ... head12
              1, ... Pow12
              1, ... Q_23
              1, ... Q_PE
              1, ... Q_PM
              1, ... Q_PLNG
              ]';
        z0 = full(rf(z0))

        z0(end-4) + z0(end-3) + z0(end)
        
        heads = [heads z0(25)];
        Ps = [Ps z0(10)];
        nus = [nus z0(24)];
        Pows = [Pows z0(26)];
        
    end
    hold on
    figure(1)
    plot(compspeeds,Ps)
    hold off
    
    hold on
    figure(2)
    plot(compspeeds,nus)
    hold off
    
    hold on
    figure(3)
    plot(compspeeds,Pows)
    hold off
    
    hold on
    figure(4)
    plot(heads,nus)
    hold off
    
    end
    

end