function [z_struct,u_struct,algs] = LNG_model_E()
    
    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])
        
    mrefr_E = 4.652;
    
    %% Extra functions
    xEthane  = [0,1,0,0,0,0,0,0,0,0,0]';
    
    g = 9.80665;                            % gravitational constant [m/s2]
    R = 8.31446;                       % universal gas constant [J/(K.mol)]
                      
    Tsat_E = @(P) 0.2108*log10(P)^2 + 0.77135*log10(P) + 2.4141; % Allprops; 42 points, R^2=0.9998 

    HvapLo_E = @(T) 3322.4*T + 337.4; % Vapor enthalpy at low pressure (0.04 - 0.2 MPa)
    
    HliqHi_E = @(T) 3376.9*T^2 - 8003.3*T - 4122.7; % Liquid enthalpy at high pressure (1 - 3 MPa)
    HvapHi_E = @(T) -5475.8*T^3 + 38737*T^2 - 89627*T + 75653; % Vapor enthalpy at high pressure (1 - 3 MPa)
    
    %% Define the streams
    z_struct = struct();
    u_struct = struct();
    
    T201  = MX.sym('T201');                               % Temperature [K]
    P201  = MX.sym('P201');                                % Pressure [MPa]
    m201  = MX.sym('m201');                       % Molar flow rate [mol/s]
    H201  = MX.sym('H201');                                      % Enthalpy
    Cp201 = MX.sym('Cp201');                    % Heat capacity [J/(mol.K)]  
    Z201  = MX.sym('Z201');
    
    T202  = MX.sym('T202');                               % Temperature [K]
    P202  = MX.sym('P202');                                 % Pressure [Pa]
    m202  = MX.sym('m202');                       % Molar flow rate [mol/s]
    H202  = MX.sym('H202');                                      % Enthalpy
    Cp202 = MX.sym('Cp202');                    % Heat capacity [J/(mol.K)] 
    
    T203  = MX.sym('T203');                               % Temperature [K]
    P203  = MX.sym('P203');                                 % Pressure [Pa]
    m203  = MX.sym('m203');                       % Molar flow rate [mol/s] 
    H203  = MX.sym('H203');                                      % Enthalpy
    Cp203 = MX.sym('Cp203');                    % Heat capacity [J/(mol.K)] 
    
    T204  = MX.sym('T204');                               % Temperature [K]
    P204  = MX.sym('P204');                                 % Pressure [Pa]
    m204  = MX.sym('m204');                       % Molar flow rate [mol/s] 
    H204  = MX.sym('H204');                                      % Enthalpy     
    Cp204 = MX.sym('Cp204');                    % Heat capacity [J/(mol.K)] 
    
    T205 = MX.sym('T205');                                % Temperature [K]
    P205 = MX.sym('P205');                                  % Pressure [Pa]
    m205 = MX.sym('m205');                        % Molar flow rate [mol/s] 
    H205 = MX.sym('H205');                                       % Enthalpy     
    
    X45 = MX.sym('X45');              % opening of Joule Thompson valve [-]
    
    N12     = MX.sym('N12');                    % average adiabatic ratio [-]
    gamma12 = MX.sym('gamma12');            % average adiabatic ratio [-]
    k12     = MX.sym('k12');                            % k = n/(n+1) [-] 
    nu12    = MX.sym('nu12');                 % compressor efficiency [%]
    head12  = MX.sym('head12');                        % compressor head [m]
    Pow12   = MX.sym('Pow12');         % compressor power consumption [W]

    Q_air = MX.sym('Q_air'); % heat transfered in condenser
    
    Q_EP = MX.sym('Q_EP'); % heat from propane to ethane
    Q_EM = MX.sym('Q_EM'); % heat from methane to ethane
    Q_ELNG = MX.sym('Q_ELNG'); % heat from lng to ethane
    
    z_struct.stream201.T  = T201;
    z_struct.stream201.P  = P201;
    z_struct.stream201.m  = m201;
    z_struct.stream201.H  = H201;
    z_struct.stream201.Cp = Cp201;
    z_struct.stream201.Z  = Z201;
    
    z_struct.stream202.T  = T202;
    z_struct.stream202.P  = P202;
    z_struct.stream202.m  = m202;
    z_struct.stream202.H  = H202;
    z_struct.stream202.Cp = Cp202;
    
    z_struct.stream203.T  = T203;
    z_struct.stream203.P  = P203;
    z_struct.stream203.m  = m203;
    z_struct.stream203.H  = H203;
    z_struct.stream203.Cp = Cp203;
    
    z_struct.stream204.T  = T204;
    z_struct.stream204.P  = P204;
    z_struct.stream204.m  = m204;
    z_struct.stream204.H  = H204;
    z_struct.stream204.Cp = Cp204;
    
    z_struct.stream205.T = T205;
    z_struct.stream205.P = P205;
    z_struct.stream205.m = m205;
    z_struct.stream205.H = H205;
    
    z_struct.comp12.gamma = gamma12;
    z_struct.comp12.k     = k12;
    z_struct.comp12.nu    = nu12;
    z_struct.comp12.head  = head12;
    z_struct.comp12.Pow   = Pow12;
 
    z_struct.cond23.Q     = Q_air;
    
    z_struct.HEX_PE.Q    = Q_EP;
    z_struct.HEX_PM.Q    = Q_EM;
    z_struct.HEX_PLNG.Q  = Q_ELNG;
    
     % Inputs
    u_struct.comp.N = N12;
    u_struct.jtvalve.X = X45;
    
    % Define equations 
    algs = struct();
    
    algs.T201  = T201 - Tsat_E(P201);
    algs.m201  = m201 - m205; % *1E4 kgmole/h
    algs.H201  = H201*1E5 - HvapLo_E(T201)*m201;
    algs.Cp201 = Cp201 - heatCapacity(xEthane,T201);
    algs.Z201  = Z201 - compressibilityFactor(xEthane,T201,P201,Z201);
    
    M1 = molarMass(xEthane);
    algs.T202  = head12*1E-2  - k12*Z201*R/(g*M1)*(T202-T201);
    algs.P202  = T202/T201 - (P202/P201)^(1/k12);
    algs.m202  = m201 - m202;
    algs.H202  = H202*1E5 - (HvapHi_E(Tsat_E(P202)) + 0.5*(Cp202+heatCapacity(xEthane,Tsat_E(P202)))*(T202-Tsat_E(P202))*100)*m202;
    algs.Cp202 = Cp202 - heatCapacity(xEthane,T202);
    
    algs.T203  = T203-2.98;
    algs.P203  = P203-P202;
    algs.m203  = m203-m202;
    algs.H203  = H203*1E5 - (HvapHi_E(Tsat_E(P203)) + 0.5*(Cp203+heatCapacity(xEthane,Tsat_E(P203)))*(T203-Tsat_E(P203))*100)*m203;
    algs.Cp203 = Cp203 - heatCapacity(xEthane,T203);
    
    % PHASE TRANSITION
    
    algs.T204  = T204-2.38;
    algs.P204  = P204-P203;
    algs.m204  = m204-m203;
    algs.H204  = H204*1E5 - (HliqHi_E(Tsat_E(P204)) + Cp204*(T204-Tsat_E(P204))*100)*m204; %HliqHi(T10)*m10;                  
    algs.Cp204 = Cp204 - 95;
    
    algs.T205 = T205 - T201;
    algs.P205 = P205 - P201;
    Cv = 6.5;
    algs.m205 = m205 - X45*Cv*sqrt(abs(P205-P204));
    algs.H205 = H205 - H204;
    
    algs.k12 = k12  - nu12*gamma12/(gamma12-1);
    algs.gamma12 = gamma12 - 1/2*( Cp201/(Cp201-R) + Cp202/(Cp202-R));

    % parameters for the fit nu = f(q)
    tmp = (mrefr_E*N12/m201); 
    %tmp = N12;
    algs.nu12 = nu12 - (0.639645 - 1.09059*tmp + 0.479651*tmp^2)/(1.14411 - 2.10497*tmp + tmp^2);
    
    %f = @(x) (0.639645 - 1.09059*x + 0.479651*x.^2)./(1.14411 - 2.10497*x + x.^2);
    %xs = 0.6:0.01:1.05;
    %plot(xs,f(xs))
    
    %f = @(x) (0.639645 - 1.09059*x + 0.479651*x.^2)./(1.14411 - 2.10497*x + x.^2);
    %xs = 0.6:0.01:1.05;
    %plot(xs,f(xs))
    
    %algs.head12 = (head89 - 1.95*(10.5+3/(1+exp(-N89*11.5+7.5))) * (1 - 0.1/mrefr*m8));
    algs.head12 = (head12 - (15.5+11.5/(1+exp(-N12*11.5+7.5))) * (1 - 0.1/mrefr_E*m201));
    %algs.head12 = head89 - 24;
    
    algs.Pow12 = Pow12 - (H202-H201);
    
    algs.Q_air  = Q_air  - (H203-H202);
    algs.Q_EP = Q_EP - (H204-H203);
    algs.Q_EM = Q_EM - 0.158;
    algs.Q_ELNG = (Pow12 + Q_EM + Q_ELNG) + (Q_air + Q_EP);
    
    %return
    
    algs.m   = m201 - mrefr_E; % *1E4 kgmole/h
    algs.N12 = N12 - 0.92;
    algs.X34 = X45 - 0.5;
    %algs.N12 = P8 - 0.1;
    %algs.X342 = P9 - 2.2;
    
    vars = vertcat(casadi_struct2vec(u_struct),casadi_struct2vec(z_struct));
    eqs  = casadi_struct2vec(algs);
    
    rf_function = casadi.Function('rf_function',{vars},{eqs});

    %algs.srg = srg - (qN-qmin);
    %algs.stw = stw - (qmax-qN);

    rf = casadi.rootfinder('rf','newton',rf_function);


    z0 = [0.92,... N_comp
          0.65,... X_jtv
          1.85,... T201
          0.10,... P201
          4.12,... m201
          0.3,... H201
          37,... Cp201
          0.96,... Z201 
          3.8,... T202
          2.3,... P202
          4.12,... m202
          0.7,... H202
          65,... Cp202
          2.98,... T203
          2.3,... P203
          4.12,... m203
          0.3,... H203
          53,... Cp203
          2.38,... T204
          2.3,... P204
          4.12,... m204
          0.1,... H204
          100,... Cp204
          1.85,... T205
          0.10,... P205
          4.12,... m205
          0.1,... H205
          1.2,... gamma12 
          4,... k12
          0.78,... nu12
          22, ... head12
          0.4, ... Pow12
          -0.2, ... Q_air
          -0.35, ... Q_PE
          0, ... Q_PM
          0.2, ... Q_PLNG
          ]';
    z0 = full(rf(z0))
    drawPFD_E(z0)
end