function [z_struct,u_struct,algs] = LNG_model_M()

    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])

    mrefr_M = 3.421;
    
    %% Extra functions
    xMethane  = [1,0,0,0,0,0,0,0,0,0,0]';

    g = 9.80665;                            % gravitational constant [m/s2]
    R = 8.31446;                       % universal gas constant [J/(K.mol)]

    Tsat_M = @(P) 0.1311*log10(P)^2 + 0.5153*log10(P) + 1.4966; % Allprops; 42 points, R^2=0.9998 

    HliqLo = @(T) 5606.1*T - 20871; % Liquid enthalpy at low pressure (0.04 - 0.2 MPa)
    HvapLo_M = @(T) 2669.9*T - 9405.5; % Vapor enthalpy at low pressure (0.04 - 0.2 MPa)

    HliqHi_M = @(T) 12520*T^2 - 34878*T + 12199; % Liquid enthalpy at high pressure (2 - 4 MPa)
    HvapHi_M = @(T) -19226*T^2 + 64047*T - 59074; % Vapor enthalpy at high pressure (2 - 4 MPa)

    %% Define the streams
    
    M1 = molarMass(xMethane);
    
    z_struct = struct();
    u_struct = struct();
    
    T301  = MX.sym('T301');                               % Temperature [K]
    P301  = MX.sym('P301');                                % Pressure [MPa]
    m301  = MX.sym('m301');                       % Molar flow rate [mol/s]
    H301  = MX.sym('H301');                                      % Enthalpy
    Cp301 = MX.sym('Cp301');                    % Heat capacity [J/(mol.K)]  
    Z301  = MX.sym('Z301');
    
    T302  = MX.sym('T302');                               % Temperature [K]
    P302  = MX.sym('P302');                                 % Pressure [Pa]
    m302  = MX.sym('m302');                       % Molar flow rate [mol/s]
    H302  = MX.sym('H302');                                      % Enthalpy
    Cp302 = MX.sym('Cp302');                    % Heat capacity [J/(mol.K)] 
    
    T303  = MX.sym('T303');                               % Temperature [K]
    P303  = MX.sym('P303');                                 % Pressure [Pa]
    m303  = MX.sym('m303');                       % Molar flow rate [mol/s] 
    H303  = MX.sym('H303');                                      % Enthalpy
    Cp303 = MX.sym('Cp303');                    % Heat capacity [J/(mol.K)] 
    
    T304  = MX.sym('T304');                               % Temperature [K]
    P304  = MX.sym('P304');                                 % Pressure [Pa]
    m304  = MX.sym('m304');                       % Molar flow rate [mol/s] 
    H304  = MX.sym('H304');                                      % Enthalpy     
    Cp304 = MX.sym('Cp304');                    % Heat capacity [J/(mol.K)] 
    
    T305 = MX.sym('T305');                                % Temperature [K]
    P305 = MX.sym('P305');                                  % Pressure [Pa]
    m305 = MX.sym('m305');                        % Molar flow rate [mol/s] 
    H305 = MX.sym('H305');                                       % Enthalpy    
    Cp305 = MX.sym('Cp305');                    % Heat capacity [J/(mol.K)] 
    
    T306 = MX.sym('T306');                                % Temperature [K]
    P306 = MX.sym('P306');                                  % Pressure [Pa]
    m306 = MX.sym('m306');                        % Molar flow rate [mol/s] 
    H306 = MX.sym('H306');                                       % Enthalpy    
    
    X56 = MX.sym('X56');              % opening of Joule Thompson valve [-]
    
    N12     = MX.sym('N12');                    % average adiabatic ratio [-]
    gamma12 = MX.sym('gamma12');            % average adiabatic ratio [-]
    k12     = MX.sym('k12');                            % k = n/(n+1) [-] 
    nu12    = MX.sym('nu12');                 % compressor efficiency [%]
    head12  = MX.sym('head12');                        % compressor head [m]
    Pow12   = MX.sym('Pow12');         % compressor power consumption [W]

    Q_air = MX.sym('Q_air'); % heat transfered in condenser
    
    Q_MP = MX.sym('Q_MP'); % heat from propane to methane
    Q_ME = MX.sym('Q_ME'); % heat from ethane to methane
    Q_MLNG = MX.sym('Q_MLNG'); % heat from lng to methane
    
    z_struct.stream301.T  = T301; 
    z_struct.stream301.P  = P301;
    z_struct.stream301.m  = m301;
    z_struct.stream301.H  = H301;
    z_struct.stream301.Cp = Cp301;
    z_struct.stream301.Z  = Z301;
    
    z_struct.stream302.T  = T302;
    z_struct.stream302.P  = P302;
    z_struct.stream302.m  = m302;
    z_struct.stream302.H  = H302;
    z_struct.stream302.Cp = Cp302;
    
    z_struct.stream303.T  = T303;
    z_struct.stream303.P  = P303;
    z_struct.stream303.m  = m303;
    z_struct.stream303.H  = H303;
    z_struct.stream303.Cp = Cp303;
    
    z_struct.stream304.T = T304;
    z_struct.stream304.P = P304;
    z_struct.stream304.m = m304;
    z_struct.stream304.H = H304;
    z_struct.stream304.Cp = Cp304;
    
    z_struct.stream305.T = T305;
    z_struct.stream305.P = P305;
    z_struct.stream305.m = m305;
    z_struct.stream305.H = H305;
    z_struct.stream305.Cp = Cp305;
    
    z_struct.stream306.T = T306;
    z_struct.stream306.P = P306;
    z_struct.stream306.m = m306;
    z_struct.stream306.H = H306;
    
    z_struct.comp12.gamma = gamma12;
    z_struct.comp12.k = k12;
    z_struct.comp12.nu = nu12;
    z_struct.comp12.head = head12;
    z_struct.comp12.Pow = Pow12;
 
    z_struct.cond23.Q = Q_air;
    
    z_struct.HEX_MP.Q = Q_MP;
    z_struct.HEX_ME.Q = Q_ME;
    z_struct.HEX_MLNG.Q = Q_MLNG;
    
     % Inputs
    u_struct.comp.N = N12;
    u_struct.jtvalve.X = X56;
    
    % Define equations 
    algs = struct();

    algs.T301  = T301 - Tsat_M(P301);
    algs.m301  = m301 - m306; % *1E4 kgmole/h
    algs.H301  = H301*1E5 - HvapLo_M(T301)*m301;
    algs.Cp301 = Cp301 - heatCapacity(xMethane,T301);
    algs.Z301  = Z301 - compressibilityFactor(xMethane,T301,P301,Z301);
    
    algs.T302  = head12*1E-2  - k12*Z301*R/(g*M1)*(T302-T301);
    algs.P302  = T302/T301 - (P302/P301)^(1/k12);
    algs.m302  = m302 - m301;
    algs.H302  = H302*1E5 - (HvapHi_M(Tsat_M(P302)) + 0.5*(Cp302+heatCapacity(xMethane,Tsat_M(P302)))*(T302-Tsat_M(P302))*100)*m302;
    algs.Cp302 = Cp302 - heatCapacity(xMethane,T302);
    
    algs.T303  = T303-2.98;
    algs.P303  = P303-P302;
    algs.m303  = m303-m302;
    algs.H303  = H303*1E5 - (HvapHi_M(Tsat_M(P303)) + 0.5*(Cp303+heatCapacity(xMethane,Tsat_M(P303)))*(T303-Tsat_M(P303))*100)*m303;
    algs.Cp303 = Cp303 - heatCapacity(xMethane,T303);
    
    algs.T304  = T304-2.38;
    algs.P304  = P304-P303;
    algs.m304  = m304-m303;
    algs.H304  = H304*1E5 - (HvapHi_M(Tsat_M(P304)) + 0.5*(Cp304+heatCapacity(xMethane,Tsat_M(P304)))*(T304-Tsat_M(P304))*100)*m304;  
    algs.Cp304 = Cp304 - heatCapacity(xMethane,T304);
    
    % PHASE TRANSITION
    
    %algs.T305 = T305 - Tsat(P305);
    algs.T305  = T305-(-83+273.15)/100;
    algs.P305  = P305-P304;
    algs.m305  = m305-m304;
    algs.H305  = H305*1E5 - (HliqHi_M(Tsat_M(P305)) + Cp305*(T305-Tsat_M(P305))*100)*m305; %HliqHi(T10)*m10;                  
    algs.Cp305 = Cp305 - 95;
    
    Cv = 4.5;
    algs.T306 = T306 - T301;
    algs.P306 = P306 - P301;
    algs.m306 = m306 - X56*Cv*sqrt(abs(P306-P305));
    algs.H306 = H306 - H305;

    algs.k12 = k12  - nu12*gamma12/(gamma12-1);
    algs.gamma12 = gamma12 - 1/2*( Cp301/(Cp301-R) + Cp302/(Cp302-R));

    % parameters for the fit nu = f(q)
    xMethane = (mrefr_M*N12/m301); 
    algs.nu12 = nu12 - (0.639645 - 1.09059*xMethane + 0.479651*xMethane^2)/(1.14411 - 2.10497*xMethane + xMethane^2);
    
    algs.head12 = (head12 - 3.4*(10.5+3/(1+exp(-N12*12.5+7.5))) * (1 - 0.1/mrefr_M*m301));
    
    algs.Pow12 = Pow12   - (H302-H301);
    algs.Q_air  = Q_air  - (H303-H302);
    algs.Q_MP   = Q_MP   - (H304-H303);
    algs.Q_ME   = Q_ME   - (H305-H304);
    algs.Q_PLNG = Q_MLNG - (H301-H306);
    
    %return
    
    algs.m = m301 - mrefr_M; % *1E4 kgmole/h
    algs.N12 = N12 - 0.92;
    algs.X34 = X56 - 0.420;
    
    vars = vertcat(casadi_struct2vec(u_struct),casadi_struct2vec(z_struct));
    eqs  = casadi_struct2vec(algs);
    
    rf_function = casadi.Function('rf_function',{vars},{eqs});

    %algs.srg = srg - (qN-qmin);
    %algs.stw = stw - (qmax-qN);

    rf = casadi.rootfinder('rf','kinsol',rf_function);
    
    z0 = [0.80,... N_comp
          0.65,... X_jtv
          ...
          1.80,... T301
          0.10,... P301
          mrefr_M,... m301
          0.275,... H301
          37,... Cp301
          0.96,... Z301
          ...
          3.9,... T302
          3.3,... P302
          mrefr_M,... m302
          0.6,... H302
          65,... Cp302
          ...
          2.98,... T303
          3.3,... P303
          mrefr_M,... m303
          0.3,... H303
          53,... Cp303
          ...
          2.38,... T304
          3.3,... P304
          mrefr_M,... m304
          0.1,... H304
          50,... Cp304
          ...
          1.85,... T305
          3.3,... P305
          mrefr_M,... m305
          0.1,... H305
          100,... Cp305
          ...
          1.85,... T306
          0.10,... P306
          mrefr_M,... m306
          0.1,... H306
          ...
          1.2,... gamma12 
          4,... k12
          0.8,... nu12
          24, ... head12
          0, ... Pow12
          0, ... Q_air
          0, ... Q_PE
          0, ... Q_PM
          0, ... Q_PLNG
          ]';
    z0 = full(rf(z0))
    drawPFD_M(z0)
end