function [z_struct,u_struct,algs] = LNG_model_P()
    
    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])

    
    mrefr = 7.253
    
    %% Extra functions
    xPropane = [0,0,1,0,0,0,0,0,0,0,0]';
    x=xPropane;
    
    g = 9.80665;                            % gravitational constant [m/s2]
    R = 8.31446;                       % universal gas constant [J/(K.mol)]
                      
    Tsat = @(P) 0.24817*log10(P)^2 + 0.93582*log10(P) + 3.0053;
 
    HliqLo = @(T) 9931.4*T - 31725; % Liquid enthalpy at low pressure (0.04 - 0.2 MPa)
    HvapLo = @(T) 5258.1*T - 2181.7; % Vapor enthalpy at low pressure (0.04 - 0.2 MPa)
    
    HliqHi = @(T) 3758.4*T^2 - 10530*T - 3520.2; % Liquid enthalpy at high pressure (0.7 - 3 MPa)
    HvapHi = @(T) -4860.7*T^3 + 42722*T^2 - 120817*T + 122555; % Vapor enthalpy at high pressure (0.7 - 3 MPa)
    
    %% Define the streams
    
    M1 = molarMass(x);
    
    z_struct = struct();
    u_struct = struct();
    
    T101  = MX.sym('T101');                               % Temperature [K]
    P101  = MX.sym('P101');                                % Pressure [MPa]
    m101  = MX.sym('m101');                       % Molar flow rate [mol/s]
    H101  = MX.sym('H101');                                      % Enthalpy
    Cp101 = MX.sym('Cp101');                    % Heat capacity [J/(mol.K)]  
    Z101  = MX.sym('Z101');
    
    T102  = MX.sym('T102');                               % Temperature [K]
    P102  = MX.sym('P102');                                 % Pressure [Pa]
    m102  = MX.sym('m102');                       % Molar flow rate [mol/s]
    H102  = MX.sym('H102');                                      % Enthalpy
    Cp102 = MX.sym('Cp102');                    % Heat capacity [J/(mol.K)] 
    
    T103  = MX.sym('T103');                               % Temperature [K]
    P103  = MX.sym('P103');                                 % Pressure [Pa]
    m103  = MX.sym('m103');                       % Molar flow rate [mol/s] 
    H103  = MX.sym('H103');                                      % Enthalpy
    Cp103 = MX.sym('Cp103');                    % Heat capacity [J/(mol.K)] 
    
    X34 = MX.sym('X34');              % opening of Joule Thompson valve [-]
    
    T104 = MX.sym('T104');                                % Temperature [K]
    P104 = MX.sym('P104');                                  % Pressure [Pa]
    m104 = MX.sym('m104');                        % Molar flow rate [mol/s] 
    H104 = MX.sym('H104');                                       % Enthalpy     
    
    N12     = MX.sym('N12');                  % average adiabatic ratio [-]
    gamma12 = MX.sym('gamma12');              % average adiabatic ratio [-]
    k12     = MX.sym('k12');                              % k = n/(n+1) [-] 
    nu12    = MX.sym('nu12');                   % compressor efficiency [%]
    head12  = MX.sym('head12');                       % compressor head [m]
    Pow12   = MX.sym('Pow12');           % compressor power consumption [W]
    
    Q_air = MX.sym('Q_air'); % heat transfered in condenser
    
    Q_PE = MX.sym('Q_PE'); % heat from ethane to propane
    Q_PM = MX.sym('Q_PM'); % heat from methane to propane
    Q_PLNG = MX.sym('Q_PLNG'); % heat from lng to propane
    
    z_struct.stream101.T  = T101;
    z_struct.stream101.P  = P101;
    z_struct.stream101.m  = m101;
    z_struct.stream101.H  = H101;
    z_struct.stream101.Cp = Cp101;
    z_struct.stream101.Z  = Z101;
    
    z_struct.stream102.T  = T102;
    z_struct.stream102.P  = P102;
    z_struct.stream102.m  = m102;
    z_struct.stream102.H  = H102;
    z_struct.stream102.Cp = Cp102;
    
    z_struct.stream103.T  = T103;
    z_struct.stream103.P  = P103;
    z_struct.stream103.m  = m103;
    z_struct.stream103.H  = H103;
    z_struct.stream103.Cp = Cp103;
    
    z_struct.stream104.T = T104;
    z_struct.stream104.P = P104;
    z_struct.stream104.m = m104;
    z_struct.stream104.H = H104;
    
    z_struct.comp12.gamma = gamma12;
    z_struct.comp12.k     = k12;
    z_struct.comp12.nu    = nu12;
    z_struct.comp12.head  = head12;
    z_struct.comp12.Pow   = Pow12;
 
    z_struct.cond23.Q = Q_air;
    
    z_struct.HEX_PE.Q = Q_PE;
    z_struct.HEX_PM.Q = Q_PM;
    z_struct.HEX_PLNG.Q = Q_PLNG;
    
     % Inputs
    u_struct.comp.N    = N12;
    u_struct.jtvalve.X = X34;
    
    % Define equations 
    algs = struct();
    
    algs.T101  = T101 - Tsat(P101);
    algs.m101  = m101 - m104; % *1E4 kgmole/h
    algs.H101  = H101*1E5 - HvapLo(T101)*m101;
    algs.Cp101 = Cp101 - heatCapacity(x,T101);
    algs.Z101  = Z101 - compressibilityFactor(x,T101,P101,Z101);
    
    algs.T102  = head12*1E-2  - k12*Z101*R/(g*M1)*(T102-T101);
    algs.P102  = T102/T101 - (P102/P101)^(1/k12);
    algs.m102  = m102 - m101;
    algs.H102  = H102*1E5 - (HvapHi(Tsat(P102)) + 0.5*(Cp102+heatCapacity(x,Tsat(P102)))*(T102-Tsat(P102))*100)*m102;
    algs.Cp102 = Cp102 - heatCapacity(x,T102);
    
    %algs.T103 = T103-Tsat(P103);
    algs.T103 = T103-2.98;
    algs.P103 = P103-P102;
    algs.m103 = m103-m102;
    algs.H103 = H103*1E5 - (HliqHi(Tsat(P103)) + Cp103*(T103-Tsat(P103))*100)*m103;
    algs.Cp103 = Cp103 - 100;
    
    algs.T104 = T104 - T101;
    algs.P104 = P104 - P101;
    Cv = 13.2;
    algs.m104 = m104 - X34*Cv*sqrt(abs(P104-P103));
    algs.H104 = H104 - H103;
    
    algs.k12     = k12  - nu12*gamma12/(gamma12-1);
    algs.gamma12 = gamma12 - 1/2*( Cp101/(Cp101-R) + Cp102/(Cp102-R));

    % parameters for the fit nu = f(q)
    x = (mrefr*N12/m101);
    x = N12;
    algs.nu12 = nu12 - (0.639645 - 1.09059*x + 0.479651*x^2)/(1.14411 - 2.10497*x + x^2);
    
    %f = @(x) (0.639645 - 1.09059*x + 0.479651*x.^2)./(1.14411 - 2.10497*x + x.^2);
    %xs = 0.6:0.01:1.05;
    %plot(xs,f(xs))
    
    %f2 = @(x) (11.5+4.7./(1+exp(-x*13.5+11.5))) * 0.9;
    %xs = 0.6:0.01:1.05;
    %plot(xs,f2(xs))
    
    algs.head12 = (head12 - (11.5+4.7/(1+exp(-N12*13.5+11.5))) * (1 - 0.1/mrefr*m101));
    
    algs.Pow12 = Pow12 - (H102-H101);        
    algs.Q_23  = Q_air  - (H103-H102);
    
    algs.Q_PE = Q_PE - 0.647;
    algs.Q_PM = Q_PM - 0.068;
    %algs.Q_PLNG = (Q_PE + Q_PM + Q_PLNG) - (H1-H4);
    algs.Q_PLNG = (Pow12 + Q_air) + (Q_PLNG + Q_PE + Q_PM);
 
    %return  
    
    algs.m   = m101 - mrefr; % *1E4 kgmole/h
    algs.N12 = N12 - 0.92;
    algs.X34 = X34 - 0.5;
    %algs.N12 = P1 - 0.1;
    %algs.X34 = P2 - 1.3;
    
    vars = vertcat(N12,X34,casadi_struct2vec(z_struct));
    eqs = casadi_struct2vec(algs);

    rf_function = casadi.Function('rf_function',{vars},{eqs});

    rf = casadi.rootfinder('rf','newton',rf_function);


    z0 = [0.92,... N12
          0.566,... X34
          2.56,... T101
          0.1022,... P101
          18.75,... m101
          1,... H101
          67,... Cp101
          0.96,... Z101 
          2.97,... T102
          1.2,... P102
          7.75,... m102
          1,... H102
          80,... Cp102
          3.83,... T103
          1.2,... P103
          7.75,... m103
          1,... H103
          100,... Cp103
          2.56,... T104
          0.1022,... P104
          7.75,... m104
          1,... H104
          1.2,... gamma12 
          7,... k12
          0.78,... nu12
          12, ... head12
          1, ... Pow12
          1, ... Q_23
          1, ... Q_PE
          1, ... Q_PM
          1, ... Q_PLNG
          ]';
    z0 = full(rf(z0))
    drawPFD_P(z0)
end