%function res = LNG_model_full()
    
    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])
    %% TODO: remove massflow specification, include compressor equation 
    %  based on compressor head
    
    %close all
    
    %% Extra functions

    m_LNG   = 3.074;
    
    Cv_P       = 14.4234;
    
    xPropane = [0,0,1,0,0,0,0,0,0,0,0]';
    xEthane  = [0,1,0,0,0,0,0,0,0,0,0]';
    xMethane = [1,0,0,0,0,0,0,0,0,0,0]';
    
    g = 9.80665;                            % gravitational constant [m/s2]
    R = 8.31446;                       % universal gas constant [J/(K.mol)]
    
    Tsat_P = @(P) 0.24817*log10(P)^2 + 0.93582*log10(P) + 3.0053;
    Tsat_E = @(P) 0.21080*log10(P)^2 + 0.77135*log10(P) + 2.4141;
    Tsat_M = @(P) 0.13110*log10(P)^2 + 0.51530*log10(P) + 1.4966;
    
    HvapLo_P = @(T) 5258.1*T - 2181.7; % Vapor enthalpy at low pressure (0.04 - 0.2 MPa)
    HvapLo_E = @(T) 3322.4*T + 337.4; % Vapor enthalpy at low pressure (0.04 - 0.2 MPa)
    HvapLo_M = @(T) 2669.9*T - 9405.5; % Vapor enthalpy at low pressure (0.04 - 0.2 MPa)
    
    %HliqHi_P = @(T) 3758.4*T^2 - 10530*T - 3520.2; % Liquid enthalpy at high pressure (0.7 - 3 MPa)
    HliqHi_P = @(P) -80.118*P^4 + 1152.9*P^3 - 5302.5*P^2 + 12747*P - 9570.5;
    %HliqHi_E = @(T) 3376.9*T^2 - 8003.3*T - 4122.7; % Liquid enthalpy at high pressure (1 - 3 MPa)
    HliqHi_E = @(P) 217.31*P^3 - 1778.9*P^2 + 6227.6*P - 8431.4;
    HliqHi_M = @(T) 12520*T^2 - 34878*T + 12199; % Liquid enthalpy at high pressure (2 - 4 MPa)
    
    HvapHi_P = @(T) -4860.7*T^3 + 42722*T^2 - 120817*T + 122555; % Vapor enthalpy at high pressure (0.7 - 3 MPa)
    HvapHi_E = @(T) -5475.8*T^3 + 38737*T^2 - 89627*T + 75653; % Vapor enthalpy at high pressure (1 - 3 MPa)
    %HvapHi_E = @(P) -77.561*P^4 + 719.02*P^3 - 2453.6*P^2 + 3630.2*P + 6238.3;
    HvapHi_M = @(T) -19226*T^2 + 64047*T - 59074; % Vapor enthalpy at high pressure (2 - 4 MPa)
    
    %% Define the streams
    z_struct = struct();
    u_struct = struct();
    
    %% PROPANE
    T101  = MX.sym('T101');                               % Temperature [K]
    P101  = MX.sym('P101');                                % Pressure [MPa]
    m101  = MX.sym('m101');                       % Molar flow rate [mol/s]
    H101  = MX.sym('H101');                                      % Enthalpy
    Cp101 = MX.sym('Cp101');                    % Heat capacity [J/(mol.K)]  
    Z101  = MX.sym('Z101');                        % Compressibility factor
    z_struct.T101   = T101;
    z_struct.P101   = P101;
    z_struct.m101   = m101;
    z_struct.H101   = H101;
    z_struct.Cp101  = Cp101;
    z_struct.Z101   = Z101;
    z0_struct.T101  = 2.3540045179488;
    z0_struct.P101  = 0.1200000000000;
    z0_struct.m101  = 7.8363683797626;
    z0_struct.H101  = 0.7989875905702;
    z0_struct.Cp101 = 62.358736052051;
    z0_struct.Z101  = 0.9521804809881;
    
    T102  = MX.sym('T102');                               % Temperature [K]
    P102  = MX.sym('P102');                                 % Pressure [Pa]
    m102  = MX.sym('m102');                       % Molar flow rate [mol/s]
    H102  = MX.sym('H102');                                      % Enthalpy
    Cp102 = MX.sym('Cp102');                    % Heat capacity [J/(mol.K)] 
    z_struct.T102   = T102;
    z_struct.P102   = P102;
    z_struct.m102   = m102;
    z_struct.H102   = H102;
    z_struct.Cp102  = Cp102;
    z0_struct.T102  = 3.3695634260764;
    z0_struct.P102  = 1.3000000000000;
    z0_struct.m102  = 7.8339242347537;
    z0_struct.H102  = 1.2431483811399;
    z0_struct.Cp102 = 82.399749899671;
    
    T103  = MX.sym('T103');                               % Temperature [K]
    P103  = MX.sym('P103');                                 % Pressure [Pa]
    m103  = MX.sym('m103');                       % Molar flow rate [mol/s] 
    H103  = MX.sym('H103');                                      % Enthalpy
    Cp103 = MX.sym('Cp103');                    % Heat capacity [J/(mol.K)] 
    z_struct.T103  = T103;
    z_struct.P103  = P103;
    z_struct.m103  = m103;
    z_struct.H103  = H103;
    z_struct.Cp103 = Cp103;
    
    T104 = MX.sym('T104');                                % Temperature [K]
    P104 = MX.sym('P104');                                  % Pressure [Pa]
    m104 = MX.sym('m104');                        % Molar flow rate [mol/s] 
    H104 = MX.sym('H104');                                       % Enthalpy     
    z_struct.T104 = T104;
    z_struct.P104 = P104;
    z_struct.m104 = m104;
    z_struct.H104 = H104;
    
    Wcomp_P = MX.sym('Wcomp_P');         % compressor power consumption [W]
    Qcond_P = MX.sym('Qcond_P');             % heat transfered in condenser
    Qhex_PE = MX.sym('Qhex_PE');              % heat from ethane to propane
    Qhex_PM = MX.sym('Qhex_PM');             % heat from methane to propane
    Qhex_PL = MX.sym('Qhex_PL');                 % heat from lng to propane
    z_struct.Wcomp_P = Wcomp_P;
    z_struct.Qcond_P = Qcond_P;
    z_struct.Qhex_PE = Qhex_PE;
    z_struct.Qhex_PM = Qhex_PM;
    z_struct.Qhex_PL = Qhex_PL;
    
    %% ETHANE
    T201  = MX.sym('T201');                               % Temperature [K]
    P201  = MX.sym('P201');                                % Pressure [MPa]
    m201  = MX.sym('m201');                       % Molar flow rate [mol/s]
    H201  = MX.sym('H201');                                      % Enthalpy
    Cp201 = MX.sym('Cp201');                    % Heat capacity [J/(mol.K)]  
    Z201  = MX.sym('Z201');                        % Compressibility factor
    z_struct.T201  = T201;
    z_struct.P201  = P201;
    z_struct.m201  = m201;
    z_struct.H201  = H201;
    z_struct.Cp201 = Cp201;
    z_struct.Z201  = Z201;
    
    T202  = MX.sym('T202');                               % Temperature [K]
    P202  = MX.sym('P202');                                 % Pressure [Pa]
    m202  = MX.sym('m202');                       % Molar flow rate [mol/s]
    H202  = MX.sym('H202');                                      % Enthalpy
    Cp202 = MX.sym('Cp202');                    % Heat capacity [J/(mol.K)] 
    z_struct.T202  = T202;
    z_struct.P202  = P202;
    z_struct.m202  = m202;
    z_struct.H202  = H202;
    z_struct.Cp202 = Cp202;
    
    T203  = MX.sym('T203');                               % Temperature [K]
    P203  = MX.sym('P203');                                 % Pressure [Pa]
    m203  = MX.sym('m203');                       % Molar flow rate [mol/s] 
    H203  = MX.sym('H203');                                      % Enthalpy
    Cp203 = MX.sym('Cp203');                    % Heat capacity [J/(mol.K)] 
    z_struct.T203  = T203;
    z_struct.P203  = P203;
    z_struct.m203  = m203;
    z_struct.H203  = H203;
    z_struct.Cp203 = Cp203;
    
    T204  = MX.sym('T204');                               % Temperature [K]
    P204  = MX.sym('P204');                                 % Pressure [Pa]
    m204  = MX.sym('m204');                       % Molar flow rate [mol/s] 
    H204  = MX.sym('H204');                                      % Enthalpy     
    Cp204 = MX.sym('Cp204');                    % Heat capacity [J/(mol.K)] 
    z_struct.T204  = T204;
    z_struct.P204  = P204;
    z_struct.m204  = m204;
    z_struct.H204  = H204;
    z_struct.Cp204 = Cp204;
    
    T205 = MX.sym('T205');                                % Temperature [K]
    P205 = MX.sym('P205');                                  % Pressure [Pa]
    m205 = MX.sym('m205');                        % Molar flow rate [mol/s] 
    H205 = MX.sym('H205');                                       % Enthalpy   
    z_struct.T205 = T205;
    z_struct.P205 = P205;
    z_struct.m205 = m205;
    z_struct.H205 = H205;
    
    Wcomp_E = MX.sym('Wcomp_E');                         % Compressor power
    Qcond_E = MX.sym('Qcond_E');                           % Condenser duty
    Qhex_EP = MX.sym('Qhex_EP');    % Heat transferred in EP heat-exchanger
    Qhex_EM = MX.sym('Qhex_EM');    % Heat transferred in EM heat-exchanger
    Qhex_EL = MX.sym('Qhex_EL');    % Heat transferred in EL heat-exchanger
    z_struct.Wcomp_E = Wcomp_E;
    z_struct.Qcond_E = Qcond_E;
    z_struct.Qhex_EP = Qhex_EP;
    z_struct.Qhex_EM = Qhex_EM;
    z_struct.Qhex_EL = Qhex_EL;
    
    %% METHANE
    T301  = MX.sym('T301');                               % Temperature [K]
    P301  = MX.sym('P301');                                % Pressure [MPa]
    m301  = MX.sym('m301');                       % Molar flow rate [mol/s]
    H301  = MX.sym('H301');                                      % Enthalpy
    Cp301 = MX.sym('Cp301');                    % Heat capacity [J/(mol.K)]  
    Z301  = MX.sym('Z301');                        % Compressibility factor
    z_struct.T301  = T301; 
    z_struct.P301  = P301;
    z_struct.m301  = m301;
    z_struct.H301  = H301;
    z_struct.Cp301 = Cp301;
    z_struct.Z301  = Z301;
    
    T302  = MX.sym('T302');                               % Temperature [K]
    P302  = MX.sym('P302');                                 % Pressure [Pa]
    m302  = MX.sym('m302');                       % Molar flow rate [mol/s]
    H302  = MX.sym('H302');                                      % Enthalpy
    Cp302 = MX.sym('Cp302');                    % Heat capacity [J/(mol.K)] 
    z_struct.T302  = T302;
    z_struct.P302  = P302;
    z_struct.m302  = m302;
    z_struct.H302  = H302;
    z_struct.Cp302 = Cp302;
    
    T303  = MX.sym('T303');                               % Temperature [K]
    P303  = MX.sym('P303');                                 % Pressure [Pa]
    m303  = MX.sym('m303');                       % Molar flow rate [mol/s] 
    H303  = MX.sym('H303');                                      % Enthalpy
    Cp303 = MX.sym('Cp303');                    % Heat capacity [J/(mol.K)] 
    z_struct.T303  = T303;
    z_struct.P303  = P303;
    z_struct.m303  = m303;
    z_struct.H303  = H303;
    z_struct.Cp303 = Cp303;
    
    T304  = MX.sym('T304');                               % Temperature [K]
    P304  = MX.sym('P304');                                 % Pressure [Pa]
    m304  = MX.sym('m304');                       % Molar flow rate [mol/s] 
    H304  = MX.sym('H304');                                      % Enthalpy     
    Cp304 = MX.sym('Cp304');                    % Heat capacity [J/(mol.K)] 
    z_struct.T304  = T304;
    z_struct.P304  = P304;
    z_struct.m304  = m304;
    z_struct.H304  = H304;
    z_struct.Cp304 = Cp304;
    
    T305  = MX.sym('T305');                               % Temperature [K]
    P305  = MX.sym('P305');                                 % Pressure [Pa]
    m305  = MX.sym('m305');                       % Molar flow rate [mol/s] 
    H305  = MX.sym('H305');                                      % Enthalpy    
    Cp305 = MX.sym('Cp305');                    % Heat capacity [J/(mol.K)] 
    z_struct.T305  = T305;
    z_struct.P305  = P305;
    z_struct.m305  = m305;
    z_struct.H305  = H305;
    z_struct.Cp305 = Cp305;
    
    T306 = MX.sym('T306');                                % Temperature [K]
    P306 = MX.sym('P306');                                  % Pressure [Pa]
    m306 = MX.sym('m306');                        % Molar flow rate [mol/s] 
    H306 = MX.sym('H306');                                       % Enthalpy    
    z_struct.T306 = T306;
    z_struct.P306 = P306;
    z_struct.m306 = m306;
    z_struct.H306 = H306;

    Wcomp_M = MX.sym('Wcomp_M');                         % Compressor power
    Qcond_M = MX.sym('Qcond_M');                           % Condenser duty
    Qhex_MP = MX.sym('Qhex_MP');    % Heat transferred in MP heat-exchanger
    Qhex_ME = MX.sym('Qhex_ME');    % Heat transferred in ME heat-exchanger
    Qhex_ML = MX.sym('Qhex_ML');    % Heat transferred in ML heat-exchanger
    z_struct.Wcomp_M = Wcomp_M;
    z_struct.Qcond_M = Qcond_M;
    z_struct.Qhex_MP = Qhex_MP;
    z_struct.Qhex_ME = Qhex_ME;
    z_struct.Qhex_ML = Qhex_ML;
       
    %% LNG
    T401 = MX.sym('T401');
    H401 = MX.sym('H401');
    T402 = MX.sym('T402');
    H402 = MX.sym('H402');
    T403 = MX.sym('T403');
    H403 = MX.sym('H403');
    T404 = MX.sym('T404');
    H404 = MX.sym('H404');
    Qhex_LP = MX.sym('Qhex_LP');    % Heat transferred in LP heat-exchanger
    Qhex_LE = MX.sym('Qhex_LE');    % Heat transferred in LE heat-exchanger
    Qhex_LM = MX.sym('Qhex_LM');    % Heat transferred in LM heat-exchanger
    z_struct.T401 = T401;
    z_struct.H401 = H401;
    z_struct.T402 = T402;
    z_struct.H402 = H402;
    z_struct.T403 = T403;
    z_struct.H403 = H403;
    z_struct.T404 = T404;
    z_struct.H404 = H404;
    z_struct.Qhex_LP = Qhex_LP;
    z_struct.Qhex_LE = Qhex_LE;
    z_struct.Qhex_LM = Qhex_LM;
    
    %% LOG-MEAN TEMPERATURE DIFFERENCES
    UA_PE = 2.688;
    UA_PM = 0.584;
    UA_EM = 1.5871;
    UA_LP = 0.29090;
    UA_LE = 0.39866;
    UA_LM = 0.75*1.4325;
    dTlm_PE = MX.sym('dTlm_PE');           % Log-mean temp. diff. in PE-HEX
    dTlm_PM = MX.sym('dTlm_PM');           % Log-mean temp. diff. in PM-HEX
    dTlm_EM = MX.sym('dTlm_EM');           % Log-mean temp. diff. in EM-HEX     
    dTlm_LP = MX.sym('dTlm_LP');           % Log-mean temp. diff. in LP-HEX
    dTlm_LE = MX.sym('dTlm_LE');           % Log-mean temp. diff. in LE-HEX
    dTlm_LM = MX.sym('dTlm_LM');           % Log-mean temp. diff. in LM-HEX
    z_struct.dTlm_PE = dTlm_PE;
    z_struct.dTlm_PM = dTlm_PM;
    z_struct.dTlm_EM = dTlm_EM;
    z_struct.dTlm_LP = dTlm_LP;
    z_struct.dTlm_LE = dTlm_LE;
    z_struct.dTlm_LM = dTlm_LM;
    
    nu_P = MX.sym('nu_P');
    nu_E = MX.sym('nu_E');
    nu_M = MX.sym('nu_M');
    head_P = MX.sym('head_P');
    head_E = MX.sym('head_E');
    head_M = MX.sym('head_M');
    qsuction_P  = MX.sym('q_P');
    qsuction_E  = MX.sym('q_E');
    qsuction_M  = MX.sym('q_M');
    z_struct.nu_P = nu_P;
    z_struct.nu_E = nu_E;
    z_struct.nu_M = nu_M;
    z_struct.head_P = head_P;
    z_struct.head_E = head_E;
    z_struct.head_M = head_M;
    z_struct.qsuction_P = qsuction_P;
    z_struct.qsuction_E = qsuction_E;
    z_struct.qsuction_M = qsuction_M;
      
    X_P = MX.sym('X_P');
    X_E = MX.sym('X_E');
    X_M = MX.sym('X_M');
    N_P = MX.sym('N_P');
    N_E = MX.sym('N_E');
    N_M = MX.sym('N_M'); 
    z_struct.X_P = X_P;
    z_struct.X_E = X_E;
    z_struct.X_M = X_M;
    z_struct.N_P = N_P;
    z_struct.N_E = N_E;
    z_struct.N_M = N_M;
    
    %% Define equations 
    algs = struct();
    
    %% PROPANE
    algs.T101  = T101 - Tsat_P(P101);
    algs.m101  = m101 - X_P*Cv_P*sqrt(P102-P101);
    algs.H101  = H101*1E5 - HvapLo_P(T101)*m101;
    algs.Cp101 = Cp101 - heatCapacity(xPropane,T101);
    algs.Z101  = Z101 - compressibilityFactor(xPropane,T101,P101,Z101);
    
    gamma12_P  = 1/2*( Cp101/(Cp101-R) + Cp102/(Cp102-R));
    k12_P      = nu_P*gamma12_P/(gamma12_P-1);
    head12_P   = 1E2*(k12_P*Z101*R/(g*molarMass(xPropane))*(T102-T101));
    
    algs.T102  = T102/T101 - (P102/P101)^(1/k12_P);
    %algs.P102  = m102 - 7.8339242347537;
    algs.m102  = m102 - m101;
    algs.H102  = H102*1E5 - (HvapHi_P(Tsat_P(P102)) + 0.5*(Cp102+heatCapacity(xPropane,Tsat_P(P102)))*(T102-Tsat_P(P102))*100)*m102;
    algs.Cp102 = Cp102 - heatCapacity(xPropane,T102);
    
    %algs.T103  = T103-3.18;
    algs.T103  = T103-Tsat_P(P103);
    %algs.T103  = 2.98-Tsat_P(P103);
    algs.P103  = P103-P102;
    algs.m103  = m103-m102;
    %Cp103 = 100;
    %algs.H103  = H103*1E5 - (HliqHi_P(P103) + Cp103*(T103-Tsat_P(P103))*100)*m103;
    algs.H103  = H103*1E5 - HliqHi_P(P103)*m103;
    %algs.Cp103 = Cp103 - 100;
    
    algs.T104  = T104 - T101;
    algs.P104  = P104 - P101;
    Cv_P       = 14.4234;
    algs.m104  = m104 - m101;
    algs.H104  = H104 - H103;
    
    algs.Wcomp_P = Wcomp_P - (H102-H101);        
    algs.Qcond_P = Qcond_P - (H103-H102);
    algs.Qhex_PE = Qhex_PE + Qhex_EP;
    algs.Qhex_PM = Qhex_PM + Qhex_MP;
    algs.Qhex_PL = Wcomp_P + Qcond_P + Qhex_PE + Qhex_PM + Qhex_PL; 
    
    %% ETHANE
    algs.T201  = T201 - Tsat_E(P201);
    algs.m201  = m201 - m205;
    algs.H201  = H201*1E5 - HvapLo_E(T201)*m201;
    algs.Cp201 = Cp201 - heatCapacity(xEthane,T201);
    algs.Z201  = Z201 - compressibilityFactor(xEthane,T201,P201,Z201);
    
    gamma12_E  = 1/2*( Cp201/(Cp201-R) + Cp202/(Cp202-R));
    k12_E      = nu_E*gamma12_E/(gamma12_E-1);
    head12_E   = 1E2*(k12_E*Z201*R/(g*molarMass(xEthane))*(T202-T201));
    
    algs.T202  = T202/T201 - (P202/P201)^(1/k12_E);
    %algs.P202  = m202 - 4.9453224102487;
    algs.m202  = m202 - m201;
    algs.H202  = H202*1E5 - (HvapHi_E(Tsat_E(P202)) + 0.5*(Cp202+heatCapacity(xEthane,Tsat_E(P202)))*(T202-Tsat_E(P202))*100)*m202;
    algs.Cp202 = Cp202 - heatCapacity(xEthane,T202);
    
    %algs.T203  = T203-2.9815;
    %algs.T203  = T203-Tsat_E(P203);
    algs.P203  = P203-P202;
    algs.m203  = m203-m202;
    algs.H203  = H203*1E5 - (HvapHi_E(Tsat_E(P203)) + 0.5*(Cp203+heatCapacity(xEthane,Tsat_E(P203)))*(T203-Tsat_E(P203))*100)*m203;
    %algs.H203  = H203*1E5 - HvapHi_E(T203)*m203;
    algs.Cp203 = Cp203 - heatCapacity(xEthane,T203);
    
    algs.T204  = Qhex_PE - UA_PE*dTlm_PE;
    algs.T204_2 = T204-Tsat_E(P204);
    algs.P204  = P204-P203;
    algs.m204  = m204-m203;
    %algs.H204  = H204*1E5 - (HliqHi_E(Tsat_E(P204)) + Cp204*(T204-Tsat_E(P204))*100)*m204; %                  
    algs.H204  = H204*1E5 - HliqHi_E(P204)*m204;
    algs.Cp204 = Cp204 - 95;
    
    algs.T205  = T205 - T201;
    algs.P205  = P205 - P201;
    Cv_E = 6.941871774004133;
    algs.m205  = m205 - X_E*Cv_E*sqrt(P204-P205);
    algs.H205  = H205 - H204;
    
    algs.Wcomp_E = Wcomp_E - (H202-H201);        
    algs.Qcond_E = Qcond_E - (H203-H202);
    algs.Qhex_EP = Qhex_EP - (H204-H203);
    algs.Qhex_EM = Qhex_EM + Qhex_ME;
    algs.Qhex_EL = Wcomp_E + Qcond_E + Qhex_EP + Qhex_EM + Qhex_EL ; 
    
    %% METHANE
    algs.T301  = T301 - Tsat_M(P301);
    algs.m301  = m301 - m306; 
    algs.H301  = H301*1E5 - HvapLo_M(T301)*m301;
    algs.Cp301 = Cp301 - heatCapacity(xMethane,T301);
    algs.Z301  = Z301 - compressibilityFactor(xMethane,T301,P301,Z301);
    
    gamma12_M  = 1/2*( Cp301/(Cp301-R) + Cp302/(Cp302-R));
    k12_M      = nu_M*gamma12_M/(gamma12_M-1);
    head12_M   = 1E2*(k12_M*Z301*R/(g*molarMass(xMethane))*(T302-T301));
    
    algs.T302  = T302/T301 - (P302/P301)^(1/k12_M);
    %algs.P302  = m302 - 7.9113396506688;
    algs.m302  = m302 - m301;
    algs.H302  = H302*1E5 - (HvapHi_M(Tsat_M(P302)) + 0.5*(Cp302+heatCapacity(xMethane,Tsat_M(P302)))*(T302-Tsat_M(P302))*100)*m302;
    algs.Cp302 = Cp302 - heatCapacity(xMethane,T302);
    
    %algs.T303  = T303-2.9815;
    %algs.T303  = T303-Tsat_M(P303);
    %algs.T303  = 2.98-Tsat_M(P303);
    algs.P303  = P303-P302;
    algs.m303  = m303-m302;
    algs.H303  = H303*1E5 - (HvapHi_M(Tsat_M(P303)) + 0.5*(Cp303+heatCapacity(xMethane,Tsat_M(P303)))*(T303-Tsat_M(P303))*100)*m303;
    %algs.H303  = H303*1E5 - HvapHi_M(T303)*m303;
    algs.Cp303 = Cp303 - heatCapacity(xMethane,T303);
    
    algs.T304  = Qhex_PM - UA_PM*dTlm_PM;
    algs.P304  = P304-P303;
    algs.m304  = m304-m303;
    algs.H304  = H304*1E5 - (HvapHi_M(Tsat_M(P304)) + 0.5*(Cp304+heatCapacity(xMethane,Tsat_M(P304)))*(T304-Tsat_M(P304))*100)*m304;  
    algs.Cp304 = Cp304 - heatCapacity(xMethane,T304);

    algs.T305  = Qhex_EM - UA_EM*dTlm_EM;
    algs.P305  = P305-P304;
    algs.m305  = m305-m304;
    %algs.H305  = H305*1E5 - (HliqHi_M(Tsat_M(P305)) + Cp305*(T305-Tsat_M(P305))*100)*m305; %HliqHi(T10)*m10;                  
    algs.H305  = H305*1E5 - HliqHi_M(Tsat_M(P305))*m305;
    algs.Cp305 = Cp305 - 95;
    
    algs.T306  = T306 - T301;
    algs.P306  = P306 - P301;
    Cv_M = 8.900952521083784;
    algs.m306  = m306 - X_M*Cv_M*sqrt(P305-P306);
    algs.H306  = H306 - H305;
    
    algs.Wcomp_M = Wcomp_M - (H302-H301);        
    algs.Qcond_M = Qcond_M - (H303-H302);
    algs.Qhex_MP = Qhex_MP - (H304-H303);
    algs.Qhex_ME = Qhex_ME - (H305-H304);
    algs.Qhex_ML = Wcomp_M + Qcond_M + Qhex_MP + Qhex_ME + Qhex_ML ; 
    
    %% LNG
    algs.T401 = T401 - 2.98;
    algs.H401 = H401*1E5 - (4348.7*T401-13430)*m_LNG;  
    algs.T402 = Qhex_PL - UA_LP*dTlm_LP;
    
    algs.H402 = H402*1E5 - (4522.5*T402-13835)*m_LNG;
    algs.T403 = Qhex_EL - UA_LE*dTlm_LE;
    
    heavi  = @(x) exp(200*x)/(1+exp(200*x));
    F_H403 = @(T) heavi(1.855-T)*( 9912.5*T-27757) + ...
                  heavi(T-1.855)*(11131.0*T-26943);
    algs.H403 = H403*1E5 - F_H403(T403)*m_LNG;
    algs.T404 = Qhex_ML - UA_LM*dTlm_LM;
    
    algs.H404 = H404*1E5 - (5664.6*T404-20725)*m_LNG;
    algs.Qhex_LP = Qhex_LP - (H402-H401);
    algs.Qhex_LE = Qhex_LE - (H403-H402);
    algs.Qhex_LM = Qhex_LM - (H404-H403);
    
    algs.Qhex_LP2 = Qhex_LP + Qhex_PL;
    algs.Qhex_LE2 = Qhex_LE + Qhex_EL;
    algs.Qhex_LM2 = Qhex_LM + Qhex_ML;
    
    %% LOG-MEAN TEMPERATURE DIFFERENCES
    dT1 = T203 - T101;
    dT2 = T204 - T101;
    algs.dTlm_PE = dTlm_PE-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T303 - T101;
    dT2 = T304 - T101;
    algs.dTlm_PM = dTlm_PM-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T304 - T201;
    dT2 = T305 - T201;
    algs.dTlm_EM = dTlm_EM-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T401 - T101;
    dT2 = T402 - T101;
    algs.dTlm_LP = dTlm_LP-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T402 - T201;
    dT2 = T403 - T201;
    algs.dTlm_LE = dTlm_LE-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T403 - T301;
    dT2 = T404 - T301;
    algs.dTlm_LM = dTlm_LM-(dT1-dT2)/log(dT1/dT2);
    
    algs.head_P = head_P-head12_P;
    algs.head_E = head_E-head12_E;
    algs.head_M = head_M-head12_M;
    
    algs.qsuction_P = qsuction_P - m101*R*T101/P101;
    algs.qsuction_E = qsuction_E - m201*R*T201/P201;
    algs.qsuction_M = qsuction_M - m301*R*T301/P301;
    
    % Compressor curves / efficiencies
    C11 = 1.9928;
    C12 = 16791;
    C13 = 8756.4;
    p   = 2.11;
    hs1 = head_P/N_P^p / (12.348301829605/0.9^p) * 7400;
    f1  = (C11*hs1 - C12)/(hs1-C13);
    q   = 1.79;
    f1  = f1/1.5*(1200/N_P^q);  
    algs.hs1 = (qsuction_P)/N_P^q - f1;
    
    e1  = 6.47*10^(-5);
    e2  = 0.353; 
    e3  = 7.101*10^(-4); 
    e4  = 6.5963;
    %hs1 = hs1+1900;
    algs.nu_P = nu_P - (-2*0.81+3*(e1*hs1+e2-10^(e3*hs1-e4)));
    
    C11 = 1.9928;
    C12 = 16791;
    C13 = 8756.4;
    p   = 2.11;
    hs1 = head_E/N_E^p / (26/0.9^p) * 7400;
    f2  = (C11*hs1 - C12)/(hs1-C13);
    q   = 1.79;
    f2  = f2/1.507*(1059.699190790036/N_E^q); 
    algs.hs2 = qsuction_E/N_E^q - f2;
    
    e1  = 6.47*10^(-5);
    e2  = 0.353; 
    e3  = 7.101*10^(-4); 
    e4  = 6.5963;
    algs.nu_E = nu_E - (-2*0.78+3*(e1*hs1+e2-10^(e3*hs1-e4)));
 
    C11  = 1.9928;
    C12  = 16791;
    C13  = 8756.4;
    p    = 2.11;
    hs1  = head_M/N_M^p / (34.871148119645/0.9^p) * 7600;
    f3   = (C11*hs1 - C12)/(hs1-C13);
    q    = 1.79;
    f3   = f3/1.507*(541.165069883052/N_M^q); 
    algs.hs3 = qsuction_M/N_M^q - f3;
    
    e1  = 6.47*10^(-5);
    e2  = 0.353; 
    e3  = 7.101*10^(-4); 
    e4  = 6.5963;
    %hs1 = hs1 - 700;
    algs.nu_M = nu_M - (-2*0.75+3*(e1*hs1+e2-10^(e3*hs1-e4)));
    
   % algs.tes1 = m101 - 2*m201;
    %algs.tes2 = m101 - 3*m301;
    
    %algs.tes1 = (P102/P101) - 0.25*(P202/P201);
    %algs.tes2 = (P102/P101) - 0.50*(P302/P301);
    
    %algs.cool1 = T103-2.98;
    %algs.cool2 = T203-2.98;
    %algs.cool3 = T303-2.98;
    
    %% SOLVE
    vars = casadi_struct2vec(z_struct);
    eqs = casadi_struct2vec(algs);

    %rf_function = casadi.Function('rf_function',{vars},{eqs});

    %rf = casadi.rootfinder('rf','newton',rf_function);

    z0 = [2.3540045179488,... T101  01
          0.1200000000000,... P101  02
          7.8363683797626,... m101  03
          0.7989875905702,... H101  04
          62.358736052051,... Cp101 05
          0.9521804809881,... Z101  06
          ...
          3.3695634260764,... T102  07
          1.3000000000000,... P102  08
          7.8339242347537,... m102  09
          1.2431483811399,... H102  10
          82.399749899671,... Cp102 11
          ...
          2.9815000000000,... T103  12
          1.3000000000000,... P103  13
          7.8339242347537,... m103  14
          -0.092997234619,... H103  15
          100.00000000000,... Cp103 16
          ...
          2.3540045179488,... T104  17
          0.1200000000000,... P104  18
          7.8339242347537,... m104  19
          -0.092997234619,... H104  20
          ...
          0.4444099929345,... Pow12 21
          -1.336145615759,... Q_23  22
          0.6710775745974,... Q_PE  23
          0.1468337073871,... Q_PM  24
          0.0738243408399,... Q_PL  25
          ...
          1.8044311048023,... T201  26
          0.0700000000000,... P201  27
          4.9453224102487,... m201  28
          0.3131596685250,... H201  29
          36.741218084402,... Cp201 30 
          0.9687473968447,... Z201  31
          ... 
          4.0767740713396,... T202  32
          2.1000000000000,... P202  33
          4.9453224102487,... m202  34
          0.7775431395702,... H202  35
          58.014910411070,... Cp202 36 
          ...
          2.9815000000000,... T203  37
          2.1000000000000,... P203  38
          4.9453224102487,... m203  39
          0.4840501135759,... H203  40
          52.951448566865,... Cp203 41
          ...
          2.4202755573702,... T204  42
          2.1000000000000,... P204  43
          4.9453224102487,... m204  44
          -0.187027461022,... H204  45
          95.000000000000,... Cp204 46
          ...
          1.8044311048023,... T205  47
          0.0700000000000,... P205  48
          4.9453224102487,... m205  49
          -0.187027461021,... H205  50
          ...
          0.4643834710452,... Wcomp_E  51
          -0.293493025994,... Qcond_E  52
          -0.671077574597,... Qhex_EP  53
          0.3997110089019,... Qhex_EM  54
          0.1004761206446,... Qhex_EP  55
          ...
          1.1521844369387,... T301  56
          0.1400000000000,... P301  57
          7.9113396506688,... m301  58
          -0.500731057530,... H301  59
          22.611763965144,... Cp301 60
          0.9378177318035,... Z301  61
          ...
          3.8220071979512,... T302  62
          3.3            ,... P302  63
          7.9            ,... m302  64
          0.6            ,... H302  65
          40.380449297738,... Cp302 66
          ...
          2.9815000000000,... T303  67
          3.3            ,... P303  68
          7.9            ,... m303  69
          -0.1           ,... H303  70
          35.063198473765,... Cp303 71
          ...
          2.4217214026460,... T304  72
          3.3            ,... P304  73
          7.9            ,... m304  74
          -0.3           ,... H304  75
          31.380870194286,... Cp304 76
          ...
          1.8747972400149,... T305  77
          3.3            ,... P305  78
          7.9            ,... m305  79
          -0.7           ,... H305  80
          95.000000000000,... Cp305 81
          ...
          1.1521844369387,... T306  82
          0.10           ,... P306  83
          7.9            ,... m306  84
          -0.736283278489,... H306  85
          ...
          0.5603942267359,... Pow12 86
          -0.249401731406,... Q_air 87
          -0.146833707387,... Q_PE  88
          -0.399711008902,... Q_PM  89
          0.2355522209593,... Q_PL  90
          ...
          2.9800000000000,... T401  91
          -0.014, ...         H401  92
          2.4240029693226,... T402  93
          -0.088, ...         H402  94
          1.8744303554407,... T403  95
          -0.18, ...          H403  96
          1.2218422458928,... T404  97
          -0.40, ...          H404  98
          -0.073824340840,... Q_LP  99
          -0.100476120645,... Q_LE  100
          -0.235552220959,... Q_LM  101
          ...
          0.2496568357877,... -     102  
          0.2514275811422,... -     103
          0.2518499205481,... -     104
          0.2537791022340,... -     105
          0.2520346175805,... -     106
          0.2790303727064,... -     107
          ...
          0.8            ,... efficiency
          0.8            ,... efficiency
          0.8            ,... efficiency
          12             ,... head
          26             ,... head 
          35             ,... head 
          1277           ,... q - suction volumetric flow rate 
          1059           ,... q - suction volumetric flow rate
          541            ,... q - suction volumetric flow rate
          ...
          0.50           ,... X_P    108
          0.50           ,... X_E    109
          0.50           ,... X_M    110
          0.90           ,... N_P    108
          0.90           ,... N_E    109
          0.90           ,... N_M    110
          ]';

    %z0 = full(rf(z0));
    
    prob_struct   = struct();
    prob_struct.x = vars;
    prob_struct.g = eqs;
    %prob_struct.f = (Wcomp_P + Wcomp_E + Wcomp_M);
    %prob_struct.f = m101*head_P + m201*head_E + m301*head_M;
    prob_struct.f = -(nu_P + nu_E + nu_M);
    
    nlpfun = Function('nlp',prob_struct,char('x'),char('f','g'));

    opts = struct('warn_initial_bounds',false,                              ...
                  'gather_stats',true,                                      ...
                  'print_time',false,                                       ...
                  'ipopt',struct('linear_solver','mumps',                   ...
                                 'max_iter',5E2,                            ...
                                 'warm_start_init_point','yes',             ...
                                 'mu_init',1E-5,                            ...
                                 'replace_bounds','yes',                    ...
                                 'print_level',5)                           ...
                  ); 

    solver = nlpsol('solver','ipopt',prob_struct,opts);
    
    %% Constraints
    
    lbx = casadi_vec(z_struct, -inf...
        ...,'T404',0.00 ...
        ,'T103',2.98 ...
        ,'T203',2.93 ...
        ,'T303',2.98 ...
        ,'P101',0.15 ...
        ,'P201',0.10 ...
        ,'P301',0.10 ...
        ...,'N_P',0.9 ...
        ...,'N_E',0.9 ...
        ...,'N_M',0.9 ...
        ...,'Z_P',0.5 ...
        ...,'Z_E',0.5 ...
        ...,'Z_M',0.5 ...
        ,'Wcomp_P',0.0 ...
        ,'Wcomp_E',0.0 ...
        ,'Wcomp_M',0.0 ...
        );

    ubx = casadi_vec(z_struct, inf...
        ,'T404',1.22 ...
        ...,'N_P',0.9 ...
        ...,'N_E',0.9 ...
        ...,'N_M',0.9 ...
        ...,'Z_P',0.5 ...
        ...,'Z_E',0.5 ...
        ...,'Z_M',0.5 ...
        ,'Qcond_P',0.0 ...
        ,'Qcond_E',0.0 ...
        ,'Qcond_M',0.0 ...
        );
    
    %z0 = asdf;
    
    lbg = zeros(size(prob_struct.g));
    ubg = zeros(size(prob_struct.g));
    
    %lbg(end) = -inf;
    
    res = solver('x0',z0,                                               ...
             'lbg',lbg,                       ...
             'ubg',ubg,                       ...
             'lbx',lbx,                      ...
             'ubx',ubx                       ...
             ); 
    
    sol = res.x;
    
    full(sol(end-5:end))
    
    sol(end-14)
    sol(end-13)
    sol(end-12)
    
    drawPFD_full(casadi_vec2struct(z_struct,full(sol)))
%end