function [rf] = LNG_model_full_eval(varargin)
    
    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])
    
    if nargin == 0 || isempty(varargin{1})
        disturbances = [2.98,7,1,1,0,0,0];
    else
        disturbances = varargin{1};
    end
    
    %close all
    
    %% Extra functions

    m_LNG   = 3.074;
    
    Cv_P = 13;
    Cv_E = 8.4;
    Cv_M = 3.4;
    
    xPropane = [0,0,1,0,0,0,0,0,0,0,0]';
    xEthane  = [0,1,0,0,0,0,0,0,0,0,0]';
    xMethane = [1,0,0,0,0,0,0,0,0,0,0]';
    
    g = 9.80665;                            % gravitational constant [m/s2]
    R = 8.31446;                       % universal gas constant [J/(K.mol)]
    
    Tsat_P = @(P) 0.24817*log10(P)^2 + 0.93582*log10(P) + 3.0053;
    Tsat_E = @(P) 0.21080*log10(P)^2 + 0.77135*log10(P) + 2.4141;
    Tsat_M = @(P) 0.13110*log10(P)^2 + 0.51530*log10(P) + 1.4966;
    
    Hliq_P = @(P) 4.8879*P^6 + 50.621*P^5 - 970.54*P^4 +...
                  4838.4*P^3 - 11110*P^2 + 15991*P - 9992.7;
    Hliq_E = @(P) -18.124*P^6 + 319.1*P^5 - 2146.5*P^4 +...
                   7061.9*P^3 - 12132*P^2 + 12605*P - 9447.4;
    Hliq_M = @(P) -7.6457*P^6 + 138.42*P^5 - 943.33*P^4 +...
                   3138.6*P^3 - 5488.7*P^2 + 5974.2*P - 15187;
    
    Hvap_P = @(P) -56.405*P^6 + 721.52*P^5 - 3644.6*P^4 + ...
                   9271.9*P^3 - 12980*P^2 + 11005*P + 9036.9;
    Hvap_E = @(P) -25.103*P^6 + 356.06*P^5 - 1973*P^4 + ...
                   5412.9*P^3 - 7884.9*P^2 + 6200.8*P + 5906;
    Hvap_M = @(P) -59.04*P^4 + 518.47*P^3 - 1650.9*P^2 + 2117.7*P - 6622.6;
    
    %% Define the streams
    z_struct = struct();
    
    %% PROPANE
    T101  = MX.sym('T101');                               % Temperature [K]
    P101  = MX.sym('P101');                                % Pressure [MPa]
    m101  = MX.sym('m101');                       % Molar flow rate [mol/s]
    H101  = MX.sym('H101');                                      % Enthalpy
    Cp101 = MX.sym('Cp101');                    % Heat capacity [J/(mol.K)]  
    Z101  = MX.sym('Z101');                        % Compressibility factor
    z_struct.T101   = T101;
    z_struct.P101   = P101;
    z_struct.m101   = m101;
    z_struct.H101   = H101;
    z_struct.Cp101  = Cp101;
    z_struct.Z101   = Z101;
    z0_struct.T101  = 2.287549407827489;
    z0_struct.P101  = 0.084846214195264;
    z0_struct.m101  = 6.258777385892930;
    z0_struct.H101  = 0.618534230814700;
    z0_struct.Cp101 = 60.99455792843873;
    z0_struct.Z101  = 0.962825583574374;
  
    T102  = MX.sym('T102');                               % Temperature [K]
    P102  = MX.sym('P102');                                 % Pressure [Pa]
    m102  = MX.sym('m102');                       % Molar flow rate [mol/s]
    H102  = MX.sym('H102');                                      % Enthalpy
    Cp102 = MX.sym('Cp102');                    % Heat capacity [J/(mol.K)] 
    z_struct.T102   = T102;
    z_struct.P102   = P102;
    z_struct.m102   = m102;
    z_struct.H102   = H102;
    z_struct.Cp102  = Cp102;
    z0_struct.T102  = 3.291322325948146;
    z0_struct.P102  = 0.939221874964043;
    z0_struct.m102  = 6.258777385892930;
    z0_struct.H102  = 0.981538633334020;
    z0_struct.Cp102 = 80.90954684800177;
    
    T103  = MX.sym('T103');                               % Temperature [K]
    P103  = MX.sym('P103');                                 % Pressure [Pa]
    m103  = MX.sym('m103');                       % Molar flow rate [mol/s] 
    H103  = MX.sym('H103');                                      % Enthalpy
    Cp103 = MX.sym('Cp103');                    % Heat capacity [J/(mol.K)] 
    z_struct.T103  = T103;
    z_struct.P103  = P103;
    z_struct.m103  = m103;
    z_struct.H103  = H103;
    z_struct.Cp103 = Cp103;
    z0_struct.T103  = 2.979999971627246;
    z0_struct.P103  = 0.939221874964043;
    z0_struct.m103  = 6.258777385892930;
    z0_struct.H103  = -0.09264972521635;
    z0_struct.Cp103 = 100;
    
    T104 = MX.sym('T104');                                % Temperature [K]
    P104 = MX.sym('P104');                                  % Pressure [Pa]
    m104 = MX.sym('m104');                        % Molar flow rate [mol/s] 
    H104 = MX.sym('H104');                                       % Enthalpy     
    z_struct.T104 = T104;
    z_struct.P104 = P104;
    z_struct.m104 = m104;
    z_struct.H104 = H104;
    z0_struct.T104 = 2.287549407827056;
    z0_struct.P104 = 0.084846214195055;
    z0_struct.m104 = 6.258777385895725;
    z0_struct.H104 = -0.09264972521639;
    
    Wcomp_P = MX.sym('Wcomp_P');         % compressor power consumption [W]
    Qcond_P = MX.sym('Qcond_P');             % heat transfered in condenser
    Qhex_PE = MX.sym('Qhex_PE');              % heat from ethane to propane
    Qhex_PM = MX.sym('Qhex_PM');             % heat from methane to propane
    Qhex_PL = MX.sym('Qhex_PL');                 % heat from lng to propane
    z_struct.Wcomp_P = Wcomp_P;
    z_struct.Qcond_P = Qcond_P;
    z_struct.Qhex_PE = Qhex_PE;
    z_struct.Qhex_PM = Qhex_PM;
    z_struct.Qhex_PL = Qhex_PL;
    z0_struct.Wcomp_P = 0.363004402519739;
    z0_struct.Qcond_P = -1.07418835855099;
    z0_struct.Qhex_PE = 0.560166052061916;
    z0_struct.Qhex_PM = 0.066362577374736;
    z0_struct.Qhex_PL = 0.084655326594595;
    
    %% ETHANE
    T201  = MX.sym('T201');                               % Temperature [K]
    P201  = MX.sym('P201');                                % Pressure [MPa]
    m201  = MX.sym('m201');                       % Molar flow rate [mol/s]
    H201  = MX.sym('H201');                                      % Enthalpy
    Cp201 = MX.sym('Cp201');                    % Heat capacity [J/(mol.K)]  
    Z201  = MX.sym('Z201');                        % Compressibility factor
    z_struct.T201  = T201;
    z_struct.P201  = P201;
    z_struct.m201  = m201;
    z_struct.H201  = H201;
    z_struct.Cp201 = Cp201;
    z_struct.Z201  = Z201;
    z0_struct.T201  = 1.742854949493515;
    z0_struct.P201  = 0.037519276457307;
    z0_struct.m201  = 3.603615718109626;
    z0_struct.H201  = 0.220823516750669;
    z0_struct.Cp201 = 35.85794502608649;
    z0_struct.Z201  = 0.981135151730953;
    
    T202  = MX.sym('T202');                               % Temperature [K]
    P202  = MX.sym('P202');                                 % Pressure [Pa]
    m202  = MX.sym('m202');                       % Molar flow rate [mol/s]
    H202  = MX.sym('H202');                                      % Enthalpy
    Cp202 = MX.sym('Cp202');                    % Heat capacity [J/(mol.K)] 
    z_struct.T202  = T202;
    z_struct.P202  = P202;
    z_struct.m202  = m202;
    z_struct.H202  = H202;
    z_struct.Cp202 = Cp202;
    z0_struct.T202  = 3.532350850292113;
    z0_struct.P202  = 0.739749319654359;
    z0_struct.m202  = 3.603615718109626;
    z0_struct.H202  = 0.510908963682559;
    z0_struct.Cp202 = 60.09750383113714;
    
    T203  = MX.sym('T203');                               % Temperature [K]
    P203  = MX.sym('P203');                                 % Pressure [Pa]
    m203  = MX.sym('m203');                       % Molar flow rate [mol/s] 
    H203  = MX.sym('H203');                                      % Enthalpy
    Cp203 = MX.sym('Cp203');                    % Heat capacity [J/(mol.K)] 
    z_struct.T203   = T203;
    z_struct.P203   = P203;
    z_struct.m203   = m203;
    z_struct.H203   = H203;
    z_struct.Cp203  = Cp203;
    z0_struct.T203  = 2.979999997091538;
    z0_struct.P203  = 0.739749319654359;
    z0_struct.m203  = 3.603615718109626;
    z0_struct.H203  = 0.398790010463715;
    z0_struct.Cp203 = 52.93160630674856;
    
    T204  = MX.sym('T204');                               % Temperature [K]
    P204  = MX.sym('P204');                                 % Pressure [Pa]
    m204  = MX.sym('m204');                       % Molar flow rate [mol/s] 
    H204  = MX.sym('H204');                                      % Enthalpy     
    z_struct.T204   = T204;
    z_struct.P204   = P204;
    z_struct.m204   = m204;
    z_struct.H204   = H204;
    z0_struct.T204  = 2.316731255770486;
    z0_struct.P204  = 0.739749319654359;
    z0_struct.m204  = 3.603615718109626;
    z0_struct.H204  = -0.16137604159820;
    
    T205 = MX.sym('T205');                                % Temperature [K]
    P205 = MX.sym('P205');                                  % Pressure [Pa]
    m205 = MX.sym('m205');                        % Molar flow rate [mol/s] 
    H205 = MX.sym('H205');                                       % Enthalpy   
    z_struct.T205  = T205;
    z_struct.P205  = P205;
    z_struct.m205  = m205;
    z_struct.H205  = H205;
    z0_struct.T205 = 1.742854949493515;
    z0_struct.P205 = 0.037519276457307;
    z0_struct.m205 = 3.603615718109626;
    z0_struct.H205 = -0.16137604159820;
    
    Wcomp_E = MX.sym('Wcomp_E');                         % Compressor power
    Qcond_E = MX.sym('Qcond_E');                           % Condenser duty
    Qhex_EP = MX.sym('Qhex_EP');    % Heat transferred in EP heat-exchanger
    Qhex_EM = MX.sym('Qhex_EM');    % Heat transferred in EM heat-exchanger
    Qhex_EL = MX.sym('Qhex_EL');    % Heat transferred in EL heat-exchanger
    z_struct.Wcomp_E  = Wcomp_E;
    z_struct.Qcond_E  = Qcond_E;
    z_struct.Qhex_EP  = Qhex_EP;
    z_struct.Qhex_EM  = Qhex_EM;
    z_struct.Qhex_EL  = Qhex_EL;
    z0_struct.Wcomp_E = 0.290085446931890;
    z0_struct.Qcond_E = -0.11211895321885;
    z0_struct.Qhex_EP = -0.56016605206192;
    z0_struct.Qhex_EM = 0.176311841496288;
    z0_struct.Qhex_EL = 0.205887716852582;
    
    %% METHANE
    T301  = MX.sym('T301');                               % Temperature [K]
    P301  = MX.sym('P301');                                % Pressure [MPa]
    m301  = MX.sym('m301');                       % Molar flow rate [mol/s]
    H301  = MX.sym('H301');                                      % Enthalpy
    Cp301 = MX.sym('Cp301');                    % Heat capacity [J/(mol.K)]  
    Z301  = MX.sym('Z301');                        % Compressibility factor
    z_struct.T301   = T301; 
    z_struct.P301   = P301;
    z_struct.m301   = m301;
    z_struct.H301   = H301;
    z_struct.Cp301  = Cp301;
    z_struct.Z301   = Z301;
    z0_struct.T301  = 1.184307376060752; 
    z0_struct.P301  = 0.178392148044451;
    z0_struct.m301  = 3.087085481106691;
    z0_struct.H301  = -0.19431576744535;
    z0_struct.Cp301 = 22.84079949432389;
    z0_struct.Z301  = 0.927715867152673;
    
    T302  = MX.sym('T302');                               % Temperature [K]
    P302  = MX.sym('P302');                                 % Pressure [Pa]
    m302  = MX.sym('m302');                       % Molar flow rate [mol/s]
    H302  = MX.sym('H302');                                      % Enthalpy
    Cp302 = MX.sym('Cp302');                    % Heat capacity [J/(mol.K)] 
    z_struct.T302   = T302;
    z_struct.P302   = P302;
    z_struct.m302   = m302;
    z_struct.H302   = H302;
    z_struct.Cp302  = Cp302;
    z0_struct.T302  = 3.518192650036316;
    z0_struct.P302  = 3.192152938685106;
    z0_struct.m302  = 3.087085481106691;
    z0_struct.H302  = -0.00874556303906;
    z0_struct.Cp302 = 38.48778301000818;
    
    T303  = MX.sym('T303');                               % Temperature [K]
    P303  = MX.sym('P303');                                 % Pressure [Pa]
    m303  = MX.sym('m303');                       % Molar flow rate [mol/s] 
    H303  = MX.sym('H303');                                      % Enthalpy
    Cp303 = MX.sym('Cp303');                    % Heat capacity [J/(mol.K)] 
    z_struct.T303   = T303;
    z_struct.P303   = P303;
    z_struct.m303   = m303;
    z_struct.H303   = H303;
    z_struct.Cp303  = Cp303;
    z0_struct.T303  = 2.979999989331724;
    z0_struct.P303  = 3.192152938685106;
    z0_struct.m303  = 3.087085481106691;
    z0_struct.H303  = -0.06953047743099;
    z0_struct.Cp303 = 35.05348179653313;
    
    T304  = MX.sym('T304');                               % Temperature [K]
    P304  = MX.sym('P304');                                 % Pressure [Pa]
    m304  = MX.sym('m304');                       % Molar flow rate [mol/s] 
    H304  = MX.sym('H304');                                      % Enthalpy     
    Cp304 = MX.sym('Cp304');                    % Heat capacity [J/(mol.K)] 
    z_struct.T304   = T304;
    z_struct.P304   = P304;
    z_struct.m304   = m304;
    z_struct.H304   = H304;
    z_struct.Cp304  = Cp304;
    z0_struct.T304  = 2.325350030280275;
    z0_struct.P304  = 3.192152938685106;
    z0_struct.m304  = 3.087085481106691;
    z0_struct.H304  = -0.13589305480572;
    z0_struct.Cp304 = 30.73554390239080;
    
    T305  = MX.sym('T305');                               % Temperature [K]
    P305  = MX.sym('P305');                                 % Pressure [Pa]
    m305  = MX.sym('m305');                       % Molar flow rate [mol/s] 
    H305  = MX.sym('H305');                                      % Enthalpy    
    z_struct.T305   = T305;
    z_struct.P305   = P305;
    z_struct.m305   = m305;
    z_struct.H305   = H305;
    z0_struct.T305  = 1.789666883886850;
    z0_struct.P305  = 3.192152938685106;
    z0_struct.m305  = 3.087085481106691;
    z0_struct.H305  = -0.31220489630201;
    
    T306 = MX.sym('T306');                                % Temperature [K]
    P306 = MX.sym('P306');                                  % Pressure [Pa]
    m306 = MX.sym('m306');                        % Molar flow rate [mol/s] 
    H306 = MX.sym('H306');                                       % Enthalpy    
    z_struct.T306  = T306;
    z_struct.P306  = P306;
    z_struct.m306  = m306;
    z_struct.H306  = H306;
    z0_struct.T306 = 1.184307376060752;
    z0_struct.P306 = 0.178392148044451;
    z0_struct.m306 = 3.087085481106691;
    z0_struct.H306 = -0.31220489630201;

    Wcomp_M = MX.sym('Wcomp_M');                         % Compressor power
    Qcond_M = MX.sym('Qcond_M');                           % Condenser duty
    Qhex_MP = MX.sym('Qhex_MP');    % Heat transferred in MP heat-exchanger
    Qhex_ME = MX.sym('Qhex_ME');    % Heat transferred in ME heat-exchanger
    Qhex_ML = MX.sym('Qhex_ML');    % Heat transferred in ML heat-exchanger
    z_struct.Wcomp_M  = Wcomp_M;
    z_struct.Qcond_M  = Qcond_M;
    z_struct.Qhex_MP  = Qhex_MP;
    z_struct.Qhex_ME  = Qhex_ME;
    z_struct.Qhex_ML  = Qhex_ML;
    z0_struct.Wcomp_M = 0.185570204406292;
    z0_struct.Qcond_M = -0.06078491439193;
    z0_struct.Qhex_MP = -0.06636257737474;
    z0_struct.Qhex_ME = -0.17631184149629;
    z0_struct.Qhex_ML = 0.117889128856658;
       
    %% LNG
    T401    = MX.sym('T401');
    H401    = MX.sym('H401');
    T402    = MX.sym('T402');
    H402    = MX.sym('H402');
    T403    = MX.sym('T403');
    H403    = MX.sym('H403');
    T404    = MX.sym('T404');
    H404    = MX.sym('H404');
    Qhex_LP = MX.sym('Qhex_LP');    % Heat transferred in LP heat-exchanger
    Qhex_LE = MX.sym('Qhex_LE');    % Heat transferred in LE heat-exchanger
    Qhex_LM = MX.sym('Qhex_LM');    % Heat transferred in LM heat-exchanger
    z_struct.T401     = T401;
    z_struct.H401     = H401;
    z_struct.T402     = T402;
    z_struct.H402     = H402;
    z_struct.T403     = T403;
    z_struct.H403     = H403;
    z_struct.T404     = T404;
    z_struct.H404     = H404;
    z_struct.Qhex_LP  = Qhex_LP;
    z_struct.Qhex_LE  = Qhex_LE;
    z_struct.Qhex_LM  = Qhex_LM;
    z0_struct.T401    = 2.980000000000000;
    z0_struct.H401    = -0.01447466676000;
    z0_struct.T402    = 2.346094343186153;
    z0_struct.H402    = -0.09912999335460;
    z0_struct.T403    = 1.799188434229178;
    z0_struct.H403    = -0.30501771020718;
    z0_struct.T404    = 1.230000011578514;
    z0_struct.H404    = -0.42290683906384;
    z0_struct.Qhex_LP = -0.08465532659460;
    z0_struct.Qhex_LE = -0.20588771685258;
    z0_struct.Qhex_LM = -0.11788912885666;
    
    %% LOG-MEAN TEMPERATURE DIFFERENCES
    UA_PE = 2.674439684513716;
    UA_PM = 0.294778127170467;
    UA_EM = 0.829808335718488;
    UA_LP = 0.329916759332703;
    UA_LE = 0.892593986669065;
    UA_LM = 0.538401346748178;
    
    dTlm_PE = MX.sym('dTlm_PE');           % Log-mean temp. diff. in PE-HEX
    dTlm_PM = MX.sym('dTlm_PM');           % Log-mean temp. diff. in PM-HEX
    dTlm_EM = MX.sym('dTlm_EM');           % Log-mean temp. diff. in EM-HEX     
    dTlm_LP = MX.sym('dTlm_LP');           % Log-mean temp. diff. in LP-HEX
    dTlm_LE = MX.sym('dTlm_LE');           % Log-mean temp. diff. in LE-HEX
    dTlm_LM = MX.sym('dTlm_LM');           % Log-mean temp. diff. in LM-HEX
    z_struct.dTlm_PE  = dTlm_PE;
    z_struct.dTlm_PM  = dTlm_PM;
    z_struct.dTlm_EM  = dTlm_EM;
    z_struct.dTlm_LP  = dTlm_LP;
    z_struct.dTlm_LE  = dTlm_LE;
    z_struct.dTlm_LM  = dTlm_LM;
    z0_struct.dTlm_PE = 0.209451742473590;
    z0_struct.dTlm_PM = 0.225127210121528;
    z0_struct.dTlm_EM = 0.212472969849873;
    z0_struct.dTlm_LP = 0.256596017631297;
    z0_struct.dTlm_LE = 0.230662227090397;
    z0_struct.dTlm_LM = 0.218961430109122;
    
    nu_P        = MX.sym('nu_P');
    nu_E        = MX.sym('nu_E');
    nu_M        = MX.sym('nu_M');
    head_P      = MX.sym('head_P');
    head_E      = MX.sym('head_E');
    head_M      = MX.sym('head_M');
    qsuction_P  = MX.sym('q_P');
    qsuction_E  = MX.sym('q_E');
    qsuction_M  = MX.sym('q_M');
    z_struct.nu_P        = nu_P;
    z_struct.nu_E        = nu_E;
    z_struct.nu_M        = nu_M;
    z_struct.head_P      = head_P;
    z_struct.head_E      = head_E;
    z_struct.head_M      = head_M;
    z_struct.qsuction_P  = qsuction_P;
    z_struct.qsuction_E  = qsuction_E;
    z_struct.qsuction_M  = qsuction_M;
    z0_struct.nu_P       = 0.792078054235500;
    z0_struct.nu_E       = 0.792566857264233;
    z0_struct.nu_M       = 0.788772283948777;
    z0_struct.head_P     = 12.27885679514012;
    z0_struct.head_E     = 20.89263711519747;
    z0_struct.head_M     = 30.31971949742990;
    z0_struct.qsuction_P = 1.403012585989862;
    z0_struct.qsuction_E = 1.391807941952341;
    z0_struct.qsuction_M = 0.170400711082942;

    X_P = MX.sym('X_P');
    X_E = MX.sym('X_E');
    X_M = MX.sym('X_M');
    N_P = MX.sym('N_P');
    N_E = MX.sym('N_E');
    N_M = MX.sym('N_M'); 
    z_struct.X_P  = X_P;
    z_struct.X_E  = X_E;
    z_struct.X_M  = X_M;
    z_struct.N_P  = N_P;
    z_struct.N_E  = N_E;
    z_struct.N_M  = N_M;
    z0_struct.X_P = 0.520860853801476;
    z0_struct.X_E = 0.511940494489961;
    z0_struct.X_M = 0.523016450499974;
    z0_struct.N_P = 0.894125593477176;
    z0_struct.N_E = 0.874182604142192;
    z0_struct.N_M = 0.884990151557131;
    
    %% Define equations 
    algs = struct();
    
    %% PROPANE
    algs.T101  = T101 - Tsat_P(P101);
    algs.m101  = m101 - X_P*Cv_P*sqrt(P102-P101);
    algs.H101  = H101*1E5 - Hvap_P(P101)*m101;
    algs.Cp101 = Cp101 - heatCapacity(xPropane,T101);
    algs.Z101  = Z101 - compressibilityFactor(xPropane,T101,P101,Z101);
    
    gamma12_P  = 1/2*( Cp101/(Cp101-R) + Cp102/(Cp102-R));
    k12_P      = nu_P*gamma12_P/(gamma12_P-1);
    head12_P   = 1E2*(k12_P*Z101*R/(g*molarMass(xPropane))*(T102-T101));
    
    algs.T102  = T102/T101 - (P102/P101)^(1/k12_P);
    algs.m102  = m102 - m101;
    algs.H102  = H102*1E5 - (Hvap_P(P102) + 0.5*(Cp102+...
        heatCapacity(xPropane,Tsat_P(P102)))*(T102-Tsat_P(P102))*100)*m102;
    algs.Cp102 = Cp102 - heatCapacity(xPropane,T102);

    %Tsub_P = MX.sym('Tsub_P');
    %z_struct.Tsub_P  = Tsub_P;
    %z0_struct.Tsub_P = 0;
    Tsub_P = 0;
    algs.T103  = T103-(Tsat_P(P103)-Tsub_P);
    algs.P103  = P103-P102;
    algs.m103  = m103-m102;
    algs.H103  = H103*1E5 - (Hliq_P(P103)-Tsub_P*100)*m103;
    algs.Cp103 = Cp103 - 100;
    
    algs.T104  = T104 - T101;
    algs.P104  = P104 - P101;
    algs.m104  = m104 - m101;
    algs.H104  = H104 - H103;
    
    algs.Wcomp_P = Wcomp_P - (H102-H101);        
    algs.Qcond_P = Qcond_P - (H103-H102);
    algs.Qhex_PE = Qhex_PE + Qhex_EP;
    algs.Qhex_PM = Qhex_PM + Qhex_MP;
    algs.Qhex_PL = Wcomp_P + Qcond_P + Qhex_PE + Qhex_PM + Qhex_PL; 
    
    %% ETHANE
    algs.T201  = T201 - Tsat_E(P201);
    algs.m201  = m201 - m205;
    algs.H201  = H201*1E5 - Hvap_E(P201)*m201;
    algs.Cp201 = Cp201 - heatCapacity(xEthane,T201);
    algs.Z201  = Z201 - compressibilityFactor(xEthane,T201,P201,Z201);
    
    gamma12_E  = 1/2*( Cp201/(Cp201-R) + Cp202/(Cp202-R));
    k12_E      = nu_E*gamma12_E/(gamma12_E-1);
    head12_E   = 1E2*(k12_E*Z201*R/(g*molarMass(xEthane))*(T202-T201));
    
    algs.T202  = T202/T201 - (P202/P201)^(1/k12_E);
    algs.m202  = m202 - m201;
    algs.H202  = H202*1E5 - (Hvap_E(P202) + 0.5*(Cp202+...
        heatCapacity(xEthane,Tsat_E(P202)))*(T202-Tsat_E(P202))*100)*m202;
    algs.Cp202 = Cp202 - heatCapacity(xEthane,T202);
    
    algs.P203  = P203-P202;
    algs.m203  = m203-m202;
    algs.H203  = H203*1E5 - (Hvap_E(P203) + 0.5*(Cp203+...
        heatCapacity(xEthane,Tsat_E(P203)))*(T203-Tsat_E(P203))*100)*m203;
    algs.Cp203 = Cp203 - heatCapacity(xEthane,T203);
    
    algs.T204  = Qhex_PE - UA_PE*dTlm_PE;
    algs.T204_2 = T204-Tsat_E(P204);
    algs.P204  = P204-P203;
    algs.m204  = m204-m203;   
    algs.H204  = H204*1E5 - Hliq_E(P204)*m204;
    
    algs.T205  = T205 - T201;
    algs.P205  = P205 - P201;
    algs.m205  = m205 - X_E*Cv_E*sqrt(P204-P205);
    algs.H205  = H205 - H204;
    
    algs.Wcomp_E = Wcomp_E - (H202-H201);        
    algs.Qcond_E = Qcond_E - (H203-H202);
    algs.Qhex_EP = Qhex_EP - (H204-H203);
    algs.Qhex_EM = Qhex_EM + Qhex_ME;
    algs.Qhex_EL = Wcomp_E + Qcond_E + Qhex_EP + Qhex_EM + Qhex_EL ; 
    
    %% METHANE
    algs.T301  = T301 - Tsat_M(P301);
    algs.m301  = m301 - m306; 
    algs.H301  = H301*1E5 - Hvap_M(P301)*m301;
    algs.Cp301 = Cp301 - heatCapacity(xMethane,T301);
    algs.Z301  = Z301 - compressibilityFactor(xMethane,T301,P301,Z301);
    
    gamma12_M  = 1/2*( Cp301/(Cp301-R) + Cp302/(Cp302-R));
    k12_M      = nu_M*gamma12_M/(gamma12_M-1);
    head12_M   = 1E2*(k12_M*Z301*R/(g*molarMass(xMethane))*(T302-T301));
    
    algs.T302  = T302/T301 - (P302/P301)^(1/k12_M);
    algs.m302  = m302 - m301;
    algs.H302  = H302*1E5 - (Hvap_M(P302) + 0.5*(Cp302+...
        heatCapacity(xMethane,Tsat_M(P302)))*(T302-Tsat_M(P302))*100)*m302;
    algs.Cp302 = Cp302 - heatCapacity(xMethane,T302);
    
    algs.P303  = P303-P302;
    algs.m303  = m303-m302;
    algs.H303  = H303*1E5 - (Hvap_M(P303) + 0.5*(Cp303+...
        heatCapacity(xMethane,Tsat_M(P303)))*(T303-Tsat_M(P303))*100)*m303;
    algs.Cp303 = Cp303 - heatCapacity(xMethane,T303);
    
    algs.T304  = Qhex_PM - UA_PM*dTlm_PM;
    algs.P304  = P304-P303;
    algs.m304  = m304-m303;
    algs.H304  = H304*1E5 - (Hvap_M(P304) + 0.5*(Cp304+...
        heatCapacity(xMethane,Tsat_M(P304)))*(T304-Tsat_M(P304))*100)*m304;  
    algs.Cp304 = Cp304 - heatCapacity(xMethane,T304);

    %Tsub_M = MX.sym('Tsub_M');
    %z_struct.Tsub_M  = Tsub_M;
    %z0_struct.Tsub_M = 0;
    Tsub_M = 0;
    algs.T305  = Qhex_EM - UA_EM*dTlm_EM;
    algs.T305_2 = T305 - (Tsat_M(P305)-Tsub_M);
    algs.P305  = P305-P304;
    algs.m305  = m305-m304;               
    algs.H305  = H305*1E5 - (Hliq_M(P305)+Tsub_M*20)*m305;
    
    algs.T306  = T306 - T301;
    algs.P306  = P306 - P301;
    algs.m306  = m306 - X_M*Cv_M*sqrt(P305-P306);
    algs.H306  = H306 - H305;
    
    algs.Wcomp_M = Wcomp_M - (H302-H301);        
    algs.Qcond_M = Qcond_M - (H303-H302);
    algs.Qhex_MP = Qhex_MP - (H304-H303);
    algs.Qhex_ME = Qhex_ME - (H305-H304);
    algs.Qhex_ML = Wcomp_M + Qcond_M + Qhex_MP + Qhex_ME + Qhex_ML ; 
    
    %% LNG
    algs.T401 = T401 - disturbances(1);
    algs.H401 = H401*1E5 - (4348.7*T401-13430)*m_LNG;  
    algs.T402 = Qhex_PL - UA_LP*dTlm_LP;
    
    algs.H402 = H402*1E5 - (4522.5*T402-13835)*m_LNG;
    algs.T403 = Qhex_EL - UA_LE*dTlm_LE;
    
    heavi  = @(x) exp(200*x)/(1+exp(200*x));
    F_H403 = @(T) heavi(1.855-T)*( 9912.5*T-27757) + ...
                  heavi(T-1.855)*(11131.0*T-26943);
    algs.H403 = H403*1E5 - F_H403(T403)*m_LNG;
    algs.T404 = Qhex_ML - UA_LM*dTlm_LM;
    
    algs.H404 = H404*1E5 - (5664.6*T404-20725)*m_LNG;
    algs.Qhex_LP = Qhex_LP - (H402-H401);
    algs.Qhex_LE = Qhex_LE - (H403-H402);
    algs.Qhex_LM = Qhex_LM - (H404-H403);
    
    algs.Qhex_LP2 = Qhex_LP + Qhex_PL;
    algs.Qhex_LE2 = Qhex_LE + Qhex_EL;
    algs.Qhex_LM2 = Qhex_LM + Qhex_ML;
    
    %% LOG-MEAN TEMPERATURE DIFFERENCES
    dT1 = T203 - T101;
    dT2 = T204 - T101;
    algs.dTlm_PE = dTlm_PE-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T303 - T101;
    dT2 = T304 - T101;
    algs.dTlm_PM = dTlm_PM-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T304 - T201;
    dT2 = T305 - T201;
    algs.dTlm_EM = dTlm_EM-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T401 - T101;
    dT2 = T402 - T101;
    algs.dTlm_LP = dTlm_LP-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T402 - T201;
    dT2 = T403 - T201;
    algs.dTlm_LE = dTlm_LE-(dT1-dT2)/log(dT1/dT2);
    
    dT1 = T403 - T301;
    dT2 = T404 - T301;
    algs.dTlm_LM = dTlm_LM-(dT1-dT2)/log(dT1/dT2);
    
    algs.head_P = head_P-head12_P;
    algs.head_E = head_E-head12_E;
    algs.head_M = head_M-head12_M;
    
    algs.qsuction_P = 1E3*qsuction_P - m101*R*T101/P101;
    algs.qsuction_E = 1E3*qsuction_E - m201*R*T201/P201;
    algs.qsuction_M = 1E3*qsuction_M - m301*R*T301/P301;
    
    % Compressor curves / efficiencies
    C11 = 1.9928;
    C12 = 35.1962;
    C13 = 18.3546;
    p   = 2.11;
    q   = 1.79;
    hs1 = head_P/(N_P-0.05)^p;
    algs.hs1 = qsuction_P/(N_P-0.05)^q - (C11*hs1 - C12)/(hs1-C13);
    
    e1 = 0.061733;
    e2 = -0.074+disturbances(5); 
    e3 = 0.338767;
    e4 = 6.5963;
    algs.nu_P = nu_P - (e1*hs1+e2-2*10^(e3*hs1-e4));
    
    C11 = 1.9928;
    C12 = 63.00;
    C13 = 32.855;
    p   = 2.11;
    q   = 1.79;
    hs2 = head_E/N_E^p;
    algs.hs2 = qsuction_E/N_E^q - (C11*hs2 - C12)/(hs2-C13);
    
    e1 = 0.03448682;
    e2 = -0.074+disturbances(6);
    e3 = 0.18925109;
    e4 = 6.5963;
    algs.nu_E = nu_E - (e1*hs2+e2-2*10^(e3*hs2-e4));
    
    C11 = 0.27146;
    C12 = 11.765;
    C13 = 45.040;
    p   = 2.11;
    q   = 1.79;
    hs3 = head_M/(N_M+0.08)^p;
    algs.hs3 = qsuction_M/(N_M+0.08)^q - (C11*hs3 - C12)/(hs3-C13);
    
    e1 = 0.0251571;
    e2 = -0.074+disturbances(7);
    e3 = 0.138053;
    e4 = 6.5963;
    algs.nu_M = nu_M - (e1*hs3+e2-2*10^(e3*hs3-e4));
    

    
    algs.T404_2 = T404 - 1.23;  % Use N_M to control outlet temperature

    cost = MX.sym('cost');
    z_struct.cost = cost;
    algs.cost = cost - ((Wcomp_P + Wcomp_E + Wcomp_M) ...
              + disturbances(2)*m101*max(2.98-T103,0)^2 ...
              + disturbances(3)*m201*max(2.98-T203,0)^2 ...
              + disturbances(4)*m301*max(2.98-T303,0)^2);

    par_struct = struct();
    par_struct.Tcond_P = T103;
    par_struct.Tcond_E = T203;
    par_struct.Tcond_M = T303;
    par_struct.N_P = N_P; 
    par_struct.N_E = N_E; 

    z_struct  = rmfield(z_struct ,{'T103','T203','T303','N_P','N_E'});

    vars = casadi_struct2vec(z_struct);
    eqs = casadi_struct2vec(algs);
    pars = casadi_struct2vec(par_struct);

    rf_function = casadi.Function('rf_function',{vars,pars},{eqs});
    rf = casadi.rootfinder('rf','newton',rf_function);

end