function [] = drawPFD()
    close all
    fid = figure();
    set(fid, 'Position', [100, 100, 800, 800]);
    hold on
    
    % Draw the units
    drawCompressor(fid,-1,0,0.4,0,'black');
    drawCondenser(fid,0,1,0.4,0,'black');
    
    drawValve2(fid,1,1,0.4,0,'black');
    drawHEX(fid,2,0,0.4,0,'black');
    drawHEX(fid,2,-1,0.4,0,'black');
    drawHEX(fid,2,-3,0.4,0,'black');
    
    % Draw the connections
    plot([-0.9,-0.9,-0.2],[0.2,1.0,1.0],'blue')
    plot([0.2,0.8],[1.0,1.0],'blue')
    plot([1.2,2.0,2.0],[1.0,1.0,0.2],'blue')
    plot([2.0,2.0],[-0.2,-0.8],'blue')
    plot([2.0,2.0],[-1.2,-2.8],'blue')
    plot([2.0,2.0,-1.2,-1.2],[-3.2,-3.8,-3.8,-0.2],'blue')

    plot([1,1.8],[0,0],'blue--')
    plot([2.2,3],[0,0],'blue--')
    plot([1,1.8],[-1,-1],'blue--')
    plot([2.2,3],[-1,-1],'blue--')
    plot([1,1.8],[-3,-3],'blue--')
    plot([2.2,3],[-3,-3],'blue--')
    
    plot([-1,-1],[-0.8,-0.2],'magenta-.')
    plot([-1,-1],[0.2,0.8],'magenta-.')
    plot([0,0],[1.0,1.8],'red:','linewidth',1.2)
    plot([-1.6,-1],[0,0],'red:','linewidth',1.2)
    
    hold off
    axis equal
    xlim(1.05*xlim);
    ylim(1.05*ylim);
    
    xl = xlim;
    yl = ylim;
    pos = @(x,y) [(x-xl(1))/(xl(2)-xl(1)),(y-yl(1))/(yl(2)-yl(1))];
    opts = {'Units','normalized','HorizontalAlignment','center','VerticalAlignment','middle','FontSize',8};
    
    % Draw the text labels
    p = pos(1.7,0.3);
    text(p(1),p(2),sprintf('HEX\nP-M'),opts{:});
    p = pos(1.7,-0.7);
    text(p(1),p(2),sprintf('HEX\nP-E'),opts{:});
    p = pos(1.7,-2.7);
    text(p(1),p(2),sprintf('HEX\nP-LNG'),opts{:});
    p = pos(-1.3,0.35);
    text(p(1),p(2),sprintf('Compressor'),opts{:});
    p = pos(1,0.65);
    text(p(1),p(2),sprintf('JT-\nvalve'),opts{:});
    p = pos(0,0.65);
    text(p(1),p(2),sprintf('Condenser'),opts{:});
    
    
end

%% Define the units

function [xrot,yrot] = drawCompressor(fid,x,y,s,rot,c) 
    figure(fid)
    
    xv = [x-0.25*s,x+0.25*s,x+0.5*s,x-0.5*s];
    yv = [y+0.5*s,y+0.5*s,y-0.5*s,y-0.5*s];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,c)
end

function [xrot,yrot] = drawValve(fid,x,y,s,rot,c) 
    figure(fid)
    
    xv = [x-0.5*s,x-0.5*s,x+0.5*s,x+0.5*s];
    yv = [y-0.5*s,y+0.5*s,y-0.5*s,y+0.5*s];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,c)
end

function [xrot,yrot] = drawValve2(fid,x,y,s,rot,c) 
    figure(fid)
    
    xv = [x-0.5*s,x-0.5*s,x+0.5*s,x+0.5*s,x,x,x+0.25*s,x+0.20*s,x+0.07*s,x-0.07*s,x-0.20*s,x-0.25*s,x,x];
    yv = [y-0.5*s,y+0.5*s,y-0.5*s,y+0.5*s,y,y+0.5*s,y+0.5*s,y+0.65*s,y+0.7*s,y+0.7*s,y+0.65*s,y+0.5*s,y+0.5*s,y];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,c)
end

function [xrot,yrot] = drawCondenser(fid,x,y,s,rot,c) 
    figure(fid)
    
    th = -3*pi/4:pi/50:5*pi/4;
    xv = 0.5*s * cos(th) + x;
    yv = 0.5*s * sin(th) + y;
    
    xv = [x-0.75*s,...
         xv,...
         0.5*(xv(end)+x),...
         xv(end),...
         x+abs(x-xv(end)),...
         0.5*(0.5*s*cos(pi/4)+2*x),...
         x+0.60*s,...
         x+0.52*s,...
         x+0.75*s,...
         x+0.68*s,...
         x+0.60*s ];
     
    yv = [y-0.75*s,...
          yv,...
          0.5*(yv(end)+y),...
          y,...
          y,...
          0.5*(0.5*s*sin(pi/4)+2*y),...
          y+0.60*s,...
          y+0.68*s,...
          y+0.75*s,...
          y+0.52*s,...
          y+0.60*s];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,c)
end

function [xrot,yrot] = drawHEX(fid,x,y,s,rot,c) 
    figure(fid)
    
    th = -3*pi/4:pi/50:5*pi/4;
    xv = 0.5*s * cos(th) + x;
    yv = 0.5*s * sin(th) + y;
    
    xv = [xv,...
         0.5*(xv(end)+x),...
         xv(end),...
         x+abs(x-xv(end)),...
         0.5*(0.5*s*cos(pi/4)+2*x),...
         0.5*s*cos(pi/4)+x,...
         ];
     
    yv = [yv,...
          0.5*(yv(end)+y),...
          y,...
          y,...
          0.5*(0.5*s*sin(pi/4)+2*y),...
          0.5*s*sin(pi/4)+y,...
          ];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot-45);
    
    plot(xrot,yrot,c)
end

function [xrot,yrot] = rotate(xv,yv,x,y,rot) 
    R = [cosd(rot),-sind(rot); sind(rot),cosd(rot)];
    
    xrot = zeros(size(xv));
    yrot = zeros(size(yv));
    center = repmat([x;y],1,2);
    
    for i=1:length(xv)-1
       v = [xv(i),xv(i+1);yv(i),yv(i+1)];
       vo = R*(v - center) + center;
       xrot(i:i+1) = vo(1,:);
       yrot(i:i+1) = vo(2,:);
    end
end