function [] = drawPFD(varargin)
    %close all
    fid = figure('Name','Propane loop');
    clf(fid)
    %title('Propane loop')
    set(fid, 'Position', [100, 100, 1600, 900]);
    
    scale = 0.4;
    hold on
    
    % Draw the units
    opts = {'color','black'};
    Comp_P = drawCompressor(fid,-2,2,scale,0,opts); % P compressor
    Cond_P = drawCondenser(fid,1,3,scale,0,opts); % P condenser
    Valv_P = drawValve2(fid,2,3,scale,0,opts); % P JT-valve
    HEX_PE = drawHEX(fid,3,1,scale,0,opts); % HEX P-E
    HEX_PM = drawHEX(fid,3,-1,scale,0,opts); % HEX P-M
    HEX_PL = drawHEX(fid,3,-3,scale,0,opts); % HEX P-LNG
    
    opts = {'color',[0.8,0.8,0.8]};
    Comp_E = drawCompressor(fid,-1,0,scale,0,opts); % E compressor
    Valv_E = drawValve2(fid,4,1,scale,0,opts); % E JT-valve
    Cond_E = drawCondenser(fid,1,1,scale,0,opts); % E condenser
    HEX_EM = drawHEX(fid,5,-1,scale,0,opts); % HEX E-M
    HEX_EL = drawHEX(fid,5,-3,scale,0,opts); % HEX E-LNG
    
    Comp_M = drawCompressor(fid,0,-2,scale,0,opts);
    Valv_M = drawValve2(fid,6,-1,scale,0,opts);
    Cond_M = drawCondenser(fid,1,-1,scale,0,opts);
    HEX_ML = drawHEX(fid,7,-3,scale,0,opts);
    
    % Draw the connections
    opts            = {'Color','blue'};
    comp_shaft_opts = {'-.','Color',[1,0,1]};
    energy_opts     = {':','Color',[1,0,0]};
    % Propane loop
    connect(fid,Comp_P,'NE',Cond_P,'W' ,'|-',opts)
    connect(fid,Cond_P,'E' ,Valv_P,'W' ,'-' ,opts)
    connect(fid,Valv_P,'E' ,HEX_PE,'N' ,'-|',opts)
    connect(fid,HEX_PE,'S' ,HEX_PM,'N' ,'-' ,opts)
    connect(fid,HEX_PM,'S' ,HEX_PL,'N' ,'-' ,opts)
    tmp = struct('N',[0,-4]);
    connect(fid,HEX_PL,'S' ,tmp   ,'N' ,'|-',opts)
    connect(fid,tmp   ,'N' ,Comp_P,'SW','-|',opts)
    Comp_P_Shaft = struct();
    Comp_P_Shaft.N = Comp_P.N+[0, 0.5];
    Comp_P_Shaft.S = Comp_P.S+[0,-0.5];
    connect(fid,Comp_P,'N',Comp_P_Shaft,'N','-',comp_shaft_opts)
    connect(fid,Comp_P,'S',Comp_P_Shaft,'S','-',comp_shaft_opts)
    Cond_P_Pow = struct();
    Cond_P_Pow.N = Cond_P.NE+[0.5,0.5];
    connect(fid,Cond_P,'NE',Cond_P_Pow,'N','|-',energy_opts)
    
    
    opts            = {'Color',[0.8,0.8,1]};
    comp_shaft_opts = {'-.','Color',[1,0.8,1]};
    energy_opts     = {':','Color',[1,0.8,0.8]};
    % Ethane loop
    connect(fid,Comp_E,'NE',Cond_E,'W' ,'|-',opts)
    connect(fid,Cond_E,'E' ,HEX_PE,'W' ,'-' ,opts)
    connect(fid,HEX_PE,'E' ,Valv_E,'W' ,'-' ,opts)
    connect(fid,Valv_E,'E' ,HEX_EM,'N' ,'-|',opts)
    connect(fid,HEX_EM,'S' ,HEX_EL,'N' ,'-' ,opts)
    tmp2 = struct('N',[1,-5]);
    connect(fid,HEX_EL,'S' ,tmp2  ,'N' ,'|-',opts)
    connect(fid,tmp2  ,'N' ,Comp_E,'SW','-|',opts)
    Comp_E_Shaft = struct();
    Comp_E_Shaft.N = Comp_E.N+[0, 0.5];
    Comp_E_Shaft.S = Comp_E.S+[0,-0.5];
    connect(fid,Comp_E,'N',Comp_E_Shaft,'N','-',comp_shaft_opts)
    connect(fid,Comp_E,'S',Comp_E_Shaft,'S','-',comp_shaft_opts)
    Cond_E_Pow = struct();
    Cond_E_Pow.N = Cond_E.NE+[0.5,0.5];
    connect(fid,Cond_E,'NE',Cond_E_Pow,'N','|-',energy_opts)
    
    % Methane loop
    connect(fid,Comp_M,'NE',Cond_M,'W' ,'|-',opts)
    connect(fid,Cond_M,'E' ,HEX_PM,'W' ,'-' ,opts)
    connect(fid,HEX_PM,'E' ,HEX_EM,'W' ,'-' ,opts)
    connect(fid,HEX_EM,'E' ,Valv_M,'W' ,'-' ,opts)
    connect(fid,Valv_M,'E' ,HEX_ML,'N' ,'-|',opts)
    tmp3 = struct('N',[2,-6]);
    connect(fid,HEX_ML,'S' ,tmp3  ,'N' ,'|-',opts)
    connect(fid,tmp3  ,'N' ,Comp_M,'SW','-|',opts)
    Comp_M_Shaft = struct();
    Comp_M_Shaft.N = Comp_M.N+[0, 0.5];
    Comp_M_Shaft.S = Comp_M.S+[0,-0.5];
    connect(fid,Comp_M,'N',Comp_M_Shaft,'N','-',comp_shaft_opts)
    connect(fid,Comp_M,'S',Comp_M_Shaft,'S','-',comp_shaft_opts)
    Cond_M_Pow = struct();
    Cond_M_Pow.N = Cond_M.NE+[0.5,0.5];
    connect(fid,Cond_M,'NE',Cond_M_Pow,'N','|-',energy_opts)
    
    % LNG line
    tmp4 = struct('N',[2,-3]);
    connect(fid,tmp4  ,'N' ,HEX_PL,'W' ,'-' ,opts)
    connect(fid,HEX_PL,'E' ,HEX_EL,'W' ,'-' ,opts)
    connect(fid,HEX_EL,'E' ,HEX_ML,'W' ,'-' ,opts)
    tmp5 = struct('N',[8,-3]);
    connect(fid,HEX_ML,'E' ,tmp5  ,'N' ,'-' ,opts)
    
    hold off
    axis equal
    xlim(1.05*xlim);
    ylim(1.05*ylim);
    
    xl = xlim;
    yl = ylim;
    pos = @(x) [(x(1)-xl(1))/(xl(2)-xl(1)),(x(2)-yl(1))/(yl(2)-yl(1))];
    opts = {'Units','normalized','HorizontalAlignment','center','VerticalAlignment','middle','FontSize',8,'Interpreter','latex'};
    
    % Draw the text labels
    p = num2cell(pos(HEX_PE.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nP-E'),opts{:});
    p = num2cell(pos(HEX_PM.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nP-M'),opts{:});
    p = num2cell(pos(HEX_PL.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nP-LNG'),opts{:});
    p = num2cell(pos(Comp_P.NE+[0.60,0.25]));
    text(p{:},sprintf('Propane\ncompressor'),opts{:});
    p = num2cell(pos(Cond_P.S+[0,-0.40]));
    text(p{:},sprintf('Condenser'),opts{:});
    p = num2cell(pos(Valv_P.S+[0,-0.60]));
    text(p{:},sprintf('JT-\nvalve'),opts{:});
    %p = pos([0,0]);
    %text(p(1),p(2),sprintf('\\textbf{Propane loop}'),'Units','normalized','HorizontalAlignment','center','VerticalAlignment','middle','FontSize',12,'Interpreter','latex');
    
    
    opts = {'Units','normalized','HorizontalAlignment','left','VerticalAlignment','middle','FontSize',8,'EdgeColor','black','Interpreter','latex','backgroundcolor','white'};
    if ~nargin==0
        x0 = varargin{1};
        p = pos([-4,-1]);
        str = sprintf('\\textbf{Stream 1:}\n $$T=%0.3f$$\n $$P=%0.3f$$\n $$m=%0.3f$$\n $$H=%0.3f$$\n $$C_p=%0.3f$$\n $$Z=%0.3f$$',x0(3),x0(4),x0(5),x0(6),x0(7),x0(8));
        text(p(1),p(2),str,opts{:});
        
        p = pos([-4,1]);
        str = sprintf('\\textbf{Compressor:}\n $$N=%0.3f$$\n $$\\gamma=%0.3f$$\n $$k=%0.3f$$\n $$\\eta=%0.3f$$\n $$h=%0.2f$$\n $$W=%0.3f$$',x0(1),x0(22),x0(23),x0(6),x0(7),x0(8));
        text(p(1),p(2),str,opts{:});
        
        p = pos([-4,3]);
        str = sprintf('\\textbf{Stream 2:}\n $$T=%0.3f$$\n $$P=%0.3f$$\n $$m=%0.3f$$\n $$H=%0.3f$$\n $$C_p=%0.3f$$',x0(9),x0(10),x0(11),x0(12),x0(13));
        text(p(1),p(2),str,opts{:});

        p = pos([1,1.5]);
        str = sprintf('\\textbf{Stream 3:}\n $$T=%0.3f$$\n $$P=%0.3f$$\n $$m=%0.3f$$\n $$H=%0.3f$$',x0(14),x0(15),x0(16),x0(17));
        text(p(1),p(2),str,opts{:});
        
        p = pos([3.5,3]);
        str = sprintf('\\textbf{Stream 4:}\n $$T=%0.3f$$\n $$P=%0.3f$$\n $$m=%0.3f$$\n $$H=%0.3f$$',x0(18),x0(19),x0(20),x0(21));
        text(p(1),p(2),str,opts{:}); 
        
        p = num2cell(pos(HEX_PE.SE+[0.25,-0.25]));
        str = sprintf('$$Q_{PE}=%0.3f$$',x0(end-2));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_PM.SE+[0.25,-0.25]));
        str = sprintf('$$Q_{PM}=%0.3f$$',x0(end-1));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_PL.SE+[0.25,-0.25]));
        str = sprintf('$$Q_{PL}=%0.3f$$',x0(end));
        text(p{:},str,opts{:}); 
        
    end
    
end

%% Define the units

function coords = drawCompressor(fid,x,y,s,rot,opts) 
    figure(fid)
    
    xv = [x-0.25*s,x+0.25*s,x+0.5*s,x-0.5*s];
    yv = [y+0.5*s,y+0.5*s,y-0.5*s,y-0.5*s];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [(xrot(1)+xrot(4))*0.5,(yrot(1)+yrot(4))*0.5];
    coords.NW = [xrot(1),yrot(1)];
    coords.N  = [(xrot(1)+xrot(2))*0.5,(yrot(1)+yrot(2))*0.5];
    coords.NE = [xrot(2),yrot(2)];
    coords.E  = [(xrot(2)+xrot(3))*0.5,(yrot(2)+yrot(3))*0.5];
    coords.SE = [xrot(3),yrot(3)];
    coords.S  = [(xrot(3)+xrot(4))*0.5,(yrot(3)+yrot(4))*0.5];
    coords.SW = [xrot(4),yrot(4)];
end

function coords = drawValve(fid,x,y,s,rot,opts) 
    figure(fid)
    
    xv = [x-0.5*s,x-0.5*s,x+0.5*s,x+0.5*s];
    yv = [y-0.5*s,y+0.5*s,y-0.5*s,y+0.5*s];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [(xrot(1)+xrot(2))*0.5,(yrot(1)+yrot(2))*0.5];
    coords.NW = [xrot(1),yrot(1)];
    coords.N  = [(xrot(4)+xrot(5))*0.5,(yrot(4)+yrot(5))*0.5];
    coords.NE = [xrot(4),yrot(4)];
    coords.E  = [(xrot(3)+xrot(4))*0.5,(yrot(3)+yrot(4))*0.5];
    coords.SE = [xrot(3),yrot(3)];
    coords.S  = [(xrot(4)+xrot(5))*0.5,(yrot(4)+yrot(5))*0.5];
    coords.SW = [xrot(2),yrot(2)];
end

function coords = drawValve2(fid,x,y,s,rot,opts) 
    figure(fid)
    
    xv = [x-0.5*s,x-0.5*s,x+0.5*s,x+0.5*s,x,x,x+0.25*s,x+0.20*s,x+0.07*s,x-0.07*s,x-0.20*s,x-0.25*s,x,x];
    yv = [y-0.5*s,y+0.5*s,y-0.5*s,y+0.5*s,y,y+0.5*s,y+0.5*s,y+0.65*s,y+0.7*s,y+0.7*s,y+0.65*s,y+0.5*s,y+0.5*s,y];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [(xrot(1)+xrot(2))*0.5,(yrot(1)+yrot(2))*0.5];
    coords.NW = [xrot(1),yrot(1)];
    coords.N  = [(xrot(9)+xrot(10))*0.5,(yrot(9)+yrot(10))*0.5];
    coords.NE = [xrot(4),yrot(4)];
    coords.E  = [(xrot(3)+xrot(4))*0.5,(yrot(3)+yrot(4))*0.5];
    coords.SE = [xrot(3),yrot(3)];
    coords.S  = [xrot(end-1),yrot(end-1)];
    coords.SW = [xrot(2),yrot(2)];
end

function coords = drawCondenser(fid,x,y,s,rot,opts) 
    figure(fid)
    
    th = -3*pi/4:pi/32:5*pi/4;
    xv = 0.5*s * cos(th) + x;
    yv = 0.5*s * sin(th) + y;
    
    xv = [x-0.75*s,...
         xv,...
         0.5*(xv(end)+x),...
         xv(end),...
         x+abs(x-xv(end)),...
         0.5*(0.5*s*cos(pi/4)+2*x),...
         x+0.60*s,...
         x+0.52*s,...
         x+0.75*s,...
         x+0.68*s,...
         x+0.60*s ];
     
    yv = [y-0.75*s,...
          yv,...
          0.5*(yv(end)+y),...
          y,...
          y,...
          0.5*(0.5*s*sin(pi/4)+2*y),...
          y+0.60*s,...
          y+0.68*s,...
          y+0.75*s,...
          y+0.52*s,...
          y+0.60*s];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [xrot(58),yrot(58)]; 
    coords.NW = [xrot(50),yrot(50)]; 
    coords.N  = [xrot(42),yrot(42)]; 
    coords.NE = [xrot(end-2),yrot(end-2)];
    coords.E  = [xrot(26),yrot(26)];
    coords.SE = [xrot(18),yrot(18)];
    coords.S  = [xrot(10),yrot(10)];
    coords.SW = [xrot( 1),yrot( 1)];
end

function coords = drawHEX(fid,x,y,s,rot,opts) 
    figure(fid)
    
    th = -3*pi/4:pi/32:5*pi/4;
    xv = 0.5*s * cos(th) + x;
    yv = 0.5*s * sin(th) + y;
    
    xv = [xv,...
         0.5*(xv(end)+x),...
         xv(end),...
         x+abs(x-xv(end)),...
         0.5*(0.5*s*cos(pi/4)+2*x),...
         0.5*s*cos(pi/4)+x,...
         ];
     
    yv = [yv,...
          0.5*(yv(end)+y),...
          y,...
          y,...
          0.5*(0.5*s*sin(pi/4)+2*y),...
          0.5*s*sin(pi/4)+y,...
          ];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot-45);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [xrot(65),yrot(65)]; 
    coords.NW = [xrot(57),yrot(57)]; 
    coords.N  = [xrot(49),yrot(49)]; 
    coords.NE = [xrot(41),yrot(41)];
    coords.E  = [xrot(33),yrot(33)];
    coords.SE = [xrot(25),yrot(25)];
    coords.S  = [xrot(17),yrot(17)];
    coords.SW = [xrot( 9),yrot( 9)];
end

function [xrot,yrot] = rotate(xv,yv,x,y,rot) 
    R = [cosd(rot),-sind(rot); sind(rot),cosd(rot)];
    
    xrot = zeros(size(xv));
    yrot = zeros(size(yv));
    center = repmat([x;y],1,2);
    
    for i=1:length(xv)-1
       v = [xv(i),xv(i+1);yv(i),yv(i+1)];
       vo = R*(v - center) + center;
       xrot(i:i+1) = vo(1,:);
       yrot(i:i+1) = vo(2,:);
    end
end

function [] = connect(fid,from,from_anchor,to,to_anchor,varargin)
    figure(fid)
    
    if length(varargin)==1
        linespec = varargin{1};
        opts = {};
    elseif length(varargin)==2
        linespec = varargin{1};
        opts = varargin{2:end};
    else
        linespec = '-';
        opts = {};
    end
    
    xy0 = from.(from_anchor); x0 = xy0(1); y0 = xy0(2);
    xy1 = to.(to_anchor); x1 = xy1(1); y1 = xy1(2);
    
    if strcmp(linespec,'-') || isempty(linespec)
        plot([x0,x1],[y0,y1],opts{:});
    elseif strcmp(linespec,'|-')
        plot([x0,x0,x1],[y0,y1,y1],opts{:});
    elseif strcmp(linespec,'-|')
        plot([x0,x1,x1],[y0,y0,y1],opts{:});
    end
      
end