function [] = drawPFD_full(varargin)
    fid = figure('Name','Full LNG plant');
    clf(fid)
    set(fid, 'Position', [100, 100, 1200, 800]);
    
    scale = 0.4;
    hold on
    
    % Draw the units
    opts = {'color','black'};
    Comp_P = drawCompressor(fid,-2,2,scale,0,opts); % P compressor
    Cond_P = drawCondenser(fid,1,3,scale,0,opts); % P condenser
    Valv_P = drawValve2(fid,2,3,scale,0,opts); % P JT-valve
    HEX_PE = drawHEX(fid,3,1,scale,0,opts); % HEX P-E
    HEX_PM = drawHEX(fid,3,-1,scale,0,opts); % HEX P-M
    HEX_PL = drawHEX(fid,3,-3,scale,0,opts); % HEX P-LNG
    
    Comp_E = drawCompressor(fid,-1,0,scale,0,opts); % E compressor
    Valv_E = drawValve2(fid,4,1,scale,0,opts); % E JT-valve
    Cond_E = drawCondenser(fid,1,1,scale,0,opts); % E condenser
    HEX_EM = drawHEX(fid,5,-1,scale,0,opts); % HEX E-M
    HEX_EL = drawHEX(fid,5,-3,scale,0,opts); % HEX E-LNG
      
    Comp_M = drawCompressor(fid,0,-2,scale,0,opts);
    Valv_M = drawValve2(fid,6,-1,scale,0,opts);
    Cond_M = drawCondenser(fid,1,-1,scale,0,opts);
    HEX_ML = drawHEX(fid,7,-3,scale,0,opts);
    
    % Draw the connections
    opts            = {'Color','blue'};
    comp_shaft_opts = {'-.','Color',[1,0,1]};
    energy_opts     = {':','Color',[1,0,0]};
    
    % Propane loop
    connect(fid,Comp_P,'NE',Cond_P,'W' ,'|-',opts)
    connect(fid,Cond_P,'E' ,Valv_P,'W' ,'-' ,opts)
    connect(fid,Valv_P,'E' ,HEX_PE,'N' ,'-|',opts)
    connect(fid,HEX_PE,'S' ,HEX_PM,'N' ,'-' ,opts)
    connect(fid,HEX_PM,'S' ,HEX_PL,'N' ,'-' ,opts)
    tmp = struct('N',[0,-4]);
    connect(fid,HEX_PL,'S' ,tmp   ,'N' ,'|-',opts)
    connect(fid,tmp   ,'N' ,Comp_P,'SW','-|',opts)
    Comp_P_Shaft = struct();
    Comp_P_Shaft.N = Comp_P.N+[0, 0.5];
    Comp_P_Shaft.S = Comp_P.S+[0,-0.5];
    connect(fid,Comp_P,'N',Comp_P_Shaft,'N','-',comp_shaft_opts)
    connect(fid,Comp_P,'S',Comp_P_Shaft,'S','-',comp_shaft_opts)
    Cond_P_Pow = struct();
    Cond_P_Pow.N = Cond_P.NE+[0.5,0.5];
    connect(fid,Cond_P,'NE',Cond_P_Pow,'N','|-',energy_opts)
    
    % Ethane loop
    connect(fid,Comp_E,'NE',Cond_E,'W' ,'|-',opts)
    connect(fid,Cond_E,'E' ,HEX_PE,'W' ,'-' ,opts)
    connect(fid,HEX_PE,'E' ,Valv_E,'W' ,'-' ,opts)
    connect(fid,Valv_E,'E' ,HEX_EM,'N' ,'-|',opts)
    connect(fid,HEX_EM,'S' ,HEX_EL,'N' ,'-' ,opts)
    tmp2 = struct('N',[1,-5]);
    connect(fid,HEX_EL,'S' ,tmp2  ,'N' ,'|-',opts)
    connect(fid,tmp2  ,'N' ,Comp_E,'SW','-|',opts)
    Comp_E_Shaft = struct();
    Comp_E_Shaft.N = Comp_E.N+[0, 0.5];
    Comp_E_Shaft.S = Comp_E.S+[0,-0.5];
    connect(fid,Comp_E,'N',Comp_E_Shaft,'N','-',comp_shaft_opts)
    connect(fid,Comp_E,'S',Comp_E_Shaft,'S','-',comp_shaft_opts)
    Cond_E_Pow = struct();
    Cond_E_Pow.N = Cond_E.NE+[0.5,0.5];
    connect(fid,Cond_E,'NE',Cond_E_Pow,'N','|-',energy_opts)
     
    % Methane loop
    connect(fid,Comp_M,'NE',Cond_M,'W' ,'|-',opts)
    connect(fid,Cond_M,'E' ,HEX_PM,'W' ,'-' ,opts)
    connect(fid,HEX_PM,'E' ,HEX_EM,'W' ,'-' ,opts)
    connect(fid,HEX_EM,'E' ,Valv_M,'W' ,'-' ,opts)
    connect(fid,Valv_M,'E' ,HEX_ML,'N' ,'-|',opts)
    tmp3 = struct('N',[2,-6]);
    connect(fid,HEX_ML,'S' ,tmp3  ,'N' ,'|-',opts)
    connect(fid,tmp3  ,'N' ,Comp_M,'SW','-|',opts)
    Comp_M_Shaft = struct();
    Comp_M_Shaft.N = Comp_M.N+[0, 0.5];
    Comp_M_Shaft.S = Comp_M.S+[0,-0.5];
    connect(fid,Comp_M,'N',Comp_M_Shaft,'N','-',comp_shaft_opts)
    connect(fid,Comp_M,'S',Comp_M_Shaft,'S','-',comp_shaft_opts)
    Cond_M_Pow = struct();
    Cond_M_Pow.N = Cond_M.NE+[0.5,0.5];
    connect(fid,Cond_M,'NE',Cond_M_Pow,'N','|-',energy_opts)
    
    % LNG line
    tmp4 = struct('N',[2,-3]);
    connect(fid,tmp4  ,'N' ,HEX_PL,'W' ,'-' ,opts)
    connect(fid,HEX_PL,'E' ,HEX_EL,'W' ,'-' ,opts)
    connect(fid,HEX_EL,'E' ,HEX_ML,'W' ,'-' ,opts)
    tmp5 = struct('N',[8,-3]);
    connect(fid,HEX_ML,'E' ,tmp5  ,'N' ,'-' ,opts)
    
    hold off
    axis equal
    xlim(1.05*xlim);
    ylim(1.05*ylim);
    
    xl = xlim;
    yl = ylim;
    pos = @(x) [(x(1)-xl(1))/(xl(2)-xl(1)),(x(2)-yl(1))/(yl(2)-yl(1))];
    opts = {'Units','normalized','HorizontalAlignment','center','VerticalAlignment','middle','FontSize',8,'Interpreter','latex'};
    
    % Draw the text labels
    p = num2cell(pos(HEX_PE.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nP-E'),opts{:});
    p = num2cell(pos(HEX_PM.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nP-M'),opts{:});
    p = num2cell(pos(HEX_PL.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nP-LNG'),opts{:});
    p = num2cell(pos(HEX_EM.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nE-M'),opts{:});
    p = num2cell(pos(HEX_EL.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nE-LNG'),opts{:});
    p = num2cell(pos(HEX_ML.NE+[0.25,0.25]));
    text(p{:},sprintf('HEX\nM-LNG'),opts{:});
    p = num2cell(pos(Comp_P.SE+[0.60,0.0]));
    text(p{:},sprintf('Propane\ncompressor'),opts{:});
    p = num2cell(pos(Comp_E.SE+[0.60,0.0]));
    text(p{:},sprintf('Ethane\ncompressor'),opts{:});
    p = num2cell(pos(Comp_M.SE+[0.60,0.0]));
    text(p{:},sprintf('Methane\ncompressor'),opts{:});
    p = num2cell(pos(Cond_P.S+[0.5,-0.40]));
    text(p{:},sprintf('Propane\ncondenser'),opts{:});
    p = num2cell(pos(Cond_E.S+[0.5,-0.40]));
    text(p{:},sprintf('Ethane\ncondenser'),opts{:});
    p = num2cell(pos(Cond_M.S+[0.5,-0.40]));
    text(p{:},sprintf('Methane\ncondenser'),opts{:});
    p = num2cell(pos(Valv_P.NE+[0.25,0.25]));
    text(p{:},sprintf('JT-\nvalve'),opts{:});
    p = num2cell(pos(Valv_E.NE+[0.25,0.25]));
    text(p{:},sprintf('JT-\nvalve'),opts{:});
    p = num2cell(pos(Valv_M.NE+[0.25,0.25]));
    text(p{:},sprintf('JT-\nvalve'),opts{:});
    
    opts = {'HorizontalAlignment','center','VerticalAlignment','middle','FontSize',6,'Interpreter','latex','BackgroundColor',[0.95,1,0.95]};
    text(-2.2,-1.1,'101',opts{:});
    text(-0.55,3,'102',opts{:});
    text(1.5,3,'103',opts{:});
    text(3,2,'104',opts{:});
    text(3,0,'105',opts{:});
    text(3,-2,'106',opts{:});
    
    opts = {'HorizontalAlignment','center','VerticalAlignment','middle','FontSize',6,'Interpreter','latex','BackgroundColor',[1,0.95,0.95]};
    text(-1.2,-2.4,'201',opts{:});
    text(-0.05,1,'202',opts{:});
    text(2,1,'203',opts{:});
    text(3.5,1,'204',opts{:});
    text(5,0,'205',opts{:});
    text(5,-2,'206',opts{:});
    
    opts = {'HorizontalAlignment','center','VerticalAlignment','middle','FontSize',6,'Interpreter','latex','BackgroundColor',[0.95,0.95,1]};
    text(-0.2,-4.1,'301',opts{:});
    text(2,-1,'303',opts{:});
    text(4,-1,'304',opts{:});
    text(7,-2,'306',opts{:});
    
    opts = {'HorizontalAlignment','center','VerticalAlignment','middle','FontSize',6,'Interpreter','latex','BackgroundColor',[1,0.95,1]};
    text(2.4,-3,'401',opts{:});
    text(4,-3,'402',opts{:});
    text(6,-3,'403',opts{:});
    text(7.6,-3,'404',opts{:});
    
    if ~nargin==0
        x0 = varargin{1};
        
        opts = {'Units','normalized','HorizontalAlignment','center','VerticalAlignment','middle','FontSize',8,'EdgeColor','black','Interpreter','latex','backgroundcolor',[0.95,1.00,0.95]};
        
        p = pos([-4,4]);
        str = sprintf(['\\textbf{Stream 101:}\n',...
                       '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
                       '$$P=%0.2f$$ bar\n',...
                       '$$m=%0.2f$$\n',...
                       '$$H=%0.3f\\cdot10^5$$ kJ/h\n',...
                       '$$q_{suction}=%0.3f$$',...
                       ],full(x0.T101)*100-273.15 ...
                        ,full(x0.P101)*10 ...
                        ,full(x0.m101) ...
                        ,full(x0.H101) ...
                        ,full(x0.qsuction_P) ...
                        );
        text(p(1),p(2),str,opts{:});
        
        p = pos([-4,2.5]);
        str = sprintf(['\\textbf{Stream 102:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T102)*100-273.15 ...
                ,full(x0.P102)*10 ...
                ,full(x0.H102) ...
                );
        text(p(1),p(2),str,opts{:});

        p = pos([-4,1]);
        str = sprintf(['\\textbf{Stream 103:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T103)*100-273.15 ...
                ,full(x0.P103)*10 ...
                ,full(x0.H103) ...
                );
        text(p(1),p(2),str,opts{:});
        
        p = pos([-4,-0.5]);
        str = sprintf(['\\textbf{Stream 104:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T104)*100-273.15 ...
                ,full(x0.P104)*10 ...
                ,full(x0.H104) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        p = num2cell(pos(Comp_P.S+[0,-0.25]));
        str = sprintf(['\\textbf{Compressor:}\n',...
                '$$W_{comp}=%0.3f$$\n',...
                '$$P_{ratio}=%0.3f$$\n',...
                '$$h=%0.3f$$\n',...
                '$$\\eta=%0.3f$$',...
                '$$N=%0.3f$$',...
                ],full(x0.Wcomp_P) ...
                 ,full(x0.P102)/full(x0.P101) ...
                 ,full(x0.head_P) ...
                 ,full(x0.nu_P) ...
                 ,full(x0.N_P) ...
                 );
        text(p{:},str,opts{:});
        
        p = num2cell(pos(Cond_P.S+[0,-0.25]));
        str = sprintf('$$Q_{cond}=%0.3f$$',full(x0.Qcond_P));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_PE.S+[0,-0.25]));
        str = sprintf('$$Q_{PE}=%0.3f$$',full(x0.Qhex_PE));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_PM.S+[0,-0.25]));
        str = sprintf('$$Q_{PM}=%0.3f$$',full(x0.Qhex_PM));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_PL.S+[0,-0.25]));
        str = sprintf('$$Q_{PL}=%0.3f$$',full(x0.Qhex_PL));
        text(p{:},str,opts{:}); 
        
        opts = {'Units','normalized','HorizontalAlignment','center','VerticalAlignment','middle','FontSize',8,'EdgeColor','black','Interpreter','latex','backgroundcolor',[1.00,0.95,0.95]};
        
        p = pos([-4,-2.0]);
        str = sprintf(['\\textbf{Stream 201:}\n',...
                       '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
                       '$$P=%0.2f$$ bar\n',...
                       '$$m=%0.2f$$ \n',...
                       '$$H=%0.3f\\cdot10^5$$ kJ/h\n',...
                       '$$q_{suction}=%0.3f$$',...
                       ],full(x0.T201)*100-273.15 ...
                        ,full(x0.P201)*10 ...
                        ,full(x0.m201) ...
                        ,full(x0.H201) ...
                        ,full(x0.qsuction_E) ...
                        );
        text(p(1),p(2),str,opts{:});
        
        p = pos([-4,-3.5]);
        str = sprintf(['\\textbf{Stream 202:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T202)*100-273.15 ...
                ,full(x0.P202)*10 ...
                ,full(x0.H202) ...
                );
        text(p(1),p(2),str,opts{:});

        p = pos([-4,-5]);
        str = sprintf(['\\textbf{Stream 203:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T203)*100-273.15 ...
                ,full(x0.P203)*10 ...
                ,full(x0.H203) ...
                );
        text(p(1),p(2),str,opts{:});
        
        p = pos([-4,-6.5]);
        str = sprintf(['\\textbf{Stream 204:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T204)*100-273.15 ...
                ,full(x0.P204)*10 ...
                ,full(x0.H204) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        p = pos([9,4]);
        str = sprintf(['\\textbf{Stream 205:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T205)*100-273.15 ...
                ,full(x0.P205)*10 ...
                ,full(x0.H205) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        p = num2cell(pos(Comp_E.S+[0,-0.25]));
        str = sprintf(['\\textbf{Compressor:}\n',...
                '$$W_{comp}=%0.3f$$\n',...
                '$$P_{ratio}=%0.3f$$\n',...
                '$$h=%0.3f$$\n',...
                '$$\\eta=%0.3f$$',...
                '$$N=%0.3f$$',...
                ],full(x0.Wcomp_E) ...
                 ,full(x0.P202)/full(x0.P201) ...
                 ,full(x0.head_E) ...
                 ,full(x0.nu_E) ...
                 ,full(x0.N_E) ...
                 );
        text(p{:},str,opts{:});
        
        p = num2cell(pos(Cond_E.S+[0,-0.25]));
        str = sprintf('$$Q_{cond}=%0.3f$$',full(x0.Qcond_E));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_PE.S+[0,-0.60]));
        str = sprintf('$$Q_{EP}=%0.3f$$',full(x0.Qhex_EP));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_EM.S+[0,-0.25]));
        str = sprintf('$$Q_{EM}=%0.3f$$',full(x0.Qhex_EM));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_EL.S+[0,-0.25]));
        str = sprintf('$$Q_{EL}=%0.3f$$',full(x0.Qhex_EL));
        text(p{:},str,opts{:}); 
        
        opts = {'Units','normalized','HorizontalAlignment','center','VerticalAlignment','middle','FontSize',8,'EdgeColor','black','Interpreter','latex','backgroundcolor',[0.95,0.95,1.00]};
        
        p = pos([9,2.5]);
        str = sprintf(['\\textbf{Stream 301:}\n',...
                       '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
                       '$$P=%0.2f$$ bar\n',...
                       '$$m=%0.2f$$ \n',...
                       '$$H=%0.3f\\cdot10^5$$ kJ/h\n',...
                       '$$q_{suction}=%0.3f$$',...
                       ],full(x0.T301)*100-273.15 ...
                        ,full(x0.P301)*10 ...
                        ,full(x0.m301) ...
                        ,full(x0.H301) ...
                        ,full(x0.qsuction_M) ...
                        );
        text(p(1),p(2),str,opts{:});
        
        p = pos([9,1.0]);
        str = sprintf(['\\textbf{Stream 302:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T302)*100-273.15 ...
                ,full(x0.P302)*10 ...
                ,full(x0.H302) ...
                );
        text(p(1),p(2),str,opts{:});

        p = pos([9,-0.5]);
        str = sprintf(['\\textbf{Stream 303:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T303)*100-273.15 ...
                ,full(x0.P303)*10 ...
                ,full(x0.H303) ...
                );
        text(p(1),p(2),str,opts{:});
        
        p = pos([9,-2]);
        str = sprintf(['\\textbf{Stream 304:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T304)*100-273.15 ...
                ,full(x0.P304)*10 ...
                ,full(x0.H304) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        p = pos([9,-3.5]);
        str = sprintf(['\\textbf{Stream 305:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T305)*100-273.15 ...
                ,full(x0.P305)*10 ...
                ,full(x0.H305) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        p = pos([9,-5]);
        str = sprintf(['\\textbf{Stream 306:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=%0.2f$$ bar\n',...
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T306)*100-273.15 ...
                ,full(x0.P306)*10 ...
                ,full(x0.H306) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        %p = num2cell(pos(Comp_M.S+[0,-0.25]));
        %str = sprintf('$$W_{comp}=%0.3f$$\n$$P_{ratio}=%0.3f$$\n$$h=%0.3f$$',x0(31+n),x0(23+n)/x0(28+n),x0(115));
        %text(p{:},str,opts{:});
        
        p = num2cell(pos(Comp_M.S+[0,-0.25]));
        str = sprintf(['\\textbf{Compressor:}\n',...
                '$$W_{comp}=%0.3f$$\n',...
                '$$P_{ratio}=%0.3f$$\n',...
                '$$h=%0.3f$$\n',...
                '$$\\eta=%0.3f$$',...
                ],full(x0.Wcomp_M) ...
                 ,full(x0.P302)/full(x0.P301) ...
                 ,full(x0.head_M) ...
                 ,full(x0.nu_M) ...
                 );
        text(p{:},str,opts{:});
        
        p = num2cell(pos(Cond_M.S+[0,-0.25]));
        str = sprintf('$$Q_{cond}=%0.3f$$',full(x0.Qcond_M));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_PM.S+[0,-0.60]));
        str = sprintf('$$Q_{MP}=%0.3f$$',full(x0.Qhex_MP));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_EM.S+[0,-0.60]));
        str = sprintf('$$Q_{ME}=%0.3f$$',full(x0.Qhex_ME));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_ML.S+[0,-0.25]));
        str = sprintf('$$Q_{ML}=%0.3f$$',full(x0.Qhex_ML));
        text(p{:},str,opts{:}); 
        
        opts = {'Units','normalized','HorizontalAlignment','center','VerticalAlignment','middle','FontSize',8,'EdgeColor','black','Interpreter','latex','backgroundcolor',[1.00,0.95,1.00]};
        
        p = pos([-1.4,-5]);
        str = sprintf(['\\textbf{Stream 401:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$P=39.00$$ bar\n',... 
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T401)*100-273.15 ...
                ,full(x0.H401) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        p = pos([1.2,-5]);
        str = sprintf(['\\textbf{Stream 402:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T402)*100-273.15 ...
                ,full(x0.H402) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        p = pos([3.8,-5]);
        str = sprintf(['\\textbf{Stream 403:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... 
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T403)*100-273.15 ...
                ,full(x0.H403) ...
                );
        text(p(1),p(2),str,opts{:}); 

        p = pos([6.4,-5]);
        str = sprintf(['\\textbf{Stream 404:}\n',...
               '$$T=%0.1f$$ $$^{\\circ}$$C\n',... ..
               '$$H=%0.3f\\cdot10^5$$ kJ/h',...
               ],full(x0.T404)*100-273.15 ...
                ,full(x0.H404) ...
                );
        text(p(1),p(2),str,opts{:}); 
        
        p = num2cell(pos(HEX_PL.S+[0,-0.6]));
        str = sprintf('$$Q_{LP}=%0.3f$$',full(x0.Qhex_LP));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_EL.S+[0,-0.6]));
        str = sprintf('$$Q_{LE}=%0.3f$$',full(x0.Qhex_LE));
        text(p{:},str,opts{:}); 
        
        p = num2cell(pos(HEX_ML.S+[0,-0.6]));
        str = sprintf('$$Q_{LM}=%0.3f$$',full(x0.Qhex_LM));
        text(p{:},str,opts{:}); 
        
        str = sprintf('W_{tot}=%0.3f',full(x0.Wcomp_P)+full(x0.Wcomp_E)+full(x0.Wcomp_M));
        text(4,4,str); 
        
    end
    
end

%% Define the units

function coords = drawCompressor(fid,x,y,s,rot,opts) 
    figure(fid)
    
    xv = [x-0.25*s,x+0.25*s,x+0.5*s,x-0.5*s];
    yv = [y+0.5*s,y+0.5*s,y-0.5*s,y-0.5*s];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [(xrot(1)+xrot(4))*0.5,(yrot(1)+yrot(4))*0.5];
    coords.NW = [xrot(1),yrot(1)];
    coords.N  = [(xrot(1)+xrot(2))*0.5,(yrot(1)+yrot(2))*0.5];
    coords.NE = [xrot(2),yrot(2)];
    coords.E  = [(xrot(2)+xrot(3))*0.5,(yrot(2)+yrot(3))*0.5];
    coords.SE = [xrot(3),yrot(3)];
    coords.S  = [(xrot(3)+xrot(4))*0.5,(yrot(3)+yrot(4))*0.5];
    coords.SW = [xrot(4),yrot(4)];
end

function coords = drawValve(fid,x,y,s,rot,opts) 
    figure(fid)
    
    xv = [x-0.5*s,x-0.5*s,x+0.5*s,x+0.5*s];
    yv = [y-0.5*s,y+0.5*s,y-0.5*s,y+0.5*s];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [(xrot(1)+xrot(2))*0.5,(yrot(1)+yrot(2))*0.5];
    coords.NW = [xrot(1),yrot(1)];
    coords.N  = [(xrot(4)+xrot(5))*0.5,(yrot(4)+yrot(5))*0.5];
    coords.NE = [xrot(4),yrot(4)];
    coords.E  = [(xrot(3)+xrot(4))*0.5,(yrot(3)+yrot(4))*0.5];
    coords.SE = [xrot(3),yrot(3)];
    coords.S  = [(xrot(4)+xrot(5))*0.5,(yrot(4)+yrot(5))*0.5];
    coords.SW = [xrot(2),yrot(2)];
end

function coords = drawValve2(fid,x,y,s,rot,opts) 
    figure(fid)
    
    xv = [x-0.5*s,x-0.5*s,x+0.5*s,x+0.5*s,x,x,x+0.25*s,x+0.20*s,x+0.07*s,x-0.07*s,x-0.20*s,x-0.25*s,x,x];
    yv = [y-0.5*s,y+0.5*s,y-0.5*s,y+0.5*s,y,y+0.5*s,y+0.5*s,y+0.65*s,y+0.7*s,y+0.7*s,y+0.65*s,y+0.5*s,y+0.5*s,y];
    xv = [xv,xv(1)];
    yv = [yv,yv(1)];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [(xrot(1)+xrot(2))*0.5,(yrot(1)+yrot(2))*0.5];
    coords.NW = [xrot(1),yrot(1)];
    coords.N  = [(xrot(9)+xrot(10))*0.5,(yrot(9)+yrot(10))*0.5];
    coords.NE = [xrot(4),yrot(4)];
    coords.E  = [(xrot(3)+xrot(4))*0.5,(yrot(3)+yrot(4))*0.5];
    coords.SE = [xrot(3),yrot(3)];
    coords.S  = [xrot(end-1),yrot(end-1)];
    coords.SW = [xrot(2),yrot(2)];
end

function coords = drawCondenser(fid,x,y,s,rot,opts) 
    figure(fid)
    
    th = -3*pi/4:pi/32:5*pi/4;
    xv = 0.5*s * cos(th) + x;
    yv = 0.5*s * sin(th) + y;
    
    xv = [x-0.75*s,...
         xv,...
         0.5*(xv(end)+x),...
         xv(end),...
         x+abs(x-xv(end)),...
         0.5*(0.5*s*cos(pi/4)+2*x),...
         x+0.60*s,...
         x+0.52*s,...
         x+0.75*s,...
         x+0.68*s,...
         x+0.60*s ];
     
    yv = [y-0.75*s,...
          yv,...
          0.5*(yv(end)+y),...
          y,...
          y,...
          0.5*(0.5*s*sin(pi/4)+2*y),...
          y+0.60*s,...
          y+0.68*s,...
          y+0.75*s,...
          y+0.52*s,...
          y+0.60*s];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [xrot(58),yrot(58)]; 
    coords.NW = [xrot(50),yrot(50)]; 
    coords.N  = [xrot(42),yrot(42)]; 
    coords.NE = [xrot(end-2),yrot(end-2)];
    coords.E  = [xrot(26),yrot(26)];
    coords.SE = [xrot(18),yrot(18)];
    coords.S  = [xrot(10),yrot(10)];
    coords.SW = [xrot( 1),yrot( 1)];
end

function coords = drawHEX(fid,x,y,s,rot,opts) 
    figure(fid)
    
    th = -3*pi/4:pi/32:5*pi/4;
    xv = 0.5*s * cos(th) + x;
    yv = 0.5*s * sin(th) + y;
    
    xv = [xv,...
         0.5*(xv(end)+x),...
         xv(end),...
         x+abs(x-xv(end)),...
         0.5*(0.5*s*cos(pi/4)+2*x),...
         0.5*s*cos(pi/4)+x,...
         ];
     
    yv = [yv,...
          0.5*(yv(end)+y),...
          y,...
          y,...
          0.5*(0.5*s*sin(pi/4)+2*y),...
          0.5*s*sin(pi/4)+y,...
          ];
    
    [xrot,yrot] = rotate(xv,yv,x,y,rot-45);
    
    plot(xrot,yrot,opts{:})
    
    coords = struct();
    coords.W  = [xrot(65),yrot(65)]; 
    coords.NW = [xrot(57),yrot(57)]; 
    coords.N  = [xrot(49),yrot(49)]; 
    coords.NE = [xrot(41),yrot(41)];
    coords.E  = [xrot(33),yrot(33)];
    coords.SE = [xrot(25),yrot(25)];
    coords.S  = [xrot(17),yrot(17)];
    coords.SW = [xrot( 9),yrot( 9)];
end

function [xrot,yrot] = rotate(xv,yv,x,y,rot) 
    R = [cosd(rot),-sind(rot); sind(rot),cosd(rot)];
    
    xrot = zeros(size(xv));
    yrot = zeros(size(yv));
    center = repmat([x;y],1,2);
    
    for i=1:length(xv)-1
       v = [xv(i),xv(i+1);yv(i),yv(i+1)];
       vo = R*(v - center) + center;
       xrot(i:i+1) = vo(1,:);
       yrot(i:i+1) = vo(2,:);
    end
end

function [] = connect(fid,from,from_anchor,to,to_anchor,varargin)
    figure(fid)
    
    if length(varargin)==1
        linespec = varargin{1};
        opts = {};
    elseif length(varargin)==2
        linespec = varargin{1};
        opts = varargin{2:end};
    else
        linespec = '-';
        opts = {};
    end
    
    xy0 = from.(from_anchor); x0 = xy0(1); y0 = xy0(2);
    xy1 = to.(to_anchor); x1 = xy1(1); y1 = xy1(2);
    
    if strcmp(linespec,'-') || isempty(linespec)
        plot([x0,x1],[y0,y1],opts{:});
    elseif strcmp(linespec,'|-')
        plot([x0,x0,x1],[y0,y1,y1],opts{:});
    elseif strcmp(linespec,'-|')
        plot([x0,x1,x1],[y0,y0,y1],opts{:});
    end
      
end