function [Juu,Gy,F,y,fval] = getSensMats()

    %addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.3.0')
    import casadi.*
    addpath([pwd,'/functions'])

%% Calculates Juu

% Get nominal solution
[fval,y,rf,red_hess,cost] = LNG_model_full([],true);

inputs = {'N_P','N_E'};

nom_y = casadi_struct2vec(rmfield(y,inputs));
nom_u = [y.N_P,y.N_E];
nom_d = [y.m401,y.T401,y.Tambient,y.alpha1,y.alpha2,y.alpha3];

ny = length(nom_y);
nu = length(nom_u);
nd = length(nom_d);
ng = 129;
nJ = 1;
ny = 129;

new_flag = true;
if new_flag 
    rfJ = rf.jacobian;
    rfH = rf.jacobian.jacobian;

    J  = full(rfJ(nom_y,nom_u,nom_d,[],[],[])); 
    % Size: [n_eqs,n_vars]
    J  = mat2cell(J,[ng,nJ,ny],[ny,nu,nd]);
    
    Gy = J{3,2};                                                    % dx/du
    Gd = J{3,3};                                                    % dx/dd
    
    Juu = rf.hessian('u','J')
    
    %test = rf.factory('rf_adj',{'x','u','d','adj:y','adj:J'},{'adj:x','adj:u','adj:d'})
    
    n_eqs  = ng+nJ+ny;
    n_vars = ny+nu+nd;
    
    H  = full(rfH(nom_y,nom_u,nom_d,[],[],[],[])); 
    % Size: [n_eqs*n_vars,n_eqs+n_vars]
    H  = mat2cell(H,[ng*n_vars,nJ*n_vars,ny*n_vars],[n_vars,n_eqs]);
    
    Juu = zeros(nu,nu);
    Jud = zeros(nu,nd);
    
    A = H{2,1};
    for i = 1:nu
        for j = 1:nu
            Juu(i,j) =  A(ny*nu*(i-1)+1,ny+j)
        end
    end
    for i = 1:nu
        for j = 1:nd
            Jud(i,j) =  A(ny*nu*(i-1)+1,ny+nu+j)
        end
    end
    
    keyboard
    % ng*nx+ng*nu+ng*nd + nJ*nx+nJ*nu+nJ*nd + nx*nx+nx*nu+nx*nd


else
% Juu =
% 
%     5.7359    3.2668
%     3.2668   10.5066
%     
% Jud =
% 
%    -0.4755    0.0775   -2.4587    0.5225   -0.0625    0.0154
%    -2.0236   -0.1794   -1.7329    0.3145   -0.1025    0.1175    
end

Gy  = rf.jacobian('u','y');
Gd  = rf.jacobian('d','y');
Juu = rf.hessian('u','J');
Jud = jacobian(rf.jacobian('u','J'),'d');

Gy  = full(Gy(nom_y,nom_u,nom_d));
Gd  = full(Gd(nom_y,nom_u,nom_d));
Juu = full(Juu(nom_y,nom_u,nom_d));
Jud = full(Jud(nom_y,nom_u,nom_d));
F   = Gd-Gy/Juu*Jud;

cost = @(u) rf(nom_y,u,nom_d);

% Juu = zeros(2);
% nu = length(nom_u);
% for i = 1:nu
%     for j = 1:nu
%         d = 1E-3;
%         h      = zeros(1,nu);
%         h(i)   = full(nom_u(i)*d);
%         k      = zeros(1,nu);
%         k(j)   = full(nom_u(j)*d);
%         
%         [~,cost_ff] = cost(nom_u + h + k);
%         [~,cost_bb] = cost(nom_u - h - k);
%         [~,cost_bf] = cost(nom_u - h + k);
%         [~,cost_fb] = cost(nom_u + h - k);
%         
%         tmp = cost_ff + cost_bb - cost_bf - cost_fb;
%         
%         Juu(i,j) = full(tmp);
%         Juu(i,j) = Juu(i,j) / (4*h(i)*k(j));
%     end
% end
% Juu

%Juu = red_hess

%% Calculates sensitivity matrix F

% Disturbances: 1) LNG flowrate
%               2) inlet temperature LNG    
%               3) condenser penalty alpha
%               4) ambient temperature

% disturbances = [3.074,2.93,2.78,1,1,1];
% Delta        = diag(0.001*disturbances);
% 
% %F  = zeros(length(full(casadi_struct2vec(y))),length(disturbances));
% F  = zeros(length(full(casadi_struct2vec(nom_y))),length(disturbances));
% 
% for i = 1:length(disturbances)
%     
%     [~,yf] = LNG_model_full(disturbances+Delta(i,:));
%     [~,yb] = LNG_model_full(disturbances-Delta(i,:));
%     
%     tmpf = rmfield(yf,inputs);
%     tmpb = rmfield(yb,inputs);
%     
%     for j = 1:length(inputs)
%         tmpf.(inputs{j}) = yf.(inputs{j}); 
%         tmpb.(inputs{j}) = yb.(inputs{j}); 
%     end
%     
%     yf = full(casadi_struct2vec(rmfield(tmpf,inputs)));
%     yb = full(casadi_struct2vec(rmfield(tmpb,inputs)));
%     
%     F(:,i)  = ((yf-yb)/(2*sum(Delta(i,:))));
% end

% tmpy = rmfield(y,inputs);
% for j = 1:length(inputs)
%     tmpy.(inputs{j}) = y.(inputs{j}); 
% end
% y  = tmpy;

fields = fieldnames(y);
for i = 1:length(fields)
    y.(fields{i}) = full(y.(fields{i}));
end

[~,cost] = cost(nom_u);
cost = full(cost);
save('sensitivity_matrices.mat','y','F','Juu','Gy','cost','Gd','Jud')

end