function [Juu,Gy,F,y,fval] = getSensMats()

    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    addpath([pwd,'/functions'])

%% Calculates Juu

% Get nominal solution
[fval,y,rf] = LNG_model_full([],true);

%nom = [y.X_P,y.X_E,y.X_M,y.N_P,y.N_E,y.N_M];
%nom = [y.T103,y.T203,y.T303,y.N_P,y.N_E,y.N_M];
nom = [y.T103,y.T203,y.T303,y.N_P,y.N_E];

% Create steps for the inputs
H = diag(nom)*0.001;
%rf = LNG_model_full_eval([]);
inputs = {'T103','T203','T303','N_P','N_E'};
z0 = [casadi_struct2vec(rmfield(y,inputs));0];

Juu = zeros(length(nom) ,length(nom));
Gy  = zeros(length(z0)-1+length(nom),length(nom));

for i = 1:length(nom)
    for j = i:length(nom)
        ff = rf(z0,nom + H(i,:) + H(j,:));
        bb = rf(z0,nom - H(i,:) - H(j,:));
        fi = rf(z0,nom + H(i,:));
        bi = rf(z0,nom - H(i,:));
        fj = rf(z0,nom + H(j,:));
        bj = rf(z0,nom - H(j,:));
        
        ff = ff(end);
        bb = bb(end);
        yf = fi(1:end-1); fi = fi(end); 
        yb = bi(1:end-1); bi = bi(end);
        fj = fj(end);
        bj = bj(end);
        
        % Calculate second derivative of cost with respect to inputs
        Juu(i,j) = full(ff-fi-bi+2*fval-fj-bj+bb)/(2*sum(full(H(i,:)))*sum(full(H(j,:)))');
        Juu(j,i) = Juu(i,j);
    end
    
    % Calculate Gy = dy/du
    e = eye(length(nom));
    Gy(:,i) = [0.5*full(yf-yb)/(sum(full(H(i,:)))) ; e(:,i)];
    
end

%% Calculates sensitivity matrix F

% Disturbances: 1) LNG flowrate
%               2) inlet temperature LNG    
%               3) condenser temperature penalty P 
%               4) condenser temperature penalty E 
%               5) condenser temperature penalty M
%               6) efficiency offset P compressor
%               7) efficiency offset E compressor
%               8) efficiency offset M compressor

disturbances = [3.074,3.03,7,1,1,-0.074,0.074,-0.074];
Delta = diag([0.1,0.005,0.07,0.01,0.01,-0.0005,-0.0005,-0.0005]);

F  = zeros(length(full(casadi_struct2vec(y))),length(disturbances));

for i = 1:length(disturbances)
    
    [~,yf] = LNG_model_full(disturbances+Delta(i,:));
    [~,yb] = LNG_model_full(disturbances-Delta(i,:));
    
    tmpf = rmfield(yf,inputs);
    tmpb = rmfield(yb,inputs);
    
    for j = 1:length(inputs)
        tmpf.(inputs{j}) = yf.(inputs{j}); 
        tmpb.(inputs{j}) = yb.(inputs{j}); 
    end
    
    yf = full(casadi_struct2vec(tmpf));
    yb = full(casadi_struct2vec(tmpb));
    
    F(:,i)  = ((yf-yb)/(2*sum(Delta(i,:))));
end

tmpy = rmfield(y,inputs);
for j = 1:length(inputs)
    tmpy.(inputs{j}) = y.(inputs{j}); 
end
y  = tmpy;

fields = fieldnames(y);
for i = 1:length(fields)
    y.(fields{i}) = full(y.(fields{i}));
end

save('sensitivity_matrices.mat','y','F','Juu','Gy')

end