function res = compressibilityFactor(x,T,P,Z)
    
    %   COMPRESSIBILITY FACTOR
    %
    %   Calculated using Dranchuk and Abou-Kassem EOS
    %   P.M. Dranchuk and H. Abou-Kassem - " Calculation of Z Factors For 
    %   Natural Gases Using Equations of State" (1975)
    

    % Critical temperatures [K]
    criticalTemperatures =[...
        1.90, ... % C1
        3.05, ... % C2
        3.70, ... % C3
        4.25, ... % n-C4
        4.69, ... % n-C5
        5.07, ... % n-C6
        6.47, ... % H2O
        3.04, ... % CO2
        1.26, ... % N2
        4.08, ... % i-C4
        4.60, ... % i-C5
        ];
    
    pseudoCriticalTemperature = criticalTemperatures*x;

    % Pseudo-reduced temperature [-]
    pseudoReducedTemperature = T/pseudoCriticalTemperature;

    % Critical pressures [MPa]
    criticalPressures =[...
        4.60551724137931, ... % C1
        4.88137931034483, ... % C2
        4.25034482758621, ... % C3
        3.80000000000000, ... % n-C4
        3.37241379310345, ... % n-C5
        3.01379310344828, ... % n-C6
        22.0620689655172, ... % H2O
        7.38344827586207, ... % CO2
        3.39000000000000, ... % N2
        3.64896551724138, ... % i-C4
        3.39000000000000, ... % i-C5
        ];

    % Pseudo-critical mixture pressure [MPa]
    pseudoCriticalPressure = criticalPressures*x;

    % Pseudo reduced pressure [-]
    pseudoReducedPressure = P/pseudoCriticalPressure;

    A1 = 0.3265;
    A2 = -1.0700;
    A3 = -0.5339;
    A4 = 0.01569;
    A5 = -0.05165;
    A6 = 0.5475;
    A7 = -0.7361;
    A8 = 0.1844;
    A9 = 0.1056;
    A10 = 0.6134;
    A11 = 0.7210;
    
    Tpr = pseudoReducedTemperature;
    Ppr = pseudoReducedPressure;
    
    tmp = (0.27*Ppr/(Z*Tpr));
    
    res = 1+(A1+A2/Tpr+A3/Tpr^3+A4/Tpr^4+A5/Tpr^5)*tmp + ...
          (A6+A7/Tpr+A8/Tpr^2)*tmp^2 - ...
          (A7/Tpr+A8/Tpr^2)*A9*tmp^5 + ...
           A10*(1+A11*tmp^2)*(tmp^2/Tpr^3)*exp(-A11*tmp^2);