function [H,c,subset,Losses,names,cost] = ...
	optimal_measurements2(n,s,l,Juu,Gy,F,y_opt)

    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    %addpath([pwd,'/functions'])

% Function to calculate H using exact local method

if ~exist('n','var')
    n = 2;
end
if ~exist('s','var')
    s = true;
end
if ~exist('l','var')
    l = true;
end

if ~l && nargin < 7
  error('Juu, Gy, F and y_opt must be provided when l = 0')
elseif l
  load('sensitivity_matrices.mat')
else 
    [Juu,Gy,F,y,~] = getSensMats();
end
y_opt = full(casadi_struct2vec(y));

% Choose temperature, pressure and flow measurements, and u
fields = fieldnames(y);

fields2 = {'T101','T102','T103','T104',...
           'P101','P102','P103','P104',...
           'T201','T202','T203','T204','T205',...
           'P201','P202','P203','P204','P205',...
           'T301','T302','T303','T304','T305','T306',...
           'P301','P302','P303','P304','P305','P306',...
           'T402','T403','T404',...
           'm101','m201','m301',...
           'Wcomp_P','Wcomp_E','Wcomp_M',...
           'T401','m401',...
           'alpha1','alpha2','alpha3','Tambient',...
           'mcond_P','mcond_E','mcond_M',...
           'Tcond_P','Tcond_E','Tcond_M',...
          };      

%fields2 = {'T402','T403'};      
      
subset = zeros(size(fields2));
for i = 1:length(fields2)
    subset(i) = strmatch(fields2{i},fields);
end

y_opt = y_opt(subset);
F  = F(subset,:);
Gy = Gy(subset,:);
Gd = Gd(subset,:);
Wd = diag([0.250,0.02,0.02,0.10,0.10,0.10]);
       
ys = y;
ys.alpha1 = 1;
ys.alpha2 = 1;
ys.alpha3 = 1;
ys.T101 = 1; ys.T102 = 1; ys.T103 = 1; ys.T104 = 1;
ys.T201 = 1; ys.T202 = 1; ys.T203 = 1; ys.T204 = 1; ys.T205 = 1;
ys.T301 = 1; ys.T302 = 1; ys.T303 = 1; ys.T304 = 1; ys.T305 = 1; ys.T306 = 1;
ys.T401 = 1; ys.T402 = 1; ys.T403 = 1; ys.T404 = 1; ys.Tambient = 1;
ys.P101 = 0.2; ys.P102 = 1; ys.P103 = 1; ys.P104 = 0.2;
ys.P201 = 0.1; ys.P202 = 1; ys.P203 = 1; ys.P204 = 1; ys.P204 = 0.1;
ys.P301 = 0.5; ys.P302 = 2; ys.P303 = 2; ys.P304 = 2; ys.P305 = 2; ys.P306 = 0.5;
ys = full(casadi_struct2vec(ys));
ys = ys(subset);
Wn = diag(ys*0.01);

% Call  branch and bound  algorithm  to  find  subset  with 
% minimal average loss.
ns = 10;
addpath([pwd '/b3av'])
[B,sset,ops,ctime,flag] = pb3av(Gy,Gd,Wd,Wn,Juu,Jud,n,inf,ns);

if abs(B(1))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(2))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(3))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(4))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(5))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(6))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(7))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(8))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(9))>1
    ns=ns-1;
    sset = sset(2:end,:);
end
if abs(B(10))>1
    keyboard
end

Losses = zeros(1,ns);
for i = 1:ns
    
    sset_i = sset(i,:);
      
    %subset = subset(sset);
    names = fields2(sset_i)

    y_opt_i = y_opt(sset_i);
    Wn_   = Wn(sset_i,sset_i);
    Gy_   = Gy(sset_i,:);
    F_    = F(sset_i,:);
    Y_    = [F_*Wd Wn_];

    H = (Gy_'/(Y_*Y_'));
    %H = H/max(max(H));
    H = diag(ones(2,1));
    c = H*y_opt_i;

    H
    c

    RGA = (H*Gy_).*pinv(H*Gy_)'
    
    eigs = eig(H*Gy_)

    %c2 = H(1,:)*diag(y_opt)

    % Calculating the average loss (formulae in pb3av is wrong)
    X = (Juu^(1/2)/(H*Gy_))*(H*Y_);
    Losses(i) = 1/2*norm(X,'fro')^2

end

if s
    save('H_matrices.mat','H','c','subset') 
end

end