function [H,c,subset,Loss,names] = ...
	optimal_measurements(n,s,l,Juu,Gy,F,y_opt)

    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    %addpath([pwd,'/functions'])

% Function to calculate H using exact local method

if ~exist('n','var')
    n = 10;
end
if ~exist('s','var')
    s = true;
end
if ~exist('l','var')
    l = true;
end

if ~l && nargin < 7
  error('Juu, Gy, F and y_opt must be provided when l = 0')
elseif l
  load('sensitivity_matrices.mat')
else 
    [Juu,Gy,F,y,~] = getSensMats();
end

y_opt = full(casadi_struct2vec(y));

% Choose temperature, pressure and flow measurements, and u
fields = fieldnames(y);
fields2 = {'T101','T102','T103','T104','T201','T202','T203','T204','T205',...
           'T301','T302','T303','T304','T304','T305','T401','T402','T403',...
           'T404','m101','m201','m301','m401',...
           'P101','P102','P103','P104','P201','P202','P203','P204','P205',...
           'P301','P302','P303','P304','P304','P305',...
           'alpha_P','alpha_E','alpha_M','comp_deg_P','comp_deg_E','comp_deg_M',...
           'Wcomp_P','Wcomp_E','Wcomp_M',...
           'Qcond_P','Qcond_E','Qcond_M',...
          };
subset = zeros(size(fields2));
for i = 1:length(fields2)
    subset(i) = strmatch(fields2{i},fields);
end

y_opt = y_opt(subset);
F  = F(subset,:);
Gy = Gy(subset,:);

%Wd = diag([0.2,0.01,0.7,0.1,0.1,0.001,0.001,0.001]);    
Wd = diag([0.5,0.02,1,0.5,0.5,0.002,0.002,0.002]);  

%Wn2 = diag([0.01,0.01,1,1,1,0.1,1,1,1,0.1,0.1,0.1,...
%            1,1,1,0.1,0.1,0.1,0.1,0.1,0.1,1]);

Wn = diag(y_opt*0.01);

% Call  branch and bound  algorithm  to  find  subset  with 
% minimal average loss.
addpath([pwd '/b3av'])
[~,sset] = pb3av(Gy,F,Wd,Wn,Juu,n,inf,1);

subset = subset(sset);
names = fields2(sset)

y_opt = y_opt(sset);
Wn_   = Wn(sset,sset);
Gy_   = Gy(sset,:);
F_    = F(sset,:);
Y_    = [F_*Wd Wn_];

H = (Gy_'/(Y_*Y_'));
c = H*y_opt;
%c2 = H(1,:)*diag(y_opt)

% Calculating the average loss (formulae in pb3av is wrong)
X = Juu^(1/2)/(H*Gy_)*(H*Y_);
Loss = 1/2*norm(X,'fro')^2;

% Save H,c and subset for easy access 
if s
    save('H_matrices.mat','H','c','subset') 
end

end