function [H,c,subset,Losses,names] = ...
	optimal_measurements_no_disturbances(n,s,l,Juu,Gy,F,y_opt)

    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*
    %addpath([pwd,'/functions'])

% Function to calculate H using exact local method

if ~exist('n','var')
    n = 9;
end
if ~exist('s','var')
    s = true;
end
if ~exist('l','var')
    l = true;
end

if ~l && nargin < 7
  error('Juu, Gy, F and y_opt must be provided when l = 0')
elseif l
  load('sensitivity_matrices.mat')
else 
    [Juu,Gy,F,y,~] = getSensMats();
end

y_opt = full(casadi_struct2vec(y));

% Choose temperature, pressure and flow measurements, and u
fields = fieldnames(y);
fields2 = {'T101','T102','T103','T104',...
           'P101','P102','P103','P104',...
           'T201','T202','T203','T204','T205',...
           'P201','P202','P203','P204','P205',...
           'T301','T302','T303','T304','T305','T306',...
           'P301','P302','P303','P304','P305','P306',...
           'T402','T403','T404',...
           'm101','m201','m301',...
           'Wcomp_P','Wcomp_E','Wcomp_M',...
           ...'Qcond_P','Qcond_E','Qcond_M',...
           ...'T401','m401','alpha','Tambient',... 
           'mcond_P','mcond_E','mcond_M',...
           'Tcond_P','Tcond_E','Tcond_M',...
          };
subset = zeros(size(fields2));
for i = 1:length(fields2)
    subset(i) = strmatch(fields2{i},fields);
end

y_opt = y_opt(subset);
F  = F(subset,:);
Gy = Gy(subset,:);
Wd = diag([0.150,0.010,0.005,0.1,0.1,0.1]);

ys = y;
ys.alpha1 = 0;
ys.alpha2 = 0;
ys.alpha3 = 0;
ys.T101 = 1; ys.T102 = 1; ys.T103 = 1; ys.T104 = 1;
ys.T201 = 1; ys.T202 = 1; ys.T203 = 1; ys.T204 = 1; ys.T205 = 1;
ys.T301 = 1; ys.T302 = 1; ys.T303 = 1; ys.T304 = 1; ys.T305 = 1; ys.T306 = 1;
ys.T401 = 1; ys.T402 = 1; ys.T403 = 1; ys.T404 = 1;
ys.Tambient = 1;
ys = full(casadi_struct2vec(ys));
ys = ys(subset);
Wn = diag(ys*0.01);

% Call  branch and bound  algorithm  to  find  subset  with 
% minimal average loss.
ns = 10;
addpath([pwd '/b3av'])
[~,sset] = pb3av(Gy,F,Wd,Wn,Juu,n,inf,ns);

Losses = zeros(1,ns);
for i = 1:ns
    
    sset_i = sset(i,:);
    
    %subset = subset(sset_i);
    names = fields2(sset_i);

    y_opt_i = y_opt(sset_i);
    Wn_   = Wn(sset_i,sset_i);
    Gy_   = Gy(sset_i,:);
    F_    = F(sset_i,:);
    Y_    = [F_*Wd Wn_];

    H = (Gy_'/(Y_*Y_'));
    c = H*y_opt_i;

    H;
    c;

    % Calculating the average loss (formulae in pb3av is wrong)
    X = Juu^(1/2)/(H*Gy_)*(H*Y_);
    Losses(i) = 1/2*norm(X,'fro')^2;

end
    
% Save H,c and subset for easy access 
if s
    save('H_matrices.mat','H','c','subset') 
end

end