function eq = mix_streams(in,out,fracs)
    
    % 

    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*  

    if length(out)~=length(fracs)
        error('Number of splits is not equal to number of outlet streams')
    end
    if sum([fracs{:}])~=1;
        error('Sum of split fractions is not = 1')
    end
    
    mg = 0;
    mc = 0;
    T = 0;
    P = inf;
    x = DM(zeros([1,11]))';
    Vg = 0;
    Vc = 0;
    
    for i = 1:length(in)
        % Mass balance
        mg = mg + in{i}.z.mgdot;
        mc = mc + in{i}.z.mcdot; 
        % Energy balance
        T = T + (in{i}.z.mgdot+in{i}.z.mcdot)*...
                 in{i}.z.Cp*(in{i}.z.T*1E2);
        % Pressure
        P = min(P,in{i}.z.P); 
        % Component mass balances
        x = x + (in{i}.z.x*1E-2)*(in{i}.z.mgdot+in{i}.z.mcdot);
        % Volume
        Vg = Vg + in{i}.z.Vgdot;
        Vc = Vc + in{i}.z.Vcdot;
    end
    m = mg+mc;
    
    mgout = num2cell(zeros([1,length(out)]));
    mcout = num2cell(zeros([1,length(out)]));
    Tout = num2cell(zeros([1,length(out)]));
    Pout = num2cell(zeros([1,length(out)]));
    xout = num2cell(zeros([1,length(out)]));
    
    for i = 1:length(out);
        
        % Mass balance
        mgout{i} = mg*fracs{i} - out{i}.z.mgdot;
        mcout{i} = mc*fracs{i} - out{i}.z.mcdot;
        
        % Energy balance
        Tout{i} = T/(m*out{i}.z.Cp) - (out{i}.z.T*1E2);
        
        % Pressure
        Pout{i} = out{i}.z.P - P;
        
        % Component mass balances
        xout{i} = x/m - (out{i}.z.x*1E-2);
        
    end

    outs.mg = mgout;
    outs.mc = mcout;
    outs.T = Tout;
    outs.P = Pout;
    outs.x = xout;

    eq = casadi_struct2vec(outs);

end