function [separator,outletg,outletl] = separator(namein,nameoutg,nameoutl,inlet)

    addpath('C:\Users\verheyle\Downloads\casadi-matlabR2014b-v3.0.0')
    import casadi.*  
    
    nm = num2str(namein);
    
    outletg = def_stream(nameoutg,inlet.p.x,false,[],[],[],[]);
    outletl = def_stream(nameoutl,inlet.p.x,false,[],[],[],[]);
    
    K     = MX.sym(['K_',nm]);         % K-value (NORSOK recommends K<0.15)
    alpha = MX.sym(['alpha_',nm]);                  % Separation efficiency
    
    z       = struct();
    z.K     = K;
    z.alpha = alpha;
    
    D = 2;                                          % Inner vessel diameter
    A = pi*(D/2)^2;                                  % Cross-sectional area
    rho_l = 10;                                            % Liquid density
    rho_g = inlet.z.rho;                                      % Gas density
    Vgdot = (inlet.z.Vgdot);                     % Gas volumetric flow rate

    algs       = struct();
    algs.K     = -K + Vgdot/A*sqrt(rho_g/(rho_l-rho_g));
    algs.alpha = -alpha + 1-(rho_g/5)^2-7*(rho_g)^2*K^3;
                                                      
    algs.gas    = struct();
    algs.gas.mg = outletg.z.mgdot - inlet.z.mgdot*alpha;
    algs.gas.mc = outletg.z.mcdot - inlet.z.mcdot*(1-alpha);
    algs.gas.T  = outletg.z.T     - inlet.z.T;
    algs.gas.P  = outletg.z.P     - inlet.z.P;
    
    algs.liq    = struct();
    algs.liq.mg = outletl.z.mgdot - inlet.z.mcdot*(1-alpha);
    algs.liq.mc = outletl.z.mcdot - inlet.z.mcdot*alpha;
    algs.liq.T  = outletl.z.T     - inlet.z.T;
    algs.liq.P  = outletl.z.P     - inlet.z.P;

	separator         = struct();
    separator.z       = z;
    separator.algs    = algs;

    %% Initialize
    test = [casadi_struct2vec(inlet.z)-inlet.z0];
    rf_function = Function('rf_function',...
        {vertcat(casadi_struct2vec(inlet.z),...
                 casadi_struct2vec(outletg.z),...
                 casadi_struct2vec(outletl.z),K,alpha)},...
        {vertcat(casadi_struct2vec(outletg.algs),...
                 casadi_struct2vec(outletl.algs),...
                 casadi_struct2vec(algs),test)});


    rf = rootfinder('rf','kinsol',rf_function);
    
    a=0.95;
    z0in = inlet.z0;
    z0outg = z0in; 
    z0outg(3) = z0outg(3)*a;
    z0outg(4) = z0outg(4)*(1-a);
    z0outg(8) = z0outg(8)*a;
    z0outg(9) = z0outg(9)*(1-a);
    
    z0outl = z0in; 
    z0outl(4) = z0outl(4)*a;
    z0outl(3) = z0outl(3)*(1-a);
    z0outl(9) = z0outl(9)*a;
    z0outl(8) = z0outl(8)*(1-a);
    
    z0 = full(rf(vertcat(z0in,z0outg,z0outl,0.17,0.95)));
    outletg.z0 = z0(1*length(inlet.z0)+1:2*length(inlet.z0));
    outletl.z0 = z0(2*length(inlet.z0)+1:3*length(inlet.z0));
    separator.z0 = z0(end-1:end);
end