function Cp = heatCapacity(x,T)

R = 8.31446;                           % universal gas constant [J/(K.mol)]

%   HEAT CAPACITY J/mol-K
%
%   Coefficients  from: 
%   "http://www.personal.utulsa.edu/~geoffrey-price/Courses/ChE7023/HeatCapacity-HeatOfFormation.pdf
%
%   IDEAL GAS ASSUMPTION

heatCapacityCoeffs =[...
    1.702,  9.081E-3,  -2.164E-6,       0; ... % C1
    1.131, 19.225E-3,  -5.561E-6,       0; ... % C2
    1.213, 28.785E-3,  -8.824E-6,       0; ... % C3
    1.935, 36.915E-3, -11.402E-6,       0; ... % n-C4
    2.464, 45.351E-3, -14.111E-6,       0; ... % n-C5
    3.025, 53.722E-3, -16.791E-6,       0; ... % n-C6
    3.470,  1.450E-3,          0, 0.121E5; ... % H2O
    5.457,  1.045E-3,          0,-1.157E5; ... % CO2
    3.280,  0.593E-3,          0, 0.040E5; ... % N2
    1.677, 37.853E-3, -11.945E-6,       0; ... % i-C4
    2.464, 45.351E-3, -14.111E-6,       0; ... % i-C5
    ]';

Cp = (  (heatCapacityCoeffs(1,:)*x)              +  ...
        (heatCapacityCoeffs(2,:)*x)*(T*1E2)      +  ...
        (heatCapacityCoeffs(3,:)*x)*(T*1E2)^( 2) +  ...
        (heatCapacityCoeffs(4,:)*x)*(T*1E2)^(-2)    ...
     )*R;